Package: M3Drop
Version: 1.39.0
Date: 2024-03-25
Title: Michaelis-Menten Modelling of Dropouts in single-cell RNASeq
Author: Tallulah Andrews <tallulandrews@gmail.com>
Maintainer: Tallulah Andrews <tallulandrews@gmail.com>
Depends: R (>= 3.4), numDeriv
Imports: RColorBrewer, gplots, bbmle, statmod, grDevices, graphics,
        stats, matrixStats, Matrix, irlba, reldist, Hmisc, methods,
        scater
Suggests: ROCR, knitr, M3DExampleData, SingleCellExperiment, Seurat,
        Biobase
VignetteBuilder: knitr
biocViews: RNASeq, Sequencing, Transcriptomics, GeneExpression,
        Software, DifferentialExpression, DimensionReduction,
        FeatureExtraction
Collate: basics.R Plotting_fxns.R Curve_fitting.R Extremes.R
        Normalization.R Brennecke_implementation.R
        Threeway_ProportionalArea_VennDiagrams.R
        Simulations_Functions.R Traditional_DE.R NB_UMI.R
        M3D_Imputation.R Other_FS_functions.R DANB_HVG.R
        DANB_Coexpression.R
Description: This package fits a model to the pattern of dropouts in
        single-cell RNASeq data. This model is used as a null to
        identify significantly variable (i.e. differentially expressed)
        genes for use in downstream analysis, such as clustering cells.
        Also includes an method for calculating exact Pearson residuals
        in UMI-tagged data using a library-size aware negative binomial
        model.
URL: https://github.com/tallulandrews/M3Drop
BugReports: https://github.com/tallulandrews/M3Drop/issues
License: GPL (>=2)
Config/pak/sysreqs: libcairo2-dev cmake libfontconfig1-dev
        libfreetype6-dev libfribidi-dev make libharfbuzz-dev libicu-dev
        libjpeg-dev libpng-dev libtiff-dev libuv1-dev libwebp-dev
        zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:44:35 UTC
RemoteUrl: https://github.com/bioc/M3Drop
RemoteRef: HEAD
RemoteSha: 987a662dbbe2c91ea3e79db1ca43457b46c15de6
NeedsCompilation: no
Packaged: 2026-05-30 09:37:51 UTC; root
Built: R 4.6.0; ; 2026-05-30 13:37:49 UTC; unix

exportPattern("M3Drop*")
exportPattern("NBumi*")
export(corFS)
export(giniFS)
export(irlbaPcaFS)
export(Consensus_FS)
export(BrenneckeGetVariableGenes)
export(PoissonUMIFeatureSelectionDropouts)
import(RColorBrewer)
import(gplots)
import(bbmle)
import(statmod)
import(irlba)
import(Matrix)
import(numDeriv)
importFrom("grDevices", "col2rgb", "colorRampPalette", "densCols")
importFrom("Hmisc", "rcorr")
importFrom("graphics", "abline", "axis", "box", "curve", "frame", "hist", "legend", "lines", "mtext", "par", "points", "text", "title", "polygon")
importFrom("stats", "aggregate", "dist", "dnorm", "fitted", "glm", "hclust", "lm", "median", "p.adjust", "pchisq", "pnorm", "quantile", "rnorm", "sd", "uniroot", "var", "wilcox.test", "cor", "dnbinom", "optim", "runif", "rbinom", "prcomp", "density", "rgamma", "aov", "loess", "rnbinom", "qnbinom", "pnbinom", "qnorm")
importFrom("reldist", "gini")
importFrom("matrixStats", "rowSds")
importFrom("matrixStats", "rowVars")
importFrom("matrixStats", "colVars")
importFrom("methods", "as")

# M3Drop - Michaelis-Menten Modelling of Dropouts for scRNASeq

This is an R package providing functions for fitting a modified Michaelis-Menten (MM) equation to the pattern of dropouts observed in a single-cell sequencing experiment. As well as the Depth-Adjusted Negative Binomial (DANB) model which is tailored for datasets quantified using unique molecular identifiers (UMIs).

Functions are provided for fitting each model as well as performing dropout-based feature selection. These can be used to reduce technical noise in downstream analyses such as clustering or pseudotime analysis.

Update 2023-02-16 :
New functions: NBumiPearsonResiduals and NBumiPearsonResidualsApprox enable the calculation of pearson residuals using the depth-adjusted negative binomial model. Pearson residuals have recently been suggested as an alternative normalization strategy for single-cell RNAseq data see: https://genomebiology.biomedcentral.com/articles/10.1186/s13059-021-02451-7#S

For comparison, the algorithm presented in Brennecke et al (2015) for detection of significantly highly variable genes is included.

## Installation :
```r
require("remotes")
install_github('tallulandrews/M3Drop')
```
OR
```r
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("M3Drop")
```
Example Data:
```r
require("remotes")
install_github('tallulandrews/M3DExampleData')
```
OR

```r
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("M3DExapleData")
```
Read More: DOI: 10.1101/065094

## Citation:
Amdrews, TS and Hemberg, M. (2018) M3Drop:dropout-based feature selection for scRNASeq. Bioinformatics, bty1044. https://doi.org/10.1093/bioinformatics/bty1044

NBumiCoexpression <- function(counts, fit, gene_list=NULL, method=c("both", "on", "off")) {
	# Set up
	if (is.null(gene_list)) {
		gene_list <- names(fit$vals$tjs)
	}
	pd_gene <- matrix(-1, nrow=length(gene_list), ncol=ncol(counts));
	name_gene <- rep("", length(gene_list))
	for (i in 1:length(gene_list)) {
		gid <- which(names(fit$vals$tjs) == gene_list[i])
		if (length(gid) == 0) {next;}
		mu_is <- fit$vals$tjs[gid]*fit$vals$tis/fit$vals$total
                p_is <- (1+mu_is/fit$sizes[gid])^(-fit$sizes[gid]);
                pd_gene[i,] <- p_is;
                name_gene[i] <- gene_list[i];

	}
	if (sum(name_gene == "") > 0) {
		warning(paste("Warning:", sum(name_gene == ""), "genes not found, check your gene list is correct."));
		exclude <- which(name_gene == "");
		pd_gene <- pd_gene[-exclude,]
		name_gene <- name_gene[-exclude]
	}
	rownames(pd_gene) <- name_gene
	lib.size <- fit$vals$tis;

	Z_mat <- matrix(-1, ncol=length(pd_gene), nrow=length(pd_gene));
	for(i in 1:nrow(pd_gene)) {
		for (j in (i):nrow(pd_gene)) {
			p_g1 <- pd_gene[i,];
			p_g2 <- pd_gene[j,];	
			expr_g1 <- counts[rownames(counts)==rownames(pd_gene)[i],]
			expr_g2 <- counts[rownames(counts)==rownames(pd_gene)[j],]
	
			if (method == "off" | method=="both") {
				# Both zero 
				expect_both_zero <- p_g1*p_g2
				expect_both_err <- expect_both_zero*(1-expect_both_zero)

				obs_both_zero <- sum(expr_g1==0 & expr_g2==0)
				Z <- (obs_both_zero - sum(expect_both_zero)) / sqrt(sum(expect_both_err))
				#p_val <- pnorm(-abs(Z))*2
			}
			if (method == "on" | method=="both") {
				# both nonzero
				obs_both_nonzero <- sum(expr_g1!=0 & expr_g2!=0)
				expect_both_nonzero <- (1-p_g1)*(1-p_g2)
				expect_non_err <- expect_both_nonzero*(1-expect_both_nonzero)
				Z <- (obs_both_nonzero - sum(expect_both_nonzero)) / sqrt(sum(expect_non_err))
				#p_val <- pnorm(-abs(Z))*2
			}

			if (method == "both") {
				# either
				obs_either <- obs_both_zero+obs_both_nonzero
				expect_either <- expect_both_zero+expect_both_nonzero
				expect_err <- expect_either*(1-expect_either)
				Z <- (obs_either - sum(expect_either)) / sqrt(sum(expect_err))
				#p_val <- pnorm(-abs(Z))*2
			}
			Z_mat[i,j] <- Z_mat[j,i] <- Z;
		}
	}
	rownames(Z_mat) <- names(pd_gene);
	colnames(Z_mat) <- names(pd_gene);
	return(Z_mat);
}

hidden__pca_fs <- function(expr_mat, pcs = c(1,2)) {
	pca <- prcomp(log(expr_mat+1)/log(2));
	if (length(pcs) > 1) {
		score <- Matrix::rowSums(abs(pca$x[,pcs]))
	} else {
		score <- abs(pca$x[,pcs])
	}
	names(score) = rownames(expr_mat);
	return(sort(-score))
}

irlbaPcaFS <- function(expr_mat, pcs=c(2,3)) {
	#require("irlba")
	#require("Matrix")
	norm <- expr_mat
	nz_genes <- which(Matrix::rowSums(norm) != 0)
	norm[nz_genes,] <- log(norm[nz_genes,] + 1)/log(2)
	# Create sparse Matrix

	gene_names <- rownames(norm);
	norm <- Matrix::Matrix(norm, sparse=TRUE)
	rownames(norm) <- gene_names
	#indices <- which(norm > 0, arr.ind=TRUE)
	#vals <- norm[indices]
	nc <- ncol(norm)

	#norm <- Matrix::sparseMatrix(i = indices[,1], j=indices[,2], x=vals)
	# Calculate the variance across genes without converting to a dense
	# matrix:
	expression_means <- Matrix::rowMeans(norm)
	expression_vars <- Matrix::rowMeans((norm - expression_means)^2)*(nc/(nc-1)) # sample variance.
	# Filter out genes that are constant across all cells:
	genes_to_keep <- expression_vars > 0
	norm <- norm[genes_to_keep,]
	expression_means <- expression_means[genes_to_keep]
	expression_vars <- expression_vars[genes_to_keep]
	# Here✬s how to take the top PCA loading genes, but using
	# sparseMatrix operations the whole time, using irlba.
	irlba_pca_res <- irlba::irlba(Matrix::t(norm), nu=0, center=expression_means, scale=sqrt(expression_vars), right_only=TRUE)$v
	row.names(irlba_pca_res) <- row.names(norm)
	if (length(pcs) > 1) {
		score <- Matrix::rowSums(abs(irlba_pca_res[, pcs]))
	} else {
		score <- abs(irlba_pca_res[, pcs])
	}
	names(score) = gene_names;
	return(sort(-score))
}


hidden__Grun_fs <- function(expr_mat, spikes) {
#	if (!is.factor(batches)) { batches <- factor(batches) }
#	batch_spikes <- sapply(levels(batches), function(b) {rowMeans(expr_mat[spikes,batches==b])})
	spike_T <- rowMeans(expr_mat[spikes,])
	beta2 <- apply(expr_mat, 2, function(c) {lm(c[spikes]~spike_T)})
	# Model 3
	beta3 <- expr_mat[spikes,]/spike_T
	fit_gamma <- function(x) {
		b = var(x)/mean(x)
		a = mean(x)/b
		return(c(a,1/b))
	}
	gammas <- apply(beta3, 1, fit_gamma)
	a_fit = lm(gammas[1,] ~ spike_T)
	b_fit = lm(gammas[2,] ~ spike_T)
	get_nb_params <- function(mu) {
		a = a_fit$coefficients[1]+a_fit$coefficients[2]*mu
		b = b_fit$coefficients[1]+b_fit$coefficients[2]*mu
		nb_mu = mu*a/b
		nb_size = a
		return(c(nb_mu, nb_size))
	}
	gammas <- log(gammas)/log(2)
	a_fit_l = lm(gammas[1,] ~ spike_T)
	b_fit_l = lm(gammas[2,] ~ spike_T)
	ka = a_fit_l$coefficients[1]
	kb = b_fit_l$coefficients[1]
	fa = a_fit_l$coefficients[2]
	fb = b_fit_l$coefficients[2]
	get_nb_params_l <- function(mu) {
		r = 2^(ka+fa*(kb-ka)/(1+fa-fb))*mu^((fa)/(1+fa-fb))
		return(r)
	}
	# deconvolution
	min_fun <- function(bio, ns, u_g, r_g) {
		u_bio=bio[1]
		r_bio=bio[2]
		diff <- function(n) {
			ms = 1:(2*n)
			deconvolve = sum(sapply(ms, function(m) {dnbinom(n,mu=m,size=get_nb_params_l(m))*dnbinom(m,mu=u_bio, size=r_bio)}));
			abs(dnbinom(n, mu=u_g, size=r_g) - deconvolve)
		}
		sum(sapply(ns, diff));
	}
	get_bio_disp <- function(g) {
	#	g = expr_mat[1,]
		u_g = mean(g);
		v_g = var(g);
		if (v_g <= u_g) {v_g = u_g+10^-10}
		r_g = u_g^2/(v_g-u_g)
		stuff = optim(c(u_g, r_g), min_fun, ns=round(g), u_g=u_g, r_g=r_g, method="BFGS")
		return(stuff$par[2])
	}
	bio_disp <- apply(expr_mat, 1, get_bio_disp)
	names(bio_disp) = rownames(expr_mat);
	return(bio_disp)
}

# How does GiniClust do it?
hidden__ginifs_simple <- function(expr_mat) {
	#require("reldist")
	ginis <- apply(expr_mat, 1, reldist::gini)
	d <- rowMeans(expr_mat>0)
	reg <- lm(ginis~d) # almost perfect linear relation in UMI data
	score <- reg$res
	return(sort(-score));
}

giniFS <- function(expr_mat, suppress.plot=TRUE) {
	# GiniClust
	expr_mat <- expr_mat[Matrix::rowSums(expr_mat) > 0,]
	#require("reldist")
	ginis <- apply(expr_mat, 1, reldist::gini)
	max_expr <- apply(expr_mat, 1, max)
	max_expr <- log(max_expr+1)/log(2)
	fit = loess(ginis~max_expr)
	outliers = abs(fit$residuals)
	outliers = outliers > quantile(outliers, probs=0.75)
	fit2 = loess(ginis[!outliers]~max_expr[!outliers])

	
	norm_ginis = rep(NA, times=length(ginis));
	norm_ginis[!outliers] = fit2$residuals;
	to_impute = which(is.na(norm_ginis))
	impute_loess <- function(i) {
		d = abs(max_expr-max_expr[i])
		closest = which(d[!outliers] == min(d[!outliers]))
		imputed = fit2$fitted[closest[1]]
		return(imputed)
	}
	fit_ginis = sapply(to_impute, impute_loess)
	norm_ginis[outliers] = ginis[outliers]-fit_ginis
	p = pnorm(norm_ginis, mean=mean(norm_ginis), sd=sd(norm_ginis), lower.tail=FALSE)
	names(p) = rownames(expr_mat)

	if (!suppress.plot) {
		my_cols <- colorRampPalette(c("grey75","black"))(20);
		plot(max_expr, ginis, pch=16, col=rev(my_cols)[cut(log(p), breaks=20)], xlab="Max Expression", ylab="Gini");
		legend("bottomleft", bty="n", c("Color indicates estimated p-value"), col="white", pch=16, cex=0.75)
		tmp <- order(max_expr);
		yes <-  rep(NA, times=length(ginis)); yes[!outliers]= fit2$fitted; yes[outliers] = fit_ginis
		lines(max_expr[tmp], yes[tmp], col="red", lwd=3)
	}

	return(sort(p));
}
corFS <- function(expr_mat, dir=c("both", "pos", "neg"), fdr=NULL) {
	# High memory
	expr_mat <- as.matrix(expr_mat);
	if (!is.null(fdr)) {
		cor_mat = Hmisc::rcorr(t(expr_mat), type="spearman")
		p_mat <- cor_mat$P
		cor_mat <- cor_mat$r
	} else {
		cor_mat <- cor(t(expr_mat), method="spearman")
	}
	diag(cor_mat) <- 0
	if (dir[1] == "both") {
		score = apply(cor_mat, 1, function(x) {sum(abs(min(x)), abs(max(x)))})
	} else if (dir[1] == "pos") {
		score = apply(cor_mat, 1, max)
	} else if (dir[1] == "neg") {
		score = abs(apply(cor_mat, 1, min))
	} else {
		stop("Unrecognized direction")
	}
	if (!is.null(fdr)) {
		sig <- matrix(p.adjust(p_mat, method="fdr"), ncol=ncol(p_mat))
		score <- score[apply(sig, 1, min, na.rm=T) < fdr]
	}
	rm(cor_mat)
	names(score) = rownames(expr_mat)
	return(sort(-score))
}

Consensus_FS <- function(counts, norm=NA, is.spike=rep(FALSE, times=nrow(counts)), pcs=c(2,3), include_cors=TRUE) {
	# Check input
	#if (!is.matrix(counts)) {
	#	counts <- as.matrix(counts);
	#}


	if (sum(dim(counts) != dim(norm)) > 0) {
		if (is.null(dim(norm))) {
			if (length(norm) < ncol(counts)) {
				# apply CPM
				sf <- Matrix::colSums(counts[!is.spike,])
			} else {
				sf <- norm
			}
			norm <- t(t(counts)/sf*median(sf));
		} else {
			stop("Error: counts and norm matrices must be of the same dimension");
		}
	}
	if (ncol(counts) < 1000) {
		row_vars <- (rowMeans(counts^2)-rowMeans(counts)^2);
	} else {
		row_vars <- Matrix::rowSums(counts);
	}
	invariant <- row_vars == 0;
	counts <- counts[!invariant,]
	norm <- norm[!invariant,]
	# apply FS methods
	# DANB
	fit <- NBumiFitModel(counts)
	DANB <- NBumiFeatureSelectionCombinedDrop(fit)
	DANB_var <- NBumiFeatureSelectionHighVar(fit)
	# HVG
	if (sum(is.spike == TRUE) > 10) {
		spikes <- rownames(norm)[is.spike]
		HVG <- BrenneckeGetVariableGenes(norm, spikes=spikes, fdr=2, suppress.plot=TRUE)
	} else {
		warning("Warning: insufficient spike-ins using all genes for HVG");
		HVG <- BrenneckeGetVariableGenes(norm, fdr=2, suppress.plot=TRUE)
	}
	# M3Drop
	m3drop <- M3DropFeatureSelection(norm, mt_method="fdr", mt_threshold=2, suppress.plot=TRUE)
	# Gini
	gini <- giniFS(norm)
	# pca
	pca_fs <- irlbaPcaFS(norm, pcs=pcs) 
	# cor
	if (include_cors) {
		cor_fs <- corFS(norm)
	} else {
		cor_fs <- rep(-1, times=nrow(norm));
		names(cor_fs) <- rownames(norm);
	}

	# sort by mean_rank of each gene.
	ranks <- 1:nrow(norm);
	ref_order <- rownames(counts); 
	out_table <- data.frame(
		DANB_drop = ranks[match(ref_order, DANB$Gene)],
		DANB_var = ranks[match(ref_order, names(DANB_var))],
		M3Drop = ranks[match(ref_order, m3drop$Gene)],
		HVG = ranks[match(ref_order, HVG$Gene)],
		PCA = ranks[match(ref_order, names(pca_fs))],
		Cor = ranks[match(ref_order, names(cor_fs))],
		Gini = ranks[match(ref_order, names(gini))]
		)
	if (!include_cors) {
		out_table$Cor <- rep(-1, times=nrow(out_table));
	}
	rownames(out_table) <- ref_order;
	consensus_score <- rowMeans(out_table, na.rm=TRUE);
	out_table <- out_table[order(consensus_score),]
	out_table$Cons <- 1:nrow(out_table) 

	return(out_table)
}

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

#This contains code by Brennecke et al. published in http://www.nature.com/nmeth/journal/v10/n11/full/nmeth.2645.html#supplementary-information
BrenneckeGetVariableGenes <- function(expr_mat, spikes=NA, suppress.plot=FALSE, fdr=0.1, minBiolDisp=0.5, fitMeanQuantile=0.8) {
        #require(statmod)

        rowVars <- function(x) { unlist(apply(x,1,var, na.rm=TRUE))}

        colGenes <- "black"
        colSp <- "blue"


        fullCountTable <- expr_mat;

        if (is.character(spikes)) {
                sp <- rownames(fullCountTable) %in% spikes;
                countsSp <- fullCountTable[sp,];
                countsGenes <- fullCountTable[!sp,];
        } else if (is.numeric(spikes)) {
                countsSp <- fullCountTable[spikes,];
                countsGenes <- fullCountTable[-spikes,];
        } else {
                countsSp <- fullCountTable;
                countsGenes <- fullCountTable;
        }

        meansSp <- rowMeans(countsSp, na.rm=TRUE)
        varsSp <- rowVars(countsSp)
        cv2Sp <- varsSp/meansSp^2
        meansGenes <- rowMeans(countsGenes, na.rm=TRUE)
        varsGenes <- rowVars(countsGenes)
        cv2Genes <- varsGenes/meansGenes^2
        # Fit Model
        minMeanForFit <- unname( quantile( meansSp[ which( cv2Sp > 0.3 ) ], fitMeanQuantile))
        useForFit <- meansSp >= minMeanForFit
        if (sum(useForFit, na.rm=TRUE) < 20) {
                warning("Too few spike-ins exceed minMeanForFit, recomputing using all genes.")
                meansAll <- c(meansGenes, meansSp)
                cv2All <- c(cv2Genes,cv2Sp)
                minMeanForFit <- unname( quantile( meansAll[ which( cv2All > 0.3 ) ], 0.80))
                useForFit <- meansSp >= minMeanForFit
        }
        if (sum(useForFit, na.rm=TRUE) < 30) {warning(paste("Only", sum(useForFit), "spike-ins to be used in fitting, may result in poor fit."))}
        fit <- glmgam.fit( cbind( a0 = 1, a1tilde = 1/meansSp[useForFit] ), cv2Sp[useForFit] )
        a0 <- unname( fit$coefficients["a0"] )
        a1 <- unname( fit$coefficients["a1tilde"])
	res <- cv2Genes - (a0 + a1/meansGenes);

        # Test
        psia1theta <- a1
        minBiolDisp <- minBiolDisp^2
        m <- ncol(countsSp);
        cv2th <- a0 + minBiolDisp + a0 * minBiolDisp
        testDenom <- (meansGenes*psia1theta + meansGenes^2*cv2th)/(1+cv2th/m)
        p <- 1-pchisq(varsGenes * (m-1)/testDenom,m-1)
        padj <- p.adjust(p,"BH")
        sig <- padj < fdr
        sig[is.na(sig)] <- FALSE
        if (!suppress.plot) {
                plot( meansGenes,cv2Genes, xaxt="n", yaxt="n", log="xy",
                        xlab = "average normalized read count",
                        ylab = "squared coefficient of variation (CV^2)", col="white")
                axis( 1, 10^(-2:5), c( "0.01", "0.1", "1", "10", "100", "1000",
                        expression(10^4), expression(10^5) ) )
                axis( 2, 10^(-2:3), c( "0.01", "0.1", "1", "10", "100","1000" ), las=2 )
                abline( h=10^(-2:1), v=10^(-1:5), col="#D0D0D0", lwd=2 )
                # Plot the genes, use a different color if they are highly variable
                points( meansGenes, cv2Genes, pch=20, cex=.2,
                        col = ifelse( padj < .1, "#C0007090", colGenes ) )
		# Plot/highlight the spike-ins if they are different from the genes
		if (length(meansSp) < length(meansGenes)) {
			points(meansSp, cv2Sp, pch=20, cex=.5, col=colSp)
		}
                # Add the technical noise fit
                xg <- 10^seq( -2, 6, length.out=1000 )
                lines( xg, (a1)/xg + a0, col="#FF000080", lwd=3 )
                # Add a curve showing the expectation for the chosen biological CV^2 thershold
                lines( xg, psia1theta/xg + a0 + minBiolDisp, lty="dashed", col="#C0007090", lwd=3)
        }
	TABLE <- data.frame(Gene = names(meansGenes)[sig], effect.size=res[sig], p.value = p[sig], q.value= padj[sig])
	TABLE <- TABLE[order(-TABLE[,2]),];
        return(TABLE)
}

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

hidden_get_K <- function(expr_mat) {
	stuff <- bg__calc_variables(expr_mat);
	fit <- bg__fit_MM(stuff$p,stuff$s);
	return(fit$K);
}

bg__get_mean2disp <- function(expr_mat) {
	cv2 <- matrixStats::rowVars(expr_mat)/((rowMeans(expr_mat, na.rm=T))^2)
	xes <- log(rowMeans(expr_mat, na.rm=T))/log(10)
	reg <- lm(log(cv2[xes > 0])~xes[xes>0])
	mean2disp_fun <- function(mu){
		cv2 <- exp(reg$coeff[1]+reg$coeff[2]*(log(mu)/log(10)))
		variance <- cv2*(mu^2)
		if (variance <= mu) {variance<-1.01*mu}
		disp <- mu^2/(variance-mu)
		return(1/disp)
	}
	return(mean2disp_fun)
}

bg__fitdispersion <- function(expr_mat) {
	V <- matrixStats::rowVars(expr_mat)
	mu <- rowMeans(expr_mat)
	xes <- log(mu)/log(10)
	V[V <= mu] <- mu[V <= mu]+10^-10;
	nb_size <- mu^2/(V-mu);

	reg <- lm(log(nb_size[xes>0])~xes[xes>0])
	return(reg$coefficients[2]);
}

hidden__cv2coeffs <- function(expr_mat) {
	cv2 <- matrixStats::rowVars(expr_mat)/((rowMeans(expr_mat, na.rm=T))^2)
	xes <- log(rowMeans(expr_mat, na.rm=T))/log(10)
	reg <- lm(log(cv2[xes > 0])~xes[xes>0])
	return(c(reg$coeff[1], reg$coeff[2]))
}

hidden_calc_p <- function(obs, mu, K, disp) {
	if (mu == 0 & obs != 0) {stop("Error:non-zero obs has zero mean")}
	if (obs == 0) {
		p <- 1-mu/(mu+K)
	} else {
		p <- dnbinom(obs, size=1/disp, mu=mu)
		if (p < 10^-200) {
			p = 10^-200
		}
	}
	return(p);
}
unfinished__m3dTraditionalDE <- function(expr_mat, groups, batches=rep(1, times=length(expr_mat[1,])), fdr=0.05) {
	# Batch-specific mean-variance
	# Check Input
	if (!is.factor(batches)) {
		batches <- factor(batches)	
	}
	if (!is.factor(groups)) {
		groups <- factor(groups)	
	}
	if (length(batches) != length(groups) | 
	    length(batches) != length(expr_mat[1,])) {
		stop("Error: length of groups and batches must match number of cells (columns of expr_mat)");
	}
	if (!is.matrix(expr_mat)) {
		expr_mat <- as.matrix(expr_mat);
	}
	# Fit Batches
	batch_levels <- levels(batches)
	Ks <- sapply(batch_levels, function(b){hidden_get_K(expr_mat[,batches == b])})
	DispFits <- sapply(batch_levels, function(b){bg__get_mean2disp(expr_mat[,batches == b])})

	Ms <- rowMeans(expr_mat, na.rm=T)
	Mis <- by(t(expr_mat), groups, colMeans)

	#### Move to C? ####
	AllOut <- sapply(1:length(expr_mat[,1]), function(g) {
#	for (g in 1:length(expr_mat[,1])) {
		probs <- sapply(1:length(expr_mat[1,]), function(i) {
				obs<-expr_mat[g,i]
				M <- Ms[g]
				b <- as.numeric(batches[i])
				group <- as.numeric(groups[i])
				Mi <- Mis[[group]][g]
				p1 <- hidden_calc_p(round(obs),M,Ks[b], DispFits[[b]](M))
				p2 <- hidden_calc_p(round(obs),Mi,Ks[b], DispFits[[b]](Mi))
				return(cbind(p1,p2))
			})
		D <- -2*(sum(log(probs[1,]))-sum(log(probs[2,])))
		df <- length(levels(groups))-1
		pval <- pchisq(D, df=df, lower.tail=FALSE)
		output <- c(as.vector(by(expr_mat[g,], groups, mean)),as.vector(by(expr_mat[g,], batches, mean)), pval)
	})
#		if (g == 1) {
#			AllOut <- output
#		} else {
#			AllOut = rbind(AllOut, output)
#		}
#	}
	#### --------- ####
	AllOut <- t(AllOut)
	rownames(AllOut) <- rownames(expr_mat)
	AllOut <- cbind(AllOut, p.adjust(AllOut[,length(AllOut[1,])], method="fdr"));		
	colnames(AllOut) <- c(levels(groups), levels(batches), "p.value", "q.value")
	AllOut <- AllOut[AllOut[,length(AllOut[1,])] < fdr,]
	return(AllOut);
}


unfinished__m3dTraditionalDEShiftDisp <- function(expr_mat, groups, batches=rep(1, times=length(expr_mat[1,])), fdr=0.05) {
	# Batch specific mean-variance, gene-specific variance.
	# Check Input
	if (!is.factor(batches)) {
		batches <- factor(batches)	
	}
	if (!is.factor(groups)) {
		groups <- factor(groups)	
	}
	if (length(batches) != length(groups) | 
	    length(batches) != length(expr_mat[1,])) {
		stop("Error: length of groups and batches must match number of cells (columns of expr_mat)");
	}
	if (!is.matrix(expr_mat)) {
		expr_mat <- as.matrix(expr_mat);
	}
	# Fit Batches
	batch_levels <- levels(batches)
	Ks <- sapply(batch_levels, function(b){hidden_get_K(expr_mat[,batches == b])})
	DispFits <- sapply(batch_levels, function(b){bg__fitdispersion(expr_mat[,batches == b])}) # Fit mean-variance for each batch

	Ms <- rowMeans(expr_mat, na.rm=T)
	Mis <- by(t(expr_mat), groups, colMeans)
	V <- matrixStats::rowVars(expr_mat)
	V[V<=Ms] <- Ms[V<=Ms] + 10^-10;
	nb_size <- Ms^2/(V-Ms); # gene-specific dataset-wide dispersion

	#### Move to C? ####
	AllOut <- sapply(1:length(expr_mat[,1]), function(g) {
#	for (g in 1:length(expr_mat[,1])) {
		probs <- sapply(1:length(expr_mat[1,]), function(i) {
				obs<-expr_mat[g,i]
				M <- Ms[g]
				b <- as.numeric(batches[i])
				group <- as.numeric(groups[i])
				Mi <- Mis[[group]][g]
				# Shift Dispersion
				slope <- DispFits[b]
				disp1 <- nb_size[g]
				if (disp1 <= 0) {disp1 = 10^-10}
				tmp_intercept <- log(disp1)-slope*log(M)
				disp2 <- exp(slope*log(Mi)+tmp_intercept)
				if (disp2 <= 0) {disp2 = 10^-10}
				# Calculate probability
				p1 <- hidden_calc_p(round(obs),M,Ks[b], 1/disp1)
				p2 <- hidden_calc_p(round(obs),Mi,Ks[b], 1/disp2)
				return(cbind(p1,p2))
			})
		D <- -2*(sum(log(probs[1,]))-sum(log(probs[2,])))
		df <- length(levels(groups))-1
		pval <- pchisq(D, df=df, lower.tail=FALSE)
		output <- c(as.vector(by(expr_mat[g,], groups, mean)),as.vector(by(expr_mat[g,], batches, mean)), pval)
	})
#		if (g == 1) {
#			AllOut <- output
#		} else {
#			AllOut = rbind(AllOut, output)
#		}
#	}
	#### --------- ####
	AllOut <- t(AllOut)
	rownames(AllOut) <- rownames(expr_mat)
	AllOut <- cbind(AllOut, p.adjust(AllOut[,length(AllOut[1,])], method="fdr"));		
	colnames(AllOut) <- c(levels(groups), levels(batches), "p.value", "q.value")
	AllOut <- AllOut[AllOut[,length(AllOut[1,])] < fdr,]
	return(AllOut);
}

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

# DE Genes functions

hidden__test_DE_K_equiv_raw <- function (expr_mat, fit=NA) {
	gene_info <- bg__calc_variables(expr_mat);
	if (is.na(fit)[1]) {
		fit <- bg__fit_MM(gene_info$p, gene_info$s);
	}
	p_obs <- gene_info$p;
	N <- length(expr_mat[1,]);
	p_err <- gene_info$p_stderr;
	S_mean <- gene_info$s
	S_err <- gene_info$s_stderr
	K_err <- fit$Kerr;
	K_equiv <- p_obs*S_mean/(1-p_obs);
	K_equiv_err <- p_obs/(1-p_obs)*S_err
		
	Z <- (K_equiv - fit$K)/sqrt(K_equiv_err^2+K_err^2); # high = shifted right, low = shifted left
	pval <- pnorm(Z, lower.tail=FALSE)
	effect_size <- K_equiv/fit$K;
	return(list(pval = pval, fold_change = effect_size))
}

bg__test_DE_K_equiv <- function(gene_info, fit=NA) {
	if (is.na(fit)[1]) {
		fit <- bg__fit_MM(gene_info$p, gene_info$s);
	}
	p_obs <- gene_info$p;
	always_detected <- p_obs==0
	p_obs[p_obs==0] <- min(p_obs[p_obs > 0])/2 # Here so that K_equiv for p_obs==0 will be 0 but K_equiv_err will not throw errors for such genes.
	p_err <- gene_info$p_stderr;
	S_mean <- gene_info$s
	S_err <- gene_info$s_stderr
	K_err <- fit$Kerr;
	K_obs <- fit$K
	K_equiv <- p_obs*S_mean/(1-p_obs);
	K_equiv_err <- abs(K_equiv)*sqrt((S_err/S_mean)^2 + (p_err/p_obs)^2)


	K_equiv_log <- log(K_equiv)
	thing <- K_equiv-K_equiv_err; thing[thing <= 0 ] <- 10^-100
	K_equiv_err_log <- abs(log(thing)-K_equiv_log) # Testing to the right so this is what we are interested in
#	K_equiv_err_log = K_equiv_err/K_equiv # This does not hold when K_equiv_err =~ K_equiv which is particularly problematic for lowly expressed genes
	K_equiv_err_log[K_equiv-K_equiv_err <= 0 ] <- 10^10
	K_obs_log <- log(fit$K)
	K_err_log <- sd(K_equiv_log-K_obs_log, na.rm=T)/sqrt(length(K_equiv_log)) 
		
	Z <- (K_equiv_log - K_obs_log)/sqrt(K_equiv_err_log^2+K_err_log^2); # high = shifted right, low = shifted left
	pval <- pnorm(Z, lower.tail=FALSE)
	pval[always_detected] <- 1;
	pval[is.na(pval)] <- 1; # deal with never detected
	effect_size <- K_equiv/fit$K;
	effect_size[is.na(effect_size)] <- 1; # deal with never detected
	return(list(pval = pval, fold_change = effect_size))
}
# Use the fact that errors of proportions are well define by converting S to proportion detected equivalents?
hidden__test_DE_P_equiv <- function (expr_mat,  fit=NA) {
	gene_info <- bg__calc_variables(expr_mat);
	if (is.na(fit)[1]) {
		fit <- bg__fit_MM(gene_info$p, gene_info$s);
	}
	p_obs <- gene_info$p;
	N <- length(expr_mat[1,]);
	p_err <- gene_info$p_stderr;
	S_mean <- gene_info$s
	S_err <- gene_info$s_stderr
	K_err <- fit$Kerr;
	p_equiv <- fit$predictions;
	propagated_err_p_equiv <- p_equiv*sqrt(((S_err+K_err)/(S_mean+fit$K))^2+(K_err/fit$K)^2)
	fitted_err_p_equiv <- fit$fitted_err
	Z <- (p_equiv - p_obs)/fitted_err_p_equiv; # low = shifted right, high = shifted left
	pval <- pnorm(Z, lower.tail=TRUE)
	effect_size <- p_obs/p_equiv;
	return(list(pval = pval, fold_change = effect_size))
}

# Use the fact that S as a function of P is more stable to noise for the main part of the curve
hidden__test_DE_S_equiv <- function (expr_mat, fit=NA, method="propagate") {
	gene_info <- bg__calc_variables(expr_mat);
	if (is.na(fit[1])) {
		fit <- bg__fit_MM(gene_info$p, gene_info$s);
	}
	p_obs <- gene_info$p;
	N <- length(expr_mat[1,]);
	p_err <- gene_info$p_stderr;
	S_mean <- gene_info$s
	S_err <- gene_info$s_stderr
	K_err <- fit$Kerr;
	S_equiv <- hidden__invert_MM(fit$K,p_obs);

	## Monte Carlo method to estimate error around S_equiv ##
	MC_err <- function (p_base) {
		p_rand <- rnorm(10000, mean = p_base, sd = p_err);
		p_rand <- p_rand[p_rand > 0 & p_rand < 1]
		K_rand <- rnorm(length(p_rand),fit$K,sd = K_err);
		K_rand[K_rand < 1] <- 1;
		S_equiv_rand <- hidden__invert_MM(K_rand, p_rand)
		sd(S_equiv_rand)
	}
	if (method == "MC") {
		S_equiv_err <- unlist(lapply(p_obs,MC_err))
	} else {
		S_equiv_err <- S_equiv*sqrt(2*(p_err/p_obs)^2+(K_err/fit$K)^2);
	}

	Z <- (S_equiv - S_mean)/sqrt(S_err^2+S_equiv_err^2); # low = shifted right, high = shifted left
	pval <- pnorm(Z, lower.tail=TRUE)*2
	effect_size <- (S_mean-S_equiv)/S_equiv;
	return(list(pval = pval, effect = effect_size))
}

bg__get_extreme_residuals <- function (expr_mat, fit=NA, fdr_threshold = 0.1, percent = NA, v_threshold=c(0.05,0.95), direction="right", suppress.plot = FALSE) {
	gene_info = bg__calc_variables(expr_mat);
	if (is.na(fit)[1]) {
		fit <- bg__fit_MM(gene_info$p, gene_info$s);
	}
	res <- bg__horizontal_residuals_MM_log10(fit$K, gene_info$p, gene_info$s)
	res <- res[gene_info$p < max(v_threshold) & gene_info$p > min(v_threshold)]

	if (is.na(percent)) {
		mu <- mean(res); sigma <- sd(res);
		# deal with potential bi-modality
		if (sum(res > mu-sigma & res < mu+sigma) < 0.5) { # should be 0.68 theoretically
			mu <- mean(res[res > quantile(res,0.33)]);
			sigma <- sd(res[res > quantile(res,0.33)]);
		}
	
		if (direction=="right") {
			pval <-pnorm((res-mu)/sigma, lower.tail=FALSE)
		} else {
			pval <- pnorm((res-mu)/sigma, lower.tail=TRUE)
		}
		qval <- p.adjust(pval, method="fdr");
		sig <- qval < fdr_threshold;

		# Plot fitted normal curve
		if (!suppress.plot) {
			hist(res, col="grey75", xlab="horizontal residuals", main="", prob=TRUE)
			curve(dnorm(x,mean=mu, sd=sigma), add=TRUE);
			if (direction=="right" & sum(sig) > 0) {
				abline(v=min(res[sig]), col="red");
			} else {
				abline(v=max(res[sig]), col="red");
			}
		}
		return(names(pval)[sig]);
	} else {
		if (direction=="right") {
			cut_off <- quantile(res,prob=1-percent);
			return(names(res)[res > cut_off]);
		} else {
			cut_off <- quantile(res,prob=percent);
			return(names(res)[res < cut_off]);
		}
	}
}
##### Assembled Analysis Chunks ####
M3DropFeatureSelection <- function(expr_mat, mt_method="bon", mt_threshold=0.05, suppress.plot=FALSE, xlim=NA) {
	BasePlot <- bg__dropout_plot_base(expr_mat, xlim = xlim, suppress.plot=suppress.plot);
	MM <- bg__fit_MM(BasePlot$gene_info$p, BasePlot$gene_info$s);
	if (!suppress.plot) {
		sizeloc <- bg__add_model_to_plot(MM, BasePlot, lty=1, lwd=2.5, col="dodgerblue", legend_loc = NULL);
	}
	DEoutput <- bg__test_DE_K_equiv(BasePlot$gene_info, fit=MM);
	sig <- which(p.adjust(DEoutput$pval, method=mt_method) < mt_threshold);
	DEgenes <- rownames(expr_mat)[sig];
	DEgenes <- DEgenes[!is.na(DEgenes)];
	if (!suppress.plot) {
		bg__highlight_genes(BasePlot, expr_mat, DEgenes);
	}
	
	TABLE <- data.frame(Gene = DEgenes, effect.size=DEoutput$fold_change[sig], p.value = DEoutput$pval[sig], q.value= p.adjust(DEoutput$pval, method=mt_method)[sig])
	TABLE <- TABLE[order(-TABLE[,2]),];
	return(TABLE)
}

M3DropGetExtremes <- function(expr_mat, fdr_threshold = 0.1, percent = NA, v_threshold=c(0.05,0.95), suppress.plot=FALSE) {
	BasePlot <- bg__dropout_plot_base(expr_mat, xlim = NA, suppress.plot=suppress.plot);
	MM <- bg__fit_MM(BasePlot$gene_info$p, BasePlot$gene_info$s);
	if (!suppress.plot) {
		sizeloc <- bg__add_model_to_plot(MM, BasePlot, lty=1, lwd=2.5, col="black",legend_loc = "topright");
	}

	if (is.na(percent)) {
		shifted_right <- bg__get_extreme_residuals(expr_mat, fit=MM, v_threshold=v_threshold, fdr_threshold = fdr_threshold, direction="right", suppress.plot=TRUE)
		shifted_left  <- bg__get_extreme_residuals(expr_mat, fit=MM, v_threshold=v_threshold, fdr_threshold = fdr_threshold, direction="left",  suppress.plot=TRUE)
	} else {
		shifted_right <- bg__get_extreme_residuals(expr_mat, fit=MM, v_threshold=v_threshold, percent = percent, direction="right", suppress.plot=TRUE)
		shifted_left  <- bg__get_extreme_residuals(expr_mat, fit=MM, v_threshold=v_threshold, percent = percent, direction="left",  suppress.plot=TRUE)

	}
	if (!suppress.plot) {
		bg__highlight_genes(BasePlot, expr_mat, shifted_right, col="orange");
		bg__highlight_genes(BasePlot, expr_mat, shifted_left, col="purple");
	}
	return(list(left=shifted_left,right=shifted_right));
}

M3DropTestShift <- function(expr_mat, genes_to_test, name="", background=rownames(expr_mat), suppress.plot=FALSE) {
	BasePlot <- bg__dropout_plot_base(expr_mat, xlim = NA, suppress.plot=suppress.plot);
	MM <- bg__fit_MM(BasePlot$gene_info$p, BasePlot$gene_info$s);
	if (!suppress.plot) {
		sizeloc <- bg__add_model_to_plot(MM, BasePlot, lty=1, lwd=2.5, col="black",legend_loc = "topright");
		bg__highlight_genes(BasePlot, expr_mat, genes_to_test);
		title(main=name);
	}

	res <- bg__horizontal_residuals_MM_log10(MM$K, BasePlot$gene_info$p, BasePlot$gene_info$s)
	res[is.infinite(res)] <- NA;
	mu <- median(res[rownames(expr_mat) %in% as.character(background)], na.rm=TRUE); #sigma = sd(res, na.rm=TRUE);
	s_mu <- median(res[rownames(expr_mat) %in% as.character(genes_to_test)], na.rm=TRUE);
	pval<-suppressWarnings(wilcox.test(res[rownames(expr_mat) %in% as.character(genes_to_test)], res[rownames(expr_mat) %in% as.character(background)], na.rm=TRUE)$p.value)
	return(data.frame(sample=s_mu, background=mu, p.value=pval));
}

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

# Normalization Functions
hidden__UQ <- function(x){quantile(x[x>0],0.75)};

bg__filter_cells <- function(expr_mat,labels=NA, suppress.plot=FALSE, min_detected_genes=NA) {
	num_detected <-  Matrix::colSums(expr_mat > 0, na.rm=TRUE);
	if (!is.na(min_detected_genes)) {
		low_quality <- num_detected < min_detected_genes;
	} else {
		num_zero <- Matrix::colSums(expr_mat == 0, na.rm=TRUE);
		cell_zero <- num_zero;
		mu <- mean(cell_zero);
		sigma <- sd(cell_zero);
		# Deal with bi-modal
		if (sum(cell_zero > mu-sigma & cell_zero < mu+sigma) < 0.5) { # should be 0.68 theoretically
			mu <- mean(cell_zero[cell_zero < median(cell_zero)]);
			sigma <- sd(cell_zero[cell_zero < median(cell_zero)]);
		}
		low_quality <- p.adjust(pnorm((cell_zero-mu)/sigma, lower.tail=FALSE), method="fdr") < 0.05;
		if (!suppress.plot) {
			hist(cell_zero, col="grey75", xlab="Number of zeros (per cell)", main="", prob=TRUE)
			curve(dnorm(x,mean=mu, sd=sigma), add=TRUE)
			if (sum(low_quality) > 0) {
				abline(v=min(cell_zero[low_quality]), col="red")
			}
		}
	}
	if (sum(low_quality) > 0) {
		if (length(labels)==length(expr_mat[1,])) {labels = labels[!low_quality]}
		expr_mat <- expr_mat[,!low_quality];
	}
	return(list(data = expr_mat, labels = labels));
}

hidden__normalize <- function(data) {
	# Combine UQ and detection rate adjusted normalization 
	# Stephanie Hick, Mingziang Teng, Rafael A Irizarry "On the widespread and critical impact of systematic single-cell RNA-Seq data" http://dx.doi.org/10.1101/025528 
	cell_zero <- Matrix::colSums(data == 0)/length(data[,1]);
	uq <- unlist(apply(data,2,hidden__UQ));
	normfactor <- (uq/median(uq)) * (median(cell_zero)/cell_zero); 
	data <- t(t(data)/normfactor);
	return(data);
}

M3DropCleanData <- function(expr_mat, labels = NA, is.counts=TRUE, suppress.plot=FALSE, pseudo_genes=NA, min_detected_genes=NA) {
	expr_mat[is.na(expr_mat)] = 0;
	if (length(pseudo_genes) > 1) {
		is_pseudo <- rownames(expr_mat) %in% as.character(pseudo_genes);
	        expr_mat <- expr_mat[!is_pseudo,];
	}

	data_list <- bg__filter_cells(expr_mat, labels, suppress.plot = suppress.plot, min_detected_genes=min_detected_genes);
	
        detected <- Matrix::rowSums(data_list$data > 0) > 3;
        expr_mat <- data_list$data[detected,];
        detected <- Matrix::rowSums(data_list$data> 0) > 3;
        expr_mat <- data_list$data[detected,];
	labels   <- data_list$labels

	spikes <- grep("ercc",rownames(expr_mat), ignore.case=TRUE)
	if (is.counts) {
		if (length(spikes) > 1) {
	                totreads <- Matrix::colSums(expr_mat[-c(spikes),])
		} else {
			totreads <- Matrix::colSums(expr_mat);
		}
                cpm <- t(t(expr_mat)/totreads)*1000000;
                lowExpr <- rowMeans(cpm) < 10^-5;
                cpm<-cpm[!lowExpr,];
                return(list(data=cpm, labels=labels));
        }

	lowExpr <- rowMeans(expr_mat) < 10^-5;
        data<-expr_mat[!lowExpr,];
        return(list(data=expr_mat, labels=labels));
}

#### Pearson Residuals ####
# February 16, 2023

NBumiPearsonResiduals <- function(counts, fits=NULL) {
	if (is.null(fits)) {
		fits <- NBumiFitModel(counts)
	}
	mus <- t(t(fits$vals$tjs/fits$vals$total)) %*% fits$vals$tis
	pearson <- (counts-mus)/sqrt(mus + mus^2/fits$sizes)
	return(pearson)
}

NBumiPearsonResidualsApprox <- function(counts, fits=NULL) {
	if (is.null(fits)) {
		vals <- hidden_calc_vals(counts);
	} else {
		vals <- fits$vals
	}
	mus <- t(t(vals$tjs/vals$total)) %*% vals$tis
	pearson <- (counts-mus)/sqrt(mus)
	return(pearson)
}
#############################

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

bg__fit_MM <- function (p,s) {
	if (length(p) != length(s)) {
		stop(print("Error: p and s not same length. Cannot fit Michaelis-Menten."))
	}
	#require("bbmle")
	LL <- function(krt,sigma) {
		R <- p-(1-(s/((krt+s))))
		R <- suppressWarnings(dnorm(R,0,sigma,log=TRUE))
		-sum(R)
	}
	fit <- mle2(LL,start=list(krt=3, sigma=0.25))
	thing <- summary(fit)
	krt <- fit@coef[1]
	res_err <- fit@coef[2];
	Kerr <- max(fit@coef[2],attributes(summary(fit))$coef[1,2]);
	predicted <- 1-(s/(krt+s))
	residuals <- p-predicted
	return(list(K=krt,Kerr=Kerr,fitted_err = res_err,predictions=predicted, model=c("MMenten",paste("K =",round(krt,digits=2))),SSr=round(sum((residuals)^2)),SAr=round(sum(abs(residuals)))))
}

hidden__fit_MM_lognormal<-function(p,s){
	# This consistently underestimates K
	if (length(p) != length(s)) {
		stop(print("Error: p and s not same length. Cannot fit Michaelis-Menten."))
	}
	#require("bbmle")
	p_c <- p[p<1 & p>0]
	s_c <- s[p<1 & p>0]
	LL <- function(krt,sigma) {
		if (krt >0) {
			obs_Ks <- p_c/(1-p_c)*s_c
			R <- log(obs_Ks)-log(krt);
			Qs <- quantile(R,prob=c(0.25,0.5,0.75))
			IQR <- Qs[2]-Qs[1];

			thing <- densCols(p_c, log(s_c)/log(10), colramp = colorRampPalette(c("black","white")));
			dens <- Matrix::colSums(col2rgb(thing))
			thresh <- quantile(dens, prob=0.05);

			R <- R[dens > thresh]
			R <- suppressWarnings(dnorm(R,0,sigma,log=TRUE))
			-sum(R)
		} else {
			10^100
		}
	}
	fit <- mle2(LL,start=list(krt=6, sigma=0.25))
	thing <- summary(fit)
	krt <- fit@coef[1]
	res_err <- fit@coef[2];
	Kerr <- max(fit@coef[2],attributes(summary(fit))$coef[1,2]);
	predicted <- 1-(s/(krt+s))
	residuals <- p-predicted
	return(list(K=krt,Kerr=Kerr,fitted_err = res_err,predictions=predicted, model=c("MMenten",paste("K =",round(krt,digits=3))),SSr=round(sum((residuals)^2)),SAr=round(sum(abs(residuals)))))
}

hidden__fit_MM_logistic <- function(p,s){
	s_nozero <- s[s>0];
	p_nozero <- p[s>0];
	fit <- glm(p_nozero ~ offset(-1*log(s_nozero)), family="binomial")
	Kerr <- summary(fit)$coeff[1,2];
	Kcoeff <- summary(fit)$coeff[1,1];
	krt <- exp(Kcoeff)
	Kerr <- Kerr*krt
	predicted <- rep(0, times=length(s));
	predicted[s>0] <- fitted(fit)
	residuals <- p-predicted
	return(list(K=krt,Kerr=Kerr,predictions=predicted, model=c("MMenten",paste("K =",round(krt,digits=3))),SSr=round(sum((residuals)^2)),SAr=round(sum(abs(residuals)))))
}

bg__fit_logistic <- function(p,s) {
	if (length(p) != length(s)) {
		stop(print("Error: p and s not same length. Cannot fit Logistic Regression."))
	}
	s_nozero <- s[s>0];
	p_nozero <- p[s>0];
        logistic <- suppressWarnings(glm(p_nozero~log(s_nozero),family="binomial")) #warns that is not binary data
        predlog <- fitted(logistic)
	fullpredictions <- rep(0, times=length(s));
	fullpredictions[s>0] <- predlog
	res <- fullpredictions-p;
	return(list(predictions=fullpredictions, B0 = logistic$coeff[1], B1=logistic$coeff[2] ,model=c( "Logistic", paste("Intercept =",round(logistic$coeff[1],digits=3)),paste("Coeff =",round(logistic$coeff[2],digits=3))),SSr=round(sum(res^2)),SAr=round(sum(abs(res)))));
}

bg__fit_ZIFA <- function(p,s) {
	if (length(p) != length(s)) {
		stop(print("Error: p and s not same length. Cannot fit double exponential."))
	}
	p_nozero <- p; p_nozero[p == 0] = min(p[p>0])/10
	fit <- lm(log(p_nozero)~-1+s^2)
	lambda <- -fit$coeff[1]
	res_err <- attributes(summary(fit))$coef[2,1]
	Lerr <- summary(fit)$coeff[1,2]
	predicted <- exp(-lambda*s*s)
	residuals <- p-predicted
	return(list(lambda=lambda,Lerr=Lerr,fitted_err = res_err,predictions=predicted, model=c("p ~ e^(-lambda*S^2)",paste("lambda =",signif(lambda,digits=2))),SSr=round(sum((residuals)^2)),SAr=round(sum(abs(residuals)))))
}

M3DropDropoutModels <- function(expr_mat, xlim=NA, suppress.plot=FALSE) {
	BasePlot <- bg__dropout_plot_base(expr_mat, xlim = xlim, suppress.plot=suppress.plot);
	MM <- bg__fit_MM(BasePlot$gene_info$p, BasePlot$gene_info$s);
	SCDE <- bg__fit_logistic(BasePlot$gene_info$p, BasePlot$gene_info$s);
	ZIFA <- bg__fit_ZIFA(BasePlot$gene_info$p, BasePlot$gene_info$s);
	if (!suppress.plot) {
	  	sizeloc <- bg__add_model_to_plot(MM, BasePlot, lty=1, lwd=2.5, col="black",legend_loc = "topright");
		sizeloc <- bg__add_model_to_plot(SCDE, BasePlot, lty=2, lwd=2.5, col="magenta3",legend_loc = c(sizeloc$rect$left+sizeloc$rect$w,sizeloc$rect$top-sizeloc$rect$h-0.05));
		sizeloc <- bg__add_model_to_plot(ZIFA, BasePlot, lty=3, lwd=2.5, col="red",legend_loc = c(sizeloc$rect$left+sizeloc$rect$w,sizeloc$rect$top-sizeloc$rect$h-0.05));
	}
	invisible(list(MMFit = MM, LogiFit = SCDE, ExpoFit = ZIFA));
}

## Cell-type DANB ##
# fit variances with contribution from global mean~var relationship for small clusters. 
#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

#### Fitting #####
hidden_calc_vals <- function(counts) {
        if (sum(counts < 0) >0) {stop("Expression matrix contains negative values! Please provide raw UMI counts!")}
	if ( sum(counts >= 1) != sum(counts > 0) ) {stop("Error: Expression matrix is not integers! Please provide raw UMI counts.")}
#        if (sum(!is.integer(counts)) >0) {stop("Expression matrix is not integers! Please provide a matrix (not data.frame) raw UMI counts!")}
	if (is.null(rownames(counts))) { rownames(counts) <- as.character(1:nrow(counts)) }

        tjs <- Matrix::rowSums(counts, na.rm=T) # Total molecules/gene
	no_detect <- sum(tjs <= 0)
	if (no_detect > 0) {
		stop(paste("Error: contains",no_detect, "undetected genes."))
	}
        tis <- Matrix::colSums(counts, na.rm=T) # Total molecules/cell
	if (sum(tis <= 0) > 0) {stop("Error: all cells must have at least one detected molecule.")}
        djs <- ncol(counts)-Matrix::rowSums(counts > 0, na.rm=T) # Observed Dropouts per gene
        dis <- nrow(counts)-Matrix::colSums(counts > 0, na.rm=T) # Observed Dropouts per cell
        nc <- length(counts[1,]) # Number of cells
        ng <- length(counts[,1]) # Number of genes
        total <- sum(tis, na.rm=T) # Total molecules sampled
        return(list(tis=tis, tjs=tjs, dis=dis, djs=djs, total=total,nc=nc,ng=ng));
}

NBumiConvertToInteger <- function(mat) {
        mat <- ceiling(as.matrix(mat))
        storage.mode(mat) <- "integer"
        mat <- mat[Matrix::rowSums(mat) > 0,]
        return(mat)
}

NBumiFitModel <- function(counts) {
	vals <- hidden_calc_vals(counts);
#	mus <- (vals$tjs) %*% t(vals$tis/vals$total)
	
#	max_size <- max(mus)^2;
	min_size <- 10^-10;
	my_rowvar <- sapply(1:nrow(counts), function(i){
				mu_is <- vals$tjs[i]*vals$tis/vals$total
				var(as.vector(unlist(counts[i,]))-mu_is)
			})
#	my_rowvar <- vector(length=nrow(counts));
#	for (i in 1:nrow(counts)) {
#		mu_is <- vals$tjs[i]*vals$tis/vals$total
#		my_rowvar[i] <- var(as.vector(unlist(counts[i,]))-mu_is)
#	}
		
	size <- vals$tjs^2*(sum(vals$tis^2)/vals$total^2)/((vals$nc-1)*my_rowvar-vals$tjs) # for this to work with sparse matrices might need to implement in C
	max_size <- 10*max(size);
	size[size < 0] <- max_size;
	size[size < min_size] <- min_size;
#	size[size > max_size] <- max_size;

	return(list(var_obs=my_rowvar, sizes=size, vals=vals))
}

NBumiFitBasicModel <- function(counts) {
	vals <- hidden_calc_vals(counts);
	mus <- vals$tjs/vals$nc
	gm <- Matrix::rowMeans(counts)
	v <- Matrix::rowSums((counts-gm)^2)/(ncol(counts)-1)
	#v <- matrixStats::rowVars(counts)
	errs <- v < mus;
	v[errs] <- mus[errs]+10^-10;
	size <- mus^2/(v-mus)
	max_size <- max(mus)^2;
	size[errs] <- max_size;
	my_rowvar <- vector(length=nrow(counts));
	for (i in 1:nrow(counts)) {
		my_rowvar[i] <- var(counts[i,]-mus[i])
	}
	#mus_mat <- matrix(rep(mus, times=vals$nc), ncol=vals$nc, byrow=FALSE)
	return(list(var_obs=my_rowvar, sizes=size, vals=vals))
}

NBumiCheckFit <- function(counts, fit, suppress.plot=FALSE) {
	vals <- fit$vals;
	size_mat <- matrix(rep(fit$sizes, times=vals$nc), ncol=vals$nc, byrow=FALSE)
#	vs <- vector(length=nrow(counts));
	row_ps <- vector(length=nrow(counts))
	col_ps <- rep(0, times=ncol(counts))
	for (i in 1:nrow(counts)) {
		mu_is <- vals$tjs[i]*vals$tis/vals$total
		p_is <- (1+mu_is/fit$size[i])^(-fit$size[i]);
		row_ps[i] <- sum(p_is);
		col_ps <- col_ps+p_is;
#		vs[i] <- mean(mu_is+mu_is*mu_is/fit$size[i])
	}
#	thing <- fit$var_obs
#	plot(thing, vs, log="xy", xlab="Observed", ylab="Expected", main="Gene-specific Variance")
#	abline(a=0, b=1, col="red")

#	exp_ps <- (1+fit$mus/size_mat)^(-size_mat)
	if (!suppress.plot) {
		plot(vals$djs, row_ps, xlab="Observed", ylab="Fit", main="Gene-specific Dropouts")
		abline(a=0, b=1, col="red")

		plot(vals$dis, col_ps, xlab="Observed", ylab="Expected", main="Cell-specific Dropouts")
		abline(a=0, b=1, col="red")
	}

	invisible(list(gene_error = sum((vals$djs-row_ps)^2), cell_error = sum((vals$dis-col_ps)^2), rowPs=row_ps, colPs=col_ps));
}

NBumiCheckFitFS <- function(counts, fit, suppress.plot=FALSE) {
	vals <- fit$vals;
	size_coeffs <- NBumiFitDispVsMean(fit, suppress.plot=TRUE) 
	smoothed_size <- exp( size_coeffs[1] + size_coeffs[2]*log(vals$tjs/vals$nc) )
#	size_mat <- matrix(rep(smoothed_size, times=vals$nc), ncol=vals$nc, byrow=FALSE)

# Matrix compatibility
#	vs <- vector(length=nrow(counts));
	row_ps <- vector(length=nrow(counts))
	col_ps <- rep(0, times=ncol(counts))
	for (i in 1:nrow(counts)) {
		mu_is <- vals$tjs[i]*vals$tis/vals$total
		p_is <- (1+mu_is/smoothed_size[i])^(-smoothed_size[i]);
		row_ps[i] <- sum(p_is);
		col_ps <- col_ps+p_is;
#		vs[i] <- mean(mu_is+mu_is*mu_is/fit$size[i])
	}
#	thing <- fit$var_obs;
#	plot(thing, vs, log="xy", xlab="Observed", ylab="Expected", main="Gene-specific Variance")
#	abline(a=0, b=1, col="red")

#	exp_ps <- (1+fit$mus/size_mat)^(-size_mat)
	if (!suppress.plot) {
		plot(vals$djs, row_ps, xlab="Observed", ylab="Fit", main="Gene-specific Dropouts")
		abline(a=0, b=1, col="red")

		plot(vals$dis, col_ps, xlab="Observed", ylab="Expected", main="Cell-specific Dropouts")
		abline(a=0, b=1, col="red")
	}

	invisible(list(gene_error = sum((vals$djs-row_ps)^2), cell_error = sum((vals$dis-col_ps)^2), rowPs=row_ps, colPs=col_ps));
}


NBumiCompareModels <- function(counts, size_factor=(Matrix::colSums(counts)/median(Matrix::colSums(counts)))) {
	if (max(counts) < max(size_factor)) {
		stop("Error: size factors are too large");
	}
	norm <- NBumiConvertToInteger(t(t(counts)/size_factor));
	counts <- counts[rownames(counts) %in% rownames(norm),]; # necessary for plotting
	fit_adjust <- NBumiFitModel(counts);
	fit_basic <- NBumiFitBasicModel(norm);
	check_adjust <- NBumiCheckFitFS(counts, fit_adjust, suppress.plot=TRUE)
	check_basic <- NBumiCheckFitFS(norm, fit_basic, suppress.plot=TRUE)
	nc = fit_adjust$vals$nc

	plot( fit_adjust$vals$tjs/nc, fit_adjust$vals$djs/nc, col="black", pch=16, xlab="Expression", log="x", ylab= "Dropout Rate", cex=0.75)
#	points( fit_adjust$vals$tjs/nc, fit_basic$vals$djs/nc, col="black", pch=16, cex=0.75)
	points( fit_adjust$vals$tjs/nc, check_adjust$rowPs/nc, col="goldenrod1", pch=16, cex=0.5 )
	points( fit_adjust$vals$tjs/nc, check_basic$rowPs/nc, col="purple", pch=16, cex=0.5 )

	err_adj <- sum(abs(check_adjust$rowPs/nc-fit_adjust$vals$djs/nc))
#	err_bas <- sum(abs(Matrix::rowSums(check_basic$exp_ps)/nc-fit_basic$vals$djs/nc))
	err_bas <- sum(abs(check_basic$rowPs/nc-fit_adjust$vals$djs/nc))
	legend("bottomleft", paste(c("Depth-Adjusted\nError:", "Basic\nError:"), round(c(err_adj, err_bas)), c("\n","\n")), col=c("goldenrod1","purple"), pch=16, bty="n", cex=0.75)
	out <- c(err_adj, err_bas); names(out) <- c("Depth-Adjusted", "Basic");
	return(list(errors=out, basic_fit=fit_basic, adjusted_fit=fit_adjust))
}

obsolete__fit_size_to_drop <- function(obs, mu_vec, max_size, min_size=10^-10, convergence=0.001) {
	step_size <- 1;
        size_fit <- 1;
        last <- 0;
        last2 <- 0;
        for (iter in 1:1000) {
        	if (size_fit < min_size) {size_fit <- min_size;}
                diff <- sum( (1+mu_vec/size_fit)^(-size_fit) ) - obs;
                if (abs(diff) < convergence) {return(size_fit)}
                if (diff > 0) {
                	size_fit <- size_fit+step_size;
                        if (last < 0) {
                                step_size <- step_size/2;
                        } else {
                                if (last2 > 0){step_size <- step_size*2;}
                        }
                } else {
                        size_fit <- size_fit-step_size;
                        if (last > 0) {
                                step_size <- step_size/2;
                        } else {
                                if (last2 < 0){step_size <- step_size*2;}
                        }
                }
                last2 <- last; last <- diff;
                if (size_fit > max_size) {return(max_size);}
       }
       warning("Fitting size did not converge.");
       return(size_fit);
}

bg__fit_size_to_var <- function(obs, mu_vec, max_size, min_size=10^-10, convergence=0.001) {
	step_size <- 1;
        size_fit <- 1;
        last <- 0;
        last2 <- 0;
        for (iter in 1:1000) {
        	if (size_fit < min_size) {size_fit <- min_size;}
		expect <- mu_vec+mu_vec*mu_vec/size_fit;
                diff <- sum( expect )/(length(mu_vec)-1) - obs;
                if (abs(diff) < convergence) {return(size_fit)}
                if (diff > 0) {
                	size_fit <- size_fit+step_size;
                        if (last < 0) {
                                step_size <- step_size/2;
                        } else {
                                if (last2 > 0){step_size <- step_size*2;}
                        }
                } else {
                        size_fit <- size_fit-step_size;
                        if (last > 0) {
                                step_size <- step_size/2;
                        } else {
                                if (last2 < 0){step_size <- step_size*2;}
                        }
                }
                last2 <- last; last <- diff;
                if (size_fit > max_size) {return(max_size);}
       }
       warning("Fitting size did not converge.");
       return(size_fit);
}



#### Dispersion vs Mean & Feature Selection ####
NBumiFitDispVsMean <- function(fit, suppress.plot=TRUE) {
	vals <- fit$vals;
	size_g <- fit$sizes
	forfit <- fit$size < max(size_g) & vals$tjs > 0 & size_g > 0
	higher <- log(vals$tjs/vals$nc)/log(2) > 4; # As per-Grun et al.
	if (sum(higher == TRUE) > 2000) {
		forfit = higher & forfit;
	}
	reg <- lm( log(size_g[forfit])~ log((vals$tjs/vals$nc)[forfit]) )
	if (!suppress.plot) {
		plot(log( (vals$tjs/vals$nc)[forfit] ), log(size_g[forfit]), xlab="Log Mean Expression", ylab="Log Size" )
		abline(reg, col="red")
	}
	return(reg$coefficients)
}

hidden_shift_size <- function(mu_all, size_all, mu_group, coeffs) {
	b <- log(size_all)-coeffs[2]*log(mu_all)
	size_group <- exp(coeffs[2]*log(mu_group)+b)
	return(size_group)
}

obsolete__nbFeatureSelectionHighVarDist2Med <- function(fit, window_size=1000) {
	vals <- fit$vals;
	mean_order <- order(vals$tjs);
	obs_mean <- vals$tjs[mean_order]/vals$nc
	fit_disp <- fit$sizes[mean_order]
	names(fit_disp) <- names(vals$tjs)
	keep <- fit_disp < max(fit_disp)
	fit_disp <- log(fit_disp[keep])
	obs_mean <- obs_mean[keep]
	if (window_size %% 2 == 0) {
		flank <- window_size/2;
	} else {
		flank <- (window_size+1)/2;
	}
	dist_from_med <- function(x) {
		low <- max(1, x-flank);
		high <- min(length(fit_disp), x+flank);
		return(fit_disp[x]-median(fit_disp[seq(from=low, to=high, by=1)]))
	}
	score <- sapply(1:length(fit_disp), dist_from_med)
	return(sort(score))
}

NBumiFeatureSelectionHighVar <- function(fit) {
	# Global mean-variance
	vals <- fit$vals;
	coeffs <- NBumiFitDispVsMean(fit, suppress.plot=TRUE);
	exp_size <- exp( coeffs[1] + coeffs[2]*log(vals$tjs/vals$nc) )
	res <- log(fit$size) - log(exp_size);
	return(sort(res))
}

obsolete__nbFeatureSelectionDropouts <- function(fit) {
	# Gene-specific variance, mean
	vals <- fit$vals;
	size_mat <- matrix(rep(fit$sizes, times=vals$nc), ncol=vals$nc, byrow=F)

	droprate_exp <- vector(length=vals$ng)
	droprate_exp_err <- vector(length=vals$ng)
	for (i in 1:vals$ng) {
		mu_is <- vals$tjs[i]*vals$tis/vals$total
		p_is <- (1+mu_is/vals$size[i])^(-vals$size[i]);
		p_var_is <- p_is*(1-p_is);
		droprate_exp[i] <- sum(p_is)/vals$nc;
		droprate_exp_err[i] <- sqrt(sum(p_var_is)/(vals$nc^2));
	}
#	Exp_p <- (1+fit$mus/size_mat)^(-size_mat)
#	Exp_p_var <- Exp_p*(1-Exp_p)

#	droprate_exp <- Matrix::rowSums(Exp_p)/vals$nc
	droprate_exp[droprate_exp < 1/vals$nc] <- 1/vals$nc # Is this necessary?
#	droprate_exp_err <- sqrt(Matrix::rowSums(Exp_p_var)/(vals$nc^2))
	droprate_obs <- vals$djs/vals$nc
	droprate_obs_err <- sqrt(droprate_obs*(1-droprate_obs)/vals$nc)

	diff <- droprate_obs-droprate_exp
	combined_err <- droprate_exp_err
	Zed <- diff/combined_err
        pvalue <- pnorm(Zed, lower.tail = FALSE)
	names(pvalue) <- names(vals$tjs)
	return(sort(pvalue))
}

NBumiFeatureSelectionCombinedDrop <- function(fit, ntop=NULL, method="fdr", qval.thresh=2, suppress.plot=TRUE) {
	# Global mean-variance, gene-specific mean
	vals <- fit$vals;

	coeffs <- NBumiFitDispVsMean(fit, suppress.plot=TRUE);
	exp_size <- exp( coeffs[1] + coeffs[2]*log(vals$tjs/vals$nc) )

	#size_mat <- matrix(rep(exp_size, times=vals$nc), ncol=vals$nc, byrow=F)
	#Exp_p <- (1+fit$mus/size_mat)^(-size_mat)
	#Exp_p_var <- Exp_p*(1-Exp_p)

	droprate_exp <- vector(length=vals$ng)
	droprate_exp_err <- vector(length=vals$ng)
	for (i in 1:vals$ng) {
		mu_is <- vals$tjs[i]*vals$tis/vals$total
		p_is <- (1+mu_is/exp_size[i])^(-exp_size[i]);
		p_var_is <- p_is*(1-p_is);
		droprate_exp[i] <- sum(p_is)/vals$nc;
		droprate_exp_err[i] <- sqrt(sum(p_var_is)/(vals$nc^2));
	}

	#droprate_exp <- Matrix::rowSums(Exp_p)/vals$nc
	droprate_exp[droprate_exp < 1/vals$nc] <- 1/vals$nc # Is this necessary?
	#droprate_exp_err <- sqrt(Matrix::rowSums(Exp_p_var)/(vals$nc^2))
	droprate_obs <- vals$djs/vals$nc
	droprate_obs_err <- sqrt(droprate_obs*(1-droprate_obs)/vals$nc)

	diff <- droprate_obs-droprate_exp
	combined_err <- sqrt(droprate_exp_err^2+droprate_obs_err^2)
	#alt_err <- sd(diff)/sqrt(vals$nc);
	Zed <- diff/combined_err
        pvalue <- pnorm(Zed, lower.tail = FALSE)
	names(pvalue) <- names(vals$tjs)
	reorder <- order(pvalue, droprate_exp-droprate_obs) # deal with ties (e.g. lots of zero p-values)

	out <- pvalue[reorder]
	diff <- diff[reorder]
	qval <- p.adjust(out, method=method)
	if (is.null(ntop)) {
		out <- out[qval < qval.thresh]
		diff <- diff[qval < qval.thresh]
		qval <- qval[qval < qval.thresh]
	} else {
		out <- out[1:ntop]
		diff <- diff[1:ntop]
		qval <- qval[1:ntop]
	}
	outTABLE <- data.frame(Gene=names(out), effect_size=diff, p.value=out, q.value=qval)

	if (!suppress.plot) {
		xes <- log10(vals$tjs/vals$nc);
        	dens.col <- densCols(xes, droprate_obs, colramp=colorRampPalette(c("grey75","black")))
		plot(xes, droprate_obs, col=dens.col, pch=16, xlab="", ylab="")
		title(ylab="Dropout Rate", line=2)
		title(xlab="log10(expression)", line=2)
		toplot = names(out)
		toplot = names(vals$tjs) %in% toplot
		points(xes[toplot], droprate_obs[toplot], col="darkorange", pch=16)
		points(xes, droprate_exp, col="dodgerblue", pch=16, cex=1)
	}
	return(outTABLE)
}

PoissonUMIFeatureSelectionDropouts <- function(fit) {
	# Poisson distribution
	vals <- fit$vals
	#Exp_p <- exp(-fit$mus)
	#Exp_p_var <- Exp_p*(1-Exp_p)
	droprate_exp <- vector(length=vals$ng)
	droprate_exp_err <- vector(length=vals$ng)
	for (i in 1:vals$ng) {
		mu_is <- vals$tjs[i]*vals$tis/vals$total
		p_is <- exp(-mu_is);
		p_var_is <- p_is*(1-p_is);
		droprate_exp[i] <- sum(p_is)/vals$nc;
		droprate_exp_err[i] <- sqrt(sum(p_var_is)/(vals$nc^2));
	}

	#droprate_exp <- Matrix::rowSums(Exp_p)/vals$nc
	droprate_exp[droprate_exp < 1/vals$nc] <- 1/vals$nc # Is this necessary?
	#droprate_exp_err <- sqrt(Matrix::rowSums(Exp_p_var)/(vals$nc^2))
	droprate_obs <- vals$djs/vals$nc
	droprate_obs_err <- sqrt(droprate_obs*(1-droprate_obs)/vals$nc)

	diff <- droprate_obs-droprate_exp
	combined_err <- droprate_exp_err
	Zed <- diff/combined_err
        pvalue <- pnorm(Zed, lower.tail = FALSE)
	names(pvalue) <- names(vals$tjs)
	return(sort(pvalue))
}

#PoissonUMIFeatureSelectionCV <- function(counts, fit) {
#	# Poisson distribution
#	vals <- fit$vals
#	g_means <- vals$tjs/vals$nc
#	CVexp <- 1/sqrt(g_means) # expected
#	CVobs <-  sqrt(apply(counts-fit$mus, 1, var))/g_means;
#
#	CVexp_log <- log(CVexp)
#	CVobs_log <- log(CVobs)
#	mean_log <- log(g_means)
#
#	res <- CVobs_log-CVexp_log;
#	res_sd <- sd(res);
#
#	# Variance of sample variance from http://mathworld.wolfram.com/SampleVarianceDistribution.html
#	# Kenney and Keeping 1951, p. 164; Rose and Smith 2002, p. 264
#	u4 <- g_means*(1+3*g_means) # central moment
#	u2 <- g_means
#	N <- vals$nc
#	V <- (N-1)^2/N^3*u4-(N-1)*(N-3)/N^3*u2^2
#
#	Zed <- (CVobs-CVexp)/sqrt(V)
#        pvalue <- pnorm(Zed, lower.tail = FALSE)
#	names(pvalue) <- names(vals$tjs)
#	return(sort(pvalue))
#}
#### Differential Expression #####

# HERE HERE HERE - editing for Matrix sparse-matrix compatibility #

unfinished__nbGroupDE <- function(counts, fit, groups) {
	# Global mean-variance, gene-specific variance & mean
	vals <- fit$vals;
	size_g <- fit$sizes
	group_specific_factor <- aggregate(t(counts), by=list(groups), sum)
	rownames(group_specific_factor) <- group_specific_factor[,1]
	group_specific_factor <- group_specific_factor[,-1]
	group_specific_factor <- t(group_specific_factor)
	group_specific_tjs <- group_specific_factor; 
	group_total <- Matrix::colSums(group_specific_tjs)
	group_specific_factor <- t(t(group_specific_tjs)/group_total) / (vals$tjs/vals$total) # Relative expression level in group vs across whole dataset.
	group_specific_factor[vals$tjs == 0,] <- rep(1, length(group_specific_factor[1,]))

	coeffs <- NBumiFitDispVsMean(fit, suppress.plot=TRUE);

	pvals <- vector(length=vals$ng)
	for (j in 1:vals$ng) {
		prob_null <- 0;
		prob_test <- 0;
		for (i in 1:vals$nc) {
			group <- which(colnames(group_specific_factor) == groups[i])
			mu_ji <- vals$tjs[j]*vals$tis[i]/vals$total
			p1 <- dnbinom(counts[j,i], mu=mu_ji, size=size_g[j], log=TRUE)
			group_mu <- mu_ji*group_specific_factor[j,group]
			shifted_size <- hidden_shift_size(mu_ji,size_g[j],group_mu,coeffs);
			p2 <- dnbinom(counts[j,i], mu=group_mu, size=shifted_size, log=TRUE)
			# Collect ps across i
			prob_null <- prob_null+p1
			prob_test <- prob_test+p2
		}
		# Perform log-likehood test for this gene
		D <- -2*(prob_null-prob_test)
		df <- length(unique(groups))-1
		pval <- pchisq(D, df=df, lower.tail=FALSE)
		pvals[j] <- pval;
	}
	# Format output.
	output <- cbind(group_specific_factor, pvals, p.adjust(pvals, method="fdr"));
	rownames(output) <- rownames(counts);
	colnames(output) <- c(colnames(group_specific_factor), "p.value","q.value")
	return(output);
}

hidden_nbImputeZeros <- function(counts, fit) {
	vals <- fit$vals
	for (i in 1:nrow(counts)) {
		mu_is <- vals$tjs[i]*vals$tis/vals$total
		low <- counts[i,] < mu_is
		counts[i,low] <- counts[i,low]+mu_is[low]
	}
	return(counts);
}

NBumiImputeNorm <- function(counts, fit, total_counts_per_cell=median(fit$vals$tis)) {
	# find p-value for current fit NB Umi
	# adjust parameters of the NB
	# determine value of equivalent p-value under new NB
	coeffs <- NBumiFitDispVsMean(fit, suppress.plot=TRUE);
	vals <- fit$vals;
	norm <- counts;
	normed_ti <- total_counts_per_cell;
	normed_mus <- vals$tjs/vals$total;
	# TODO modify so adjusts size to new mean expression level
	#normed_size <- fit$size;
	for (i in 1:nrow(counts)) {
		mu_is <- vals$tjs[i]*vals$tis/vals$total;
		p_orig <- pnbinom(counts[i,], mu=mu_is, size=fit$sizes[i]);
		new_size <- hidden_shift_size(mean(mu_is), fit$sizes[i], normed_mus[i]*normed_ti, coeffs)
		normed <- qnbinom(p_orig, mu=normed_mus[i]*normed_ti, size=new_size);
		norm[i,] <- normed;
	}
	return(norm);
}


NBumiConvertData <- function(input, is.log=FALSE, is.counts=FALSE, pseudocount=1) {
	type <- class(input)[1]
	lognorm <- NULL
	counts <- NULL
	if (type == "SingleCellExperiment") {
		# New scater
		c <- which(names(assays(input)) == "counts")
		ln <- which(names(assays(input)) == "logcounts")
		if (length(c) > 0) {
			counts <- assays(input)[[c]]
		} else if (length(ln) > 0) {
			lognorm <- assays(input)[[ln]]
		} else {
			stop("Error: Recognized SingleCellExperiment object but cannot find either counts or lognorm expression.")
		}
	} else if (type == "ExpressionSet") {
		# monocle
		if (is.log) {
			lognorm <- Biobase::exprs(input)
		} else {
			counts <- Biobase::exprs(input)
		}
	} else if (type == "seurat") {
		# Seurat
		counts <- input@raw.data
	} else if (type == "matrix" | 
		   type == "data.frame" | 
		   type == "dgCMatrix" | 
		   type == "data.table" |
		   type == "DataTable" |
		   type == "DataFrame" |
		   type == "AnnotatedDataFrame") {
		if (type != "dgCMatrix") {
			input <- as.matrix(input)
		}

		if (is.log) {
			lognorm <- input;
		} else if (is.counts) {
			counts <- input
		} else {
			norm <- input;
		}
	} else {
		stop(paste("Error: Unrecognized input format :", type))
	}

	# Prefer raw counts to lognorm
	remove_undetected_genes <- function(mat) {
		no_detect <- Matrix::rowSums(mat > 0, na.rm=T) == 0;
		print(paste("Removing ",sum(no_detect), "undetected genes."))
		return(mat[!no_detect,])
	}
	# CPM transform raw counts
	if (!is.null(dim(counts))) {
		counts <- ceiling(counts)
		counts <- remove_undetected_genes(counts);
		return( counts )
	}
	# If normalized rescale by number of detected genes
	if (!is.null(dim(lognorm))) {
		norm <- 2^lognorm-pseudocount
	}
	if (!is.null(dim(norm))) {
		sf <- apply(norm, 2, function(x){min(x[x>0])}) #colSums(norm)
		sf <- 1/sf
		#detected <- colSums(norm > 0);
		#detected <- detected/median(detected);
		#libsize <- detected*median(sf)
		counts <- t( t(norm)/sf) #*libsize) )
		counts <- ceiling(counts)
		counts <- remove_undetected_genes(counts);
		return(counts)
	} 
}

NBumiHVG <- function(counts, fit, fdr_thresh=0.05, suppress.plot=FALSE, method=c("DANB", "basic")) {
	# v of v : https://math.stackexchange.com/questions/72975/variance-of-sample-variance
	# Moments of NB: http://mathworld.wolfram.com/NegativeBinomialDistribution.html

	n <- ncol(counts);
	
	# DANB model
	if (method[1] == "DANB") {
		#fit <- NBumiFitModel(counts);
		mu_obs <- fit$vals$tjs/n
		v_obs <- mu_obs + mu_obs^2/fit$size
	} else {
		mu_obs <- Matrix::rowMeans(counts)
		v_obs <- Matrix::rowSums((counts-mu_obs)^2)/(n-1)
	}
	
	# var = mu + disp * mu^2

	tmp <- mu_obs^2
	disp <- glm((v_obs-mu_obs)~tmp+0)$coef[1]
	v_fitted <- mu_obs+disp*mu_obs^2

	p <- mu_obs/v_fitted
	r <- mu_obs*p/(1-p)	


	mu4 <- r*(1-p)*(6-6*p+p^2+3*r-3*p*r)/(p^4)
	sigma2 <- r*(1-p)/(p^2)

	#v_of_v <- mu4/n - (sigma2^2*(n-3)/(n*(n-1)) #https://math.stackexchange.com/questions/72975/variance-of-sample-variance

	v_of_v <- mu4*(n-1)^2/n^3 - (sigma2^2*(n-3)*(n-1))/(n^3) #http://mathworld.wolfram.com/SampleVarianceDistribution.html

	z <- (v_obs - sigma2)/sqrt(v_of_v)
	p <- pnorm(z, lower.tail=FALSE)
	q <- p.adjust(p, method="fdr")
        eff <- v_obs-sigma2
	tab <- data.frame(rownames(counts), eff, p, q)
	colnames(tab) <- c("Gene", "effect.size", "p.value", "q.value")
	tab <- tab[!is.na(tab$p.value),]
	tab <- tab[order(-tab$q.value, tab$effect.size, decreasing=TRUE),]
	if (!suppress.plot) {
		plot(mu_obs, v_obs, cex=0.75, pch=16, xlab="mean", ylab="variance",log="xy")
		points(mu_obs[q < fdr_thresh], v_obs[q < fdr_thresh], col="red", pch=16)
		# Lines
		reorder <- order(mu_obs)
		lines(mu_obs[reorder], sigma2[reorder], col="grey80", lwd=2, lty=1)
		lines(mu_obs[reorder], sigma2[reorder]+sqrt(v_of_v[reorder])*qnorm(fdr_thresh, lower.tail=FALSE), col="grey80", lwd=2, lty=2)
	}
	return(tab[tab$q.value < fdr_thresh,])
}

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

# By: Tallulah Andrews, Senior Ph.D Student, 
# MRC Functional Genomics Unit, University of Oxford.
#
# Using three steps takes three sets and creates a labelled three-way Venn Diagram with areas approximately proportional to the counts.
# steps:
# table = ConvertSetstoTable(set1, set2, set3, names)
# circles = plot.venn.diagram(table)
# PlaceLabels(circles, table, 1/0) -> last argument indicates whether to place set names within their circle

#Based on Original Code from: http://tolstoy.newcastle.edu.au/R/help/03a/1115.html
# By: David J. States, M.D., PhD. Professor of Human Genetics, Director of Bioinformatics, University of Mishigan School of Medicine, Ann Arbor, MI, USA
# Date Downloaded: 10 Apr 2014
# Areas of circles and areas of pairwise overlaps proportional to counts, hopes for the best for the 3-way overlap

venn.overlap <-
function(r, a, b, target = 0)
{
#
# calculate the overlap area for circles of radius a and b
# with centers separated by r
# target is included for the root finding code
#
        pi <- acos(-1)
        if(r >= a + b) {
                return( - target)
        }
        if(r < a - b) { #b completely overlapped by a
                return(pi * b * b - target)
        }
        if(r < b - a) { #a complely overlapped by b
                return(pi * a * a - target)
        }
        s <- (a + b + r)/2
        triangle.area <- sqrt(s * (s - a) * (s - b) * (s - r))
        h <- (2 * triangle.area)/r
        aa <- 2 * atan(sqrt(((s - r) * (s - a))/(s * (s - b))))
        ab <- 2 * atan(sqrt(((s - r) * (s - b))/(s * (s - a))))
        sector.area <- aa * (a * a) + ab * (b * b)
        overlap <- sector.area - 2 * triangle.area
        return(overlap - target)
} 

plot.venn.diagram <-
function(d)
{
#
# Draw Venn diagrams with proportional overlaps
# d$table = 3 way table of overlaps
# d$labels = array of character string to use as labels
#
pi <- acos(-1)
csz <- 0.1
# Normalize the data
n <- length(dim(d$table))
c1 <- vector(length = n)
c1[1] <- sum(d$table[2, , ])
c1[2] <- sum(d$table[, 2, ])
c1[3] <- sum(d$table[, , 2])
n1 <- c1
#
c2 <- matrix(nrow = n, ncol = n, 0)
c2[1, 2] <- sum(d$table[2, 2, ])
c2[2, 1] <- c2[1, 2]
c2[1, 3] <- sum(d$table[2, , 2])
c2[3, 1] <- c2[1, 3]
c2[2, 3] <- sum(d$table[, 2, 2])
c2[3, 2] <- c2[2, 3]
n2 <- c2
#
c3 <- d$table[2, 2, 2]
n3 <- c3
c2 <- c2/sum(c1)
c3 <- c3/sum(c1)
c1 <- c1/sum(c1)
n <- length(c1)
# Radii are set so the area is proporitional to number of counts
pi <- acos(-1)
r <- sqrt(c1/pi)
r12 <- uniroot(venn.overlap, interval = c(max(r[1] - r[2], r[2] - r[1],
0) + 0.01, r[1] + r[2] - 0.01), a = r[1], b = r[
2], target = c2[1, 2])$root
r13 <- uniroot(venn.overlap, interval = c(max(r[1] - r[3], r[3] - r[1],
0) + 0.01, r[1] + r[3] - 0.01), a = r[1], b = r[
3], target = c2[1, 3])$root
r23 <- uniroot(venn.overlap, interval = c(max(r[2] - r[3], r[3] - r[2],
0) + 0.01, r[2] + r[3] - 0.01), a = r[2], b = r[
3], target = c2[2, 3])$root
s <- (r12 + r13 + r23)/2
x <- vector()
y <- vector()
x[1] <- 0
y[1] <- 0
x[2] <- r12
y[2] <- 0
angle <- 2 * atan(sqrt(((s - r12) * (s - r13))/(s * (s - r13))))
x[3] <- r13 * cos(angle)
y[3] <- r13 * sin(angle)
xc <- cos(seq(from = 0, to = 2 * pi, by = 0.01)) #Resolution of Circles
yc <- sin(seq(from = 0, to = 2 * pi, by = 0.01)) #Resolution of Circles
cmx <- sum(x * c1)
cmy <- sum(y * c1)
x <- x - cmx
y <- y - cmy
rp<-sqrt(x*x + y*y)
frame()
par(usr = c(-1, 1, -1, 1), pty = "s")
box()
for(i in 1:3) { #Draw Circles
lines(xc * r[i] + x[i], yc * r[i] + y[i])
}
return(list(r = r, x = x, y = y))
} 

# Calculate intersection to robustly place labels in appropriate locations
circle_intersection <- function (x1,y1,r1,x2,y2,r2) {
	if (x1 == x2) {
		y <- (r2*r2-r1*r1+x1*x1-x2*x2+y1*y1-y2*y2)/(2*y1-2*y2);
		a <- 1; b <- 2*x1; c <- (y-y1)*(y-y1)-r1*r1+x1*x1;
		xi1 <- (-b+sqrt(b*b-4*a*c))/(2*a)
                xi2 <- (-b-sqrt(b*b-4*a*c))/(2*a)
 		return(rbind(c(xi1,y),c(xi2,y)));
	} else {
		A <- (r2*r2-r1*r1+x1*x1-x2*x2+y1*y1-y2*y2)/(2*x1-2*x2)
		B <- (2*y2-2*y1)/(2*x1-2*x2)

		a <- (B*B)+1; b <- -2*B*x1+2*A*B-2*y1; c <- A*A-2*x1*A+x1*x1+y1*y1-r1*r1;
		yi1 <- (-b+sqrt(b*b-4*a*c))/(2*a)
		yi2 <- (-b-sqrt(b*b-4*a*c))/(2*a)
		xi1 <- A+B*yi1
		xi2 <- A+B*yi2
		return(rbind(c(xi1,yi1),c(xi2,yi2)));
	}
}


circle_line_intersection<-function (xc,yc,r,x1,y1,x2,y2) {
	if (x2 == x1) {
		xi <- x1;
		a <- 1; b <- -2*yc; c <- (xi-xc)*(xi-xc)+yc*yc-r*r;
	        yi1 <- (-b+sqrt(b*b-4*a*c))/(2*a)
	        yi2 <- (-b-sqrt(b*b-4*a*c))/(2*a)
		return(rbind(c(xi, yi1), c(xi, yi2)))
	} else {
		M <- (y2-y1)/(x2-x1);
		B <- y1-M*x1;
		a <- M*M+1; b <- 2*M*(B-yc)-2*xc; c <- xc*xc+(B-yc)*(B-yc)-r*r;
	        xi1 <- (-b+sqrt(b*b-4*a*c))/(2*a)
	        xi2 <- (-b-sqrt(b*b-4*a*c))/(2*a)
		yi1 <- M*xi1+B
		yi2 <- M*xi2+B
		return(rbind(c(xi1,yi1),c(xi2,yi2)));
	}
}


#By: Tallulah Andrews
#Date: 4th April 2014
# Places labels in each section of the venn diagram based on the interection points of the three circles

PlaceLabels <- function(circles, d, labelsincircles) {
  circles$left <- circles$x-circles$r
  circles$right <- circles$x+circles$r
  circles$top <- circles$y+circles$r
  circles$bottom <- circles$y-circles$r


  # Better labelling - Does not cope with one circle completely within another circle
  topcircle <- which(circles$y == max(circles$y))
  rightcircle <- which(circles$x == max(circles$x))
  leftcircle <- which(circles$x == min(circles$x))

  # Names for each Circle
  # Outside most distant edge of the circle
  if (!labelsincircles) {
	  text(circles$x[topcircle],circles$top[topcircle], d$labels[topcircle],pos=3)
	  text(circles$right[rightcircle],circles$y[rightcircle], d$labels[rightcircle],pos=4)
	  text(circles$left[leftcircle],circles$y[leftcircle], d$labels[leftcircle],pos=2)
  }

  # Intersection Points
  intersectionLR <- circle_intersection(circles$x[leftcircle],circles$y[leftcircle],circles$r[leftcircle],circles$x[rightcircle],circles$y[rightcircle],circles$r[rightcircle])
  intersectionLT <- circle_intersection(circles$x[leftcircle],circles$y[leftcircle],circles$r[leftcircle],circles$x[topcircle],circles$y[topcircle],circles$r[topcircle])
  intersectionTR <- circle_intersection(circles$x[topcircle],circles$y[topcircle],circles$r[topcircle],circles$x[rightcircle],circles$y[rightcircle],circles$r[rightcircle])


  onewayvalues <- c(d$table[2, 1, 1], d$table[1, 2, 1], d$table[1, 1, 2]) #1, 2, 3
  twowayvalues <- c(d$table[2,2,1],d$table[2,1,2],d$table[1,2,2]) #1&2, 1&3, 2&3 ->ceiling(a*b/2)
  threewayvalue <- d$table[2,2,2];

  # Names in circles above their one-way value
  if (labelsincircles) {onewayvalues <- paste(d$labels, onewayvalues, sep="\n");}

  # 3-way intersection
  topLRintersection <- intersectionLR[which(intersectionLR[,2] == max(intersectionLR[,2])),];
  leftTRintersection <- intersectionTR[which(intersectionTR[,1] == min(intersectionTR[,1])),];
  rightTLintersection <- intersectionLT[which(intersectionLT[,1] == max(intersectionLT[,1])),];
  boundarypoints <- rbind(topLRintersection, leftTRintersection, rightTLintersection) #R of LT intersection
  midpoint <- colMeans(boundarypoints); text(midpoint[1],midpoint[2], threewayvalue)


  #2-way intersection
  # LR - take advantage of fact that L & R circles has centres at the same height/
  textx <- mean(intersectionLR[,1]); # mid-point betweed the intersection points
  lowestT <- min(leftTRintersection[2],rightTLintersection[2], topLRintersection[2]);
  if (circles$x[topcircle] > rightTLintersection[1] & circles$x[topcircle] < leftTRintersection[1]) {lowestT <- min(circles$bottom[topcircle], lowestT);}
  texty <- mean(c(min(intersectionLR[,2]), lowestT))
  text(textx, texty, twowayvalues[ceiling(leftcircle*rightcircle/2)])

  # LT
  intersectpoint <- circle_line_intersection(circles$x[rightcircle], circles$y[rightcircle], circles$r[rightcircle], rightTLintersection[1], rightTLintersection[2], intersectionLT[which(intersectionLT[,1] != max(intersectionLT[,1])),1], intersectionLT[which(intersectionLT[,1] != max(intersectionLT[,1])),2])

# if intersect points are outside of the area wedge use the inner circle-pairwise intersection instead
  midpointv <- rbind(intersectionLT[which(intersectionLT[,1] == min(intersectionLT[,1])),], intersectpoint[which(intersectpoint[,1] == min(intersectpoint[,1])),]);
  if (midpointv[2,1] > rightTLintersection[1]) {midpointv[2,] <- rightTLintersection;}
  midpoint <- colMeans(midpointv); 
  text(midpoint[1], midpoint[2], twowayvalues[ceiling(leftcircle*topcircle/2)])

  # RT
  intersectpoint <- circle_line_intersection(circles$x[leftcircle], circles$y[leftcircle], circles$r[leftcircle], leftTRintersection[1], leftTRintersection[2], intersectionTR[which(intersectionTR[,1] !=  min(intersectionTR[,1])),1], intersectionTR[which(intersectionTR[,1] !=  min(intersectionTR[,1])),2])
  midpointv <- rbind(intersectionTR[which(intersectionTR[,1] !=  min(intersectionTR[,1])),], intersectpoint[which(intersectpoint[,1] == max(intersectpoint[,1])),]);
  if (midpointv[2,1] < leftTRintersection[1]) {midpointv[2,] <- leftTRintersection;}
  midpoint <- colMeans(midpointv);
  text(midpoint[1], midpoint[2], twowayvalues[ceiling(rightcircle*topcircle/2)])
  #1-way intersection
  #L 
#if distance from  leftTRintersection to the centre is smaller than the radius. else put in centre
  if (dist(rbind(leftTRintersection,c(circles$x[leftcircle], circles$y[leftcircle])), method="euclidean") < circles$r[leftcircle]) {
	  intersectpoint <- circle_line_intersection(circles$x[leftcircle], circles$y[leftcircle], circles$r[leftcircle], intersectionTR[which(intersectionTR[,1] != leftTRintersection[1]),1], intersectionTR[which(intersectionTR[,1] != leftTRintersection[1]),2], leftTRintersection[1], leftTRintersection[2])
	  midpoint <- colMeans(rbind(leftTRintersection, intersectpoint[which(intersectpoint[,1] == min(intersectpoint[,1])),]))
	  text(midpoint[1], midpoint[2], onewayvalues[leftcircle])
  } else {
	  text(circles$x[leftcircle], circles$y[leftcircle],onewayvalues[leftcircle]);
  }
  #R 
#if distance from  rightTLintersection to the centre is smaller than the radius. else put in centre
  if (dist(rbind(rightTLintersection,c(circles$x[rightcircle], circles$y[rightcircle])), method="euclidean") < circles$r[rightcircle]) {
	  intersectpoint <- circle_line_intersection(circles$x[rightcircle], circles$y[rightcircle], circles$r[rightcircle], intersectionLT[which(intersectionLT[,1] != rightTLintersection[1]),1], intersectionLT[which(intersectionLT[,1] != rightTLintersection[1]),2] , rightTLintersection[1], rightTLintersection[2])
	  midpoint <- colMeans(rbind(rightTLintersection, intersectpoint[which(intersectpoint[,1] == max(intersectpoint[,1])),]))
	  text(midpoint[1], midpoint[2], onewayvalues[rightcircle])
  } else {
	text(circles$x[rightcircle], circles$y[rightcircle],onewayvalues[rightcircle]);
  }
  #T
#if distance from  topLRintersection to the centre is smaller than the radius. else put in centre
  if (dist(rbind(topLRintersection,c(circles$x[topcircle], circles$y[topcircle])), method="euclidean") < circles$r[topcircle]) {
  	intersectpoint <- circle_line_intersection(circles$x[topcircle], circles$y[topcircle], circles$r[topcircle], intersectionLR[which(intersectionLR[,2] != topLRintersection[2]),1], intersectionLR[which(intersectionLR[,2] != topLRintersection[2]),2], topLRintersection[1], topLRintersection[2])
  	midpoint <- colMeans(rbind(topLRintersection, intersectpoint[which(intersectpoint[,2] == max(intersectpoint[,2])),]))
  	text(midpoint[1], midpoint[2], onewayvalues[topcircle])
  } else {
	text(circles$x[topcircle], circles$y[topcircle],onewayvalues[topcircle]);
  }
}



# Based on code from: http://research.stowers-institute.org/efg/R/Math/VennDiagram.htm

ConvertSetstoTable <- function(set1, set2, set3, names)
{stopifnot( length(names) == 3)
  # Form universe as union of all three sets
  universe <- sort( unique( c(set1, set2, set3) ) )
  InOut <- matrix(0, nrow=length(universe), ncol=3) #each element in/out of each set
  colnames(InOut) <- names

  for (i in 1:length(universe)) {
    InOut[i,1] <- universe[i] %in% set1
    InOut[i,2] <- universe[i] %in% set2
    InOut[i,3] <- universe[i] %in% set3
  }
  d <-list()
  d$table<-table(InOut[,1],InOut[,2], InOut[,3])
  d$labels <- names;
  return(d)
}

# Top-level function
M3DropThreeSetVenn <- function(set1,set2,set3,names){
  table <- ConvertSetstoTable(as.character(set1), as.character(set2), as.character(set3), names=names)
  nicer_table <- table
  if (max(table$table) > 5*mean(table$table)) {
    bigones <- table$table > 5*mean(table$table);
    nicer_table$table[bigones] <- table$table[bigones]/5
  }
  if (sum(table$table == 0) > 1) {
    smallest_notzero <- min(table$table[table$table > 0])
    nicer_table$table[table$table == 0] <- smallest_notzero;
  }
  circles <- plot.venn.diagram(nicer_table);
  PlaceLabels(circles, table, labelsincircles=TRUE)
}

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

### Functions ###
hidden_add_dropouts <- function(x,mu,K){
	p_drop <- 1 - mu/(mu+K);
	expect_pos <- mu*length(x)/sum(x>0);
	p_drop <- K/expect_pos
#	p_new_drop = (p_drop*length(x)-sum(x==0))/sum(x > 0)
	toss <- runif(n=length(x));
	x[toss < p_drop] <- 0;
	return(x);
}

hidden_amplification1 <- function (x, rounds=12, efficiency=0.97) {
	tot = sum(x);
	if (tot == 0) {return(x)} # Nothing to be done if all zero already
	# Amplify
	for (i in 1:rounds) {
		x <- sapply(x, function(y) {y+rbinom(1,size=y, prob=efficiency)})
	}
	amped = sum(x)
	# Downsample back to starting amounts
	x <- sapply(x, function(z) {rbinom(1,size=z, prob=1/((1+efficiency)^rounds))})
	return(x);
}

bg__default_mean2disp <- function(mu, coeffs=c(3.967816,-1.855054)){
		cv2 <- exp(coeffs[1]+coeffs[2]*(log(mu)/log(10)))
		variance <- cv2*(mu^2)
		disp <- mu^2/(variance-mu)
		return(1/disp)
}

bg__MakeSimData <- function(dispersion_fun=bg__default_mean2disp, n_cells=300, dispersion_factor=1, base_means=10^rnorm(25000,1,1), K=10.3) {
	# Make Simulated Matrix
        n_genes <- length(base_means);
        expr_mat <- sapply(1:n_genes, function(x){
                    base <- rnbinom(n_cells, 
			size=1/(dispersion_factor*dispersion_fun(base_means[x])), 
			mu=base_means[x])
                    if (!is.null(K)) {
                         base <- hidden_add_dropouts(base,base_means[x],K)
                    }
                    return(base)
                    })
        expr_mat <- t(expr_mat);
        rownames(expr_mat) <- 1:length(expr_mat[,1])
        colnames(expr_mat) <- 1:length(expr_mat[1,])
	return(expr_mat);
}

bg__MakeSimDE <- function(dispersion_fun=bg__default_mean2disp, fold_change=10, frac_change=0.1, n_cells=300, sub_pop=0.5, dispersion_factor=1, base_means=10^rnorm(25000,1,1), K=10.3){
        n_genes <- length(base_means);
        TP <- sample(1:n_genes, frac_change*n_genes)
        sub_pop <- round(sub_pop*n_cells)
        Pop_lab <- c(rep(1, times=n_cells-sub_pop),rep(2,times=sub_pop))
	# Base Population
	base <- bg__MakeSimData(dispersion_fun=dispersion_fun, 
			n_cells=sum(Pop_lab==1), 
			dispersion_factor=dispersion_factor, 
			base_means=base_means, K=K)
	changed_means <- base_means;
	changed_means[TP] <- base_means[TP]*fold_change;
	# Changed Subpopulation
	sub_pop <- bg__MakeSimData(dispersion_fun=dispersion_fun, 
			n_cells=sum(Pop_lab==2), 
			dispersion_factor=dispersion_factor, 
			base_means=changed_means, K=K)
	return(list(data=cbind(base,sub_pop), cell_labels=Pop_lab, TP=TP));
}


bg__MakeSimDVar <- function(dispersion_fun=bg__default_mean2disp, fold_change=10, frac_change=0.1, n_cells=300, sub_pop=0.5, dispersion_factor=1, base_means=10^rnorm(25000, 1,1), K=10.3) {
	# Make Simulated Matrix
        n_genes <- length(base_means);
        TP <- sample(1:n_genes, frac_change*n_genes)
        sub_pop <- round(sub_pop*n_cells)
        Pop_lab <- c(rep(1, times=n_cells-sub_pop),rep(2,times=sub_pop))
	# Whole Population
	base <- bg__MakeSimData(dispersion_fun=dispersion_fun, 
			n_cells=n_cells, 
			dispersion_factor=dispersion_factor, 
			base_means=base_means, K=K)
	# Changed Vals
	subpop <- bg__MakeSimData(dispersion_fun=dispersion_fun, 
			n_cells=sum(Pop_lab==2), 
			dispersion_factor=fold_change*dispersion_factor, 
			base_means=base_means[TP], K=K)
	base[TP,Pop_lab==2] <- subpop
	return(list(data=base, cell_labels=Pop_lab, TP=TP));
}

bg__MakeSimHVar <- function(dispersion_fun=bg__default_mean2disp, fold_change=10, frac_change=0.1, n_cells=300, dispersion_factor=1, base_means=10^rnorm(25000, 1,1), K=10.3) {
        n_genes <- length(base_means);
        TP <- sample(1:n_genes, frac_change*n_genes)
        Pop_lab <- rep(1, times=n_cells)
	# Whole Population
	base <- bg__MakeSimData(dispersion_fun=dispersion_fun, 
			n_cells=n_cells, 
			dispersion_factor=dispersion_factor, 
			base_means=base_means, K=K)
	# Changed Vals
	subpop <- bg__MakeSimData(dispersion_fun=dispersion_fun, 
			n_cells=n_cells, 
			dispersion_factor=fold_change*dispersion_factor, 
			base_means=base_means[TP], K=K)
	base[TP,] <- subpop
	return(list(data=base, cell_labels=Pop_lab, TP=TP));
}

bg__get_stats <- function(sig, TP, ngenes) {
	TPs=sum(sig %in% TP)
	FPs=length(sig)-TPs
	FNs=length(TP)-TPs
	TNs=ngenes-length(TP)-FPs;
	if ((TPs+FPs) == 0) {
		FDR <- 0;
	} else {
		FDR <- FPs/(TPs+FPs);
	}
	if ((FNs+TPs) == 0) {
		FNR <- 0;
	} else {
		FNR <- FNs/(FNs+TPs)
	}
	return(c(FDR,FNR));
}

##### Update #####


obsolete__calc_DE_stats <- function(expr_mat, TP, Observed_Means, mt_threshold=0.05, suppress.plot=TRUE) {
	#require("M3Drop")
        # Test DE
        mt_method="fdr"
	M3Drop_col="black"
	HVG_col="forestgreen"

        expr_mat = expr_mat[Matrix::rowSums(expr_mat) > 0,]
        expr_mat = expr_mat[Matrix::rowSums(expr_mat == 0) > 0,]
        TP = TP[TP %in% rownames(expr_mat)]

        DE = M3DropFeatureSelection(expr_mat, mt_method = mt_method, mt_threshold=2, suppress.plot=TRUE)
        HVG = BrenneckeGetVariableGenes(expr_mat, fdr=2, suppress.plot=TRUE)

        output = rbind(c(0,0,0),c(0,0,0));
	colnames(output) = c("AUC","FDR","FNR")
	rownames(output) = c("M3Drop","HVG")
        tmp = list(DE, HVG)

        # Plot Stuff - setup #
        act_means = rowMeans(expr_mat)
        thing = log(act_means)/log(10)
        bins = 10^seq(from = floor(min(thing)), to = ceiling(max(thing)), by = 1)

	for (bf in 1:9) { # Shift bins to make curve smoother
                binned = as.numeric(cut(rowMeans(expr_mat), bins*bf))
                names(binned) = rownames(expr_mat)
                plot_output = bins[2:length(bins)]*bf

	        for (i in 1:2) { # Do same thing for HVG & M3Drop
	                Diff = tmp[[i]]
	                sig = Diff[Diff$q.value < mt_threshold,]


	                stats = bg__get_stats(sig[,1], TP, ngenes=length(expr_mat[,1]));

	                #require("ROCR")
	                Diff$Truth = rep(0, times=length(Diff[,1]))
	                Diff[Diff[,1] %in% TP,]$Truth = 1;
	                pred <- ROCR::prediction(1-Diff$p.value, Diff$Truth)
	                val <- unlist(ROCR::performance(pred,"auc")@y.values)
	                output[i,]=c(val,stats);

	                # Plot Stuff - data #
	                get_bin_stats <- function(b){
	                        this_bin = names(binned)[binned==b];
	                        this_sig = this_bin[this_bin %in% sig[,1]];
	                        this_TP = this_bin[this_bin %in% TP];
	                        out = bg__get_stats(this_sig, this_TP, ngenes = length(expr_mat[,1]));
	                        c(out, length(this_sig), length(this_TP))
	                }
	                bin_stats = sapply(1:(length(bins)-1), get_bin_stats)
	                #colnames(bin_stats) = bins[2:length(bins)]
	                rownames(bin_stats) = c("FDR","FNR","Ncalled","nTrue")
	                plot_output = rbind(plot_output, bin_stats)
	        }
                if (bf==1) {
                        final_plot_output <- plot_output;
                } else {
                        final_plot_output = cbind(final_plot_output, plot_output);
                }
        }

	final_plot_output[1,] = final_plot_output[1,]/2
	if (!suppress.plot) {
#                par(mar=c(3.5,3.5,4,1))
                background = density(log(Observed_Means)/log(10))
                drawing=list()
                drawing$ylim<-c(0,max(c(background$y)));
#                drawing$xlim<-c(min(background$x),max(background$x));
                drawing$xlim<-c(min(background$x),max(c(background$x,ceiling(log(final_plot_output[1,])/log(10)))));
                plot(1, col="white", xlim=drawing$xlim, ylim=drawing$ylim, xaxt="n", yaxt="n", xlab="", ylab=""); polygon(background, col="grey75", border="grey75")


                convert_yvals = function(y) { # convert 0-1 to appropriate scale for plot
                        y*drawing$ylim[2]

                }
                final_plot_output <- final_plot_output[,order(final_plot_output[1,])]


                xes = log(final_plot_output[1,])/log(10)

                ticks = seq(from=1, to =length(xes), by=9)
		ticks = final_plot_output[1,ticks]*2
#                axis(1, at=xes[ticks], labels = final_plot_output[1,ticks])
                axis(1, at=log(ticks)/log(10), labels = ticks)

                axis(2, at=convert_yvals(seq(from=0, to=1, by=0.1)), labels = seq(from=0, to=1, by=0.1))
                title(xlab="Gene Expression (CPM)", ylab="FDR/FNR", line=2)
                abline(h=convert_yvals(mt_threshold), col="red", lty=2, xpd=F)
                # FDR
                lines(xes,convert_yvals(final_plot_output[2,]), col=M3Drop_col, lty=1, lwd=3)
                lines(xes,convert_yvals(final_plot_output[6,]), col=HVG_col, lwd=3)

                # FNR
                lines(xes,convert_yvals(final_plot_output[3,]), col=M3Drop_col, lwd=3, lty=2)
                lines(xes,convert_yvals(final_plot_output[7,]), col=HVG_col, lwd=3, lty=2)

                par(xpd=T)
                legend("top", inset=c(0,-0.22), c(expression(bold("Method :")), expression(bold("Measure :")), "M3Drop", "FDR","HVG", "FNR"), lty=c(1,1,1,1,1,2), col=c("white","white",M3Drop_col,"black",HVG_col,"black"), lwd=2.5, ncol=3, bty="n")
		par(xpd=F);
        }
	return(list(summary=output, per_expr=final_plot_output))
}

hidden__calc_DE_stats_simplified <- function(expr_mat, TP, Observed_Means, mt_threshold=0.05, suppress.plot=TRUE) {
	# Just ge AUC, FDR, FNR
	#require("M3Drop")
        # Test DE
        mt_method="fdr"

        expr_mat = expr_mat[Matrix::rowSums(expr_mat) > 0,]
        expr_mat = expr_mat[Matrix::rowSums(expr_mat == 0) > 0,]
        TP = TP[TP %in% rownames(expr_mat)]

        DE = M3DropFeatureSelection(expr_mat, mt_method = mt_method, mt_threshold=2, suppress.plot=TRUE)
        HVG = BrenneckeGetVariableGenes(expr_mat, fdr=2, suppress.plot=T)

        output = rbind(c(0,0,0),c(0,0,0));
	colnames(output) = c("AUC","FDR","FNR")
	rownames(output) = c("M3Drop","HVG")
        tmp = list(DE, HVG)

        # Plot Stuff - setup #
        act_means = rowMeans(expr_mat)
        thing = log(act_means)/log(10)
        bins = 10^seq(from = floor(min(thing)), to = ceiling(max(thing)), by = 1)

        for (i in 1:2) { # Do same thing for HVG & M3Drop
                Diff = tmp[[i]]
                sig = Diff[Diff$q.value < mt_threshold,]


                stats = bg__get_stats(sig[,1], TP, ngenes=length(expr_mat[,1]));

		# Need to somehow downsample so distribution of gene expression =~ observed distribution of gene expression
		# or do this when define means? - latter makes more sense b/c do it once for all sim
                #require("ROCR")
                Diff$Truth = rep(0, times=length(Diff[,1]))
                Diff[Diff[,1] %in% TP,]$Truth = 1;
                pred <- ROCR::prediction(1-Diff$p.value, Diff$Truth)
                val <- unlist(ROCR::performance(pred,"auc")@y.values)
                output[i,]=c(val,stats);

        }
	return(list(summary=output, per_expr=NULL))
}

hidden__calc_DE_stats_simplified_singular <- function(expr_mat, TP, DE) {
	qvals <- DE$q.value
	if (!is.numeric(qvals)) {
		qvals <- as.numeric(qvals)
	}
	pvals <- DE$p.value
	if (!is.numeric(pvals)) {
		pvals <- as.numeric(pvals)
	}

        sig = DE[qvals < 0.05,]

        stats = bg__get_stats(sig[,1], TP, ngenes=length(expr_mat[,1]));

        #require("ROCR")
        DE$Truth = rep(0, times=length(DE[,1]))
        DE[DE[,1] %in% TP,]$Truth = 1;
        pred <- ROCR::prediction(1-pvals, DE$Truth)
        val <- unlist(ROCR::performance(pred,"auc")@y.values)
        output=c(val,stats);

	return(list(summary=output, per_expr=NULL))
}

bg__var_vs_drop <- function(pop_size, fixed_mean, K=10.3, dispersion_from_mean=bg__default_mean2disp, suppress.plot=TRUE) {
        fc = seq(from=1, to=100, by=1)
        labels = c(rep(1, times=pop_size),rep(2,times=pop_size))
        lowmean_fun <- function(fc) {2*fixed_mean/(1+fc)}
        test = sapply(fc, function(f) {
                low_mean = lowmean_fun(f)
                high_mean = low_mean*f
                base = rnbinom(pop_size, size=1/dispersion_from_mean(low_mean), mu=low_mean);
                subpop = rnbinom(pop_size, size=1/dispersion_from_mean(high_mean), mu=high_mean)
                base = hidden_add_dropouts(base,low_mean,K)
                subpop = hidden_add_dropouts(subpop,high_mean*f,K)
                return(c(base,subpop))
        })
        var_btw_fun <- function(x) {
                a = aov(x~labels)
                sse_btw =unlist(summary(a))[3]
                var_btw = sse_btw/(2*pop_size-1)
                return(var_btw)
        }
        var_within_fun <- function(x) {
                a = aov(x~labels)
                sse_within =unlist(summary(a))[4]
                var_within = sse_within/(2*pop_size-1)
                return(var_within)
        }
        Vbtw = apply(test,2,var_btw_fun)
        Vwithin = apply(test,2,var_within_fun)
        vars = apply(test, 2, var)
        drops = apply(test, 2, function(x){sum(x==0)})/length(test[,1])
        if (!suppress.plot) {
                par(mar=c(3.5,3.5,4,1))
                # Var plot
                plot(fc, vars, pch=16, xlab="",ylab="", ylim=c(0,max(vars)), type="l")
                title(xlab="Fold Change", ylab="Variance", line=2)
                title(main=paste("mu = ",fixed_mean,", n = ",2*pop_size,sep=""))
                lines(fc, Vbtw, pch=16, xlab="",ylab="", col="blue")
                lines(fc, Vwithin, pch=16, xlab="",ylab="", col="red")
                legend("topleft", c("Total", "Btw","Within"), lty=1, col=c("black","blue","red"), bty="n")
                # Drop plot
                plot(fc, drops, pch=16, xlab="",ylab="")
                title(xlab="Fold Change", ylab="Dropout Rate", line=2)
                title(main=paste("mu = ",fixed_mean,", n = ",2*pop_size,sep=""))
        }
        var_r = cor(vars,fc)
        drop_r = cor(drops,fc)
        return(list(var_r=var_r, drop_r=drop_r, vars=vars, drops=drops, fc=fc, Vbtw=Vbtw, Vwithin=Vwithin));
}

hidden__calc_DE_stats_fpr<- function(expr_mat, TP, Observed_Means, mt_threshold=0.05, suppress.plot=TRUE) {
	#require("M3Drop")
        # Test DE
        mt_method="fdr"

        expr_mat = expr_mat[Matrix::rowSums(expr_mat) > 0,]
        expr_mat = expr_mat[Matrix::rowSums(expr_mat == 0) > 0,]
        TP = TP[TP %in% rownames(expr_mat)]

        DE = M3DropFeatureSelection(expr_mat, mt_method = mt_method, mt_threshold=2, suppress.plot=TRUE)
        HVG = BrenneckeGetVariableGenes(expr_mat, fdr=2, suppress.plot=T)

        output = c(0,0);
        tmp = list(DE, HVG)

        # Plot Stuff - setup #
        act_means = rowMeans(expr_mat)
        thing = log(act_means)/log(10)
        bins = 10^seq(from = floor(min(thing)), to = ceiling(max(thing)), by = 1)

        for (i in 1:2) { # Do same thing for HVG & M3Drop
                Diff = tmp[[i]]
                sig = Diff[Diff$q.value < mt_threshold,]
		val <- 1-sum(sig[,1] %in% TP)/length(sig[,1])
                output[i]=val;
        }
	return(list(summary=output, per_expr=NULL))
}

bg__fit_gamma <- function(x) {
	s = var(x)/mean(x)
	a = mean(x)/s
	return(list(shape=a, scale=s))
}
bg__shift_size <- function(mu_all, size_all, mu_group, coeffs) {
        b <- log(size_all)-coeffs[2]*log(mu_all)
        size_group <- exp(coeffs[2]*log(mu_group)+b)
        return(size_group)
}

NBumiSimulationTrifecta <- function(original_data, n_genes=25000, n_cells=250, sub_pop_prop=0.5) {
	# 31 Aug 2017
	n_cells1 = round(n_cells*2*sub_pop_prop)
	n_cells2 = round(n_cells*2*(1-sub_pop_prop))
#	require("M3Drop")
	fit <- NBumiFitModel(original_data);
	Tis = fit$vals$tis
	Tis_gamma = bg__fit_gamma(Tis)
	Mjs = fit$vals$tjs/fit$vals$nc
	Mjs_norm = c(mean(log(Mjs)/log(10)), sd(log(Mjs)/log(10)))
	mean2disp_coeffs = NBumiFitDispVsMean(fit, suppress.plot=TRUE)
	min_mean = 10^-5;

	g_means = rnorm(n_genes, mean=Mjs_norm[1], sd=Mjs_norm[2]);
	g_means = 10^g_means
	g_means[g_means < min_mean] = min_mean;
	g_means[g_means > max(Mjs)] = max(Mjs)
	g_means1 = g_means*(n_cells1);
	g_means2 = g_means*(n_cells2);

	c_depths1 = round(rgamma(n_cells1, shape=Tis_gamma$shape, scale=Tis_gamma$scale));
	c_depths2 = round(rgamma(n_cells2, shape=Tis_gamma$shape, scale=Tis_gamma$scale));
	l2fc <- rnorm(n_genes, sd=2)

	mus1 <- ((g_means1) %*% t(c_depths1)/sum(c_depths1)) # Fix 11 May 2017
	mus2 <- ((g_means2) %*% t(c_depths2)/sum(c_depths2)) # Fix 11 May 2017
	disp_size <- exp(log(rowMeans(cbind(mus1,mus2)))*mean2disp_coeffs[2]+mean2disp_coeffs[1])

	base <- sapply(1:n_genes, function(i) {sapply(mus1[i,], function(m) {rnbinom(1, mu=m, size=disp_size[i])})})

	shifted_size = bg__shift_size(rowMeans(mus2), disp_size, rowMeans(mus2)*2^l2fc, mean2disp_coeffs)
	de <- sapply(1:n_genes, function(i) {sapply(mus2[i,], function(m) {rnbinom(1, mu=m*2^l2fc[i], size=shifted_size[i])})})

	dv <- sapply(1:n_genes, function(i) {sapply(mus2[i,], function(m) {rnbinom(1, mu=m, size=disp_size[i]*2^l2fc[i])})})
	hv <- sapply(1:n_genes, function(i) {sapply(mus2[i,], function(m) {rnbinom(1, mu=m, size=disp_size[i]*2^l2fc[i])})})

	return(list(truth=l2fc, groups=c(rep(1, times=n_cells1), rep(2, times=n_cells2)), de = cbind(t(base),t(de)), dv = cbind(t(base),t(dv)), hv = cbind(t(dv),t(hv))))

}
M3DropSimulationTrifecta <- function(original_data, n_genes=25000, n_cells=250, sub_pop_prop=0.5) {
	n_cells1 = round(n_cells*2*sub_pop_prop)
	n_cells2 = round(n_cells*2*(1-sub_pop_prop))
#	require("M3Drop")
	tis = Matrix::colSums(original_data)
	norm = t(t(original_data)/tis*median(tis))
	Mjs = rowMeans(norm);
	vals <- bg__calc_variables(norm)
	fit = bg__fit_MM(vals$p, vals$s)
	K = fit$K
	Mjs_norm = c(mean(log(Mjs)/log(10)), sd(log(Mjs)/log(10)))
	mean2disp <- bg__get_mean2disp(norm);
	min_mean = 10^-5;
	
#	g_means = rgamma(n_genes, shape=Mjs_gamma$shape, scale=Mjs_gamma$scale)
	g_means = rnorm(n_genes, mean=Mjs_norm[1], sd=Mjs_norm[2])
	g_means = 10^g_means
	g_means[g_means < min_mean] = min_mean
	g_means[g_means > max(Mjs)] = max(Mjs)
	l2fc <- rnorm(n_genes, sd=2)
	
	# Make datasets
	base <- sapply(g_means, function(mu) {hidden_add_dropouts(rnbinom(n_cells1, mu=mu, size=1/mean2disp(mu)), mu, K)})
	de <- sapply(g_means*2^l2fc, function(mu) {hidden_add_dropouts(rnbinom(n_cells2, mu=mu, size=1/mean2disp(mu)), mu, K)})
	dv <- sapply(1:n_genes, function(i) {
			mu = g_means[i];
			hidden_add_dropouts(rnbinom(n_cells2, mu=mu, size=1/(mean2disp(mu))*2^l2fc[i]), mu, K)
			})
	hv <-  sapply(1:n_genes, function(i) {
                        mu = g_means[i];
                        hidden_add_dropouts(rnbinom(n_cells2, mu=mu, size=1/(mean2disp(mu))*2^l2fc[i]), mu, K)
                        })
	return(list(truth=l2fc, groups=c(rep(1, times=n_cells1), rep(2, times=n_cells2)), de = cbind(t(base),t(de)), dv = cbind(t(base),t(dv)), hv = cbind(t(dv),t(hv))))
}

#Fit_Simulation_Params <- function(original_data) {
#
#}
Make_Sim <- function(model_params, n_genes=25000, pop_params=data.frame(cells=c(125,125), size=c(1,1), hetero=c(0,0), distinct=c(0,0)), dV_params=list(mu=0, sd=1), dE_params=list(mu=0, sd=1)) {
# model_params from above function include:
#	slope & intercept of log-log relationship btw mean & variance
#	mean & sd of distribution of mean expression per gene
#	shape & scale of gamma distribution of total counts per cell (in 100,000s reads)
#	K of MM dropouts
# parameters for each cell population:
#	number of cells
#	relative size of the cells (affects total counts per cell)
#	heterogeneity of the population (multiply FC in variance)a  - setting to 0 = no change in variance.
#	distinctness of the population (multiply FC in mean expression); - setting to 0 = no change in mean expression.
# the "reference" population is never seen, (unless hetero=0, distinct=0, size=1)
# DE & DV are added to every population therefore actual distribution of FC is 
# sum of distribution of FC between two populations, therefore sd=1 is actually sd=2

# Does not support batch effects -> would require one population to have same dV/dE as another but with an additiona effect on top.

}

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

# Modularize this stuff more sensibly
#  Plotting Functions
bg__dropout_plot_base <- function (expr_mat, xlim = NA, suppress.plot=FALSE) {
	#require("RColorBrewer")
	
	gene_info <- bg__calc_variables(expr_mat);

        xes <- log(gene_info$s)/log(10);
        put_in_order <- order(xes);
#        fancy <- densCols(xes, gene_info$p, colramp=colorRampPalette(c("black","white")))
#        dens <- col2rgb(fancy)[1,]+1L
#        colours <-  colorRampPalette(c("#000099", "#00FEFF", "#45FE4F",
#                                    "#FCFF00", "#FF9400", "#FF3100"))(256) #rainbow
#        colours <-  colorRampPalette(c("#000099", "#00FEFF", "#FCFF00"))(256) #blue->yellow
#        dens.col <- colours[dens]

        dens.col <- densCols(xes, gene_info$p, colramp=colorRampPalette(c("grey75","black")))

	if (!suppress.plot) {
        	par(fg="black")
		if (!(sum(is.na(xlim)))) {
	        	plot(xes,gene_info$p, main="", ylab="", xlab="", col = dens.col,pch=16, xlim=xlim, ylim=c(0,1))
		} else {
	        	plot(xes,gene_info$p, main="", ylab="", xlab="", col = dens.col,pch=16, ylim=c(0,1))
		}
		title(ylab="Dropout Rate", line=2)
		title(xlab="log10(expression)", line=2)
	}
	invisible(list(gene_info = gene_info, xes=xes, order=put_in_order));
}

bg__add_model_to_plot <- function(fitted_model, base_plot, lty=1, lwd=1, col="dodgerblue",legend_loc = "topright") {
	lines(base_plot$xes[base_plot$order],fitted_model$predictions[base_plot$order],lty=lty,lwd=lwd,col=col);
	#par(fg=col)
	if (length(legend_loc) == 2) {
        	this_loc <- legend(legend_loc[1], legend_loc[2], fitted_model$model, xjust=1, bty="n", lty=lty, lwd=lwd, col=c(col, "white"))
		invisible(this_loc)
	} 
	if (is.character(legend_loc)) {
		this_loc <- legend(legend_loc[1], fitted_model$model, xjust=1, bty="n", lty=lty, lwd=lwd, col=c(col, "white"))
		invisible(this_loc)
	}
	#par(fg="black")
}

bg__highlight_genes <- function (base_plot, expr_mat, genes, col="darkorange", pch=16) {
	if(!is.numeric(genes) && !is.logical(genes)) {
		genes <- match(as.character(genes), rownames(expr_mat));
		nomatch <- sum(is.na(genes));
		if (nomatch > 0) {warning(paste(nomatch, " genes could not be matched to data, they will not be highlighted."));}
		if (nomatch == length(genes)) {invisible(cbind(c(NA,NA),c(NA,NA)))}
		genes <- genes[!is.na(genes)];
	}
	points(base_plot$xes[genes],base_plot$gene_info$p[genes],col=col, pch=pch)
	invisible(cbind(base_plot$gene_info$s[genes],base_plot$gene_info$p[genes]));
}

bg__expression_heatmap <- function (genes, expr_mat, cell_labels=NA, gene_labels=NA, key_genes=genes, key_cells=NA) { 
	par_orig <- par()
	#require("RColorBrewer")
	#require("gplots")
	if(!is.numeric(genes)) {
		new_genes <- match(genes, rownames(expr_mat));
		nomatch <- sum(is.na(new_genes));
		if (nomatch > 0) {warning(paste("Warning: ",nomatch, " genes could not be matched to data, they will not be included in the heatmap."));}
		genes <- new_genes[!is.na(new_genes)];
	}
	if (length(genes) < 1) {stop("Error: No genes for heatmap.");return();}
	# Plot heatmap of expression
	heatcolours <- rev(brewer.pal(11,"RdBu"))
	col_breaks <- c(-100,seq(-2,2,length=10),100)
	heat_data <- as.matrix(expr_mat[genes,])
	heat_data <- log(heat_data+1)/log(2);
	ColColors <- rep("white", times=length(heat_data[1,]))
	RowColors <- rep("white", times=length(heat_data[,1]))
	# remove row & column labels
	rownames(heat_data) <- rep("", length(heat_data[,1]));
	if (!is.na(key_genes[1])) {
		rownames(heat_data)[rownames(expr_mat[genes,]) %in% key_genes] <- rownames(expr_mat[genes,])[rownames(expr_mat[genes,]) %in% key_genes]; 
	}
	if(length(unique(colnames(heat_data))) < length(heat_data[1,])) {
		colnames(heat_data) <- 1:length(colnames(heat_data));
	}
	if (!is.na(key_cells[1])) {
		colnames(heat_data)[colnames(expr_mat[genes,]) %in% key_cells] <- colnames(expr_mat[genes,])[colnames(expr_mat[genes,]) %in% key_cells]; 
	}
	if (!is.na(cell_labels[1])) {
		cell_labels <- as.character(cell_labels);
		colours <- as.factor(cell_labels)
		palette <- brewer.pal(max(3,length(unique(cell_labels))), "Set3");
		ColColors <- palette[colours];	
		mylegend<- list(names = unique(cell_labels), fill = unique(ColColors));
	} 
	if (!is.na(gene_labels[1])) {
		# lowest factor level = grey (so 0-1 is striking)
		if (!is.numeric(gene_labels)) {
			colours <- as.factor(gene_labels)
		} else {
			colours <- gene_labels
		}
		palette <- c("grey75",brewer.pal(max(3,length(unique(gene_labels))), "Set1"));
		RowColors <- palette[colours];
	}
	# Custom Shit
	lwid<-c(1,0.2,4)
	lhei<-c(1,0.2,4)
	lmat<-rbind(c(6,0,5),c(0,0,2),c(4,1,3))


	if (dim(heat_data)[1] < 10000) {
		heatmap_output <- suppressWarnings(heatmap.2(heat_data, ColSideColors = ColColors, RowSideColors = RowColors, col=heatcolours, breaks=col_breaks, scale="row",symbreaks=TRUE, trace="none", dendrogram="column", key=FALSE, Rowv=TRUE, Colv=TRUE,lwid=lwid, lhei=lhei,lmat=lmat, hclustfun=function(x){hclust(x,method="ward.D2")}))
	} else {
		heatmap_output = suppressWarnings(heatmap.2(heat_data, ColSideColors = ColColors, RowSideColors = RowColors, col=heatcolours, breaks=col_breaks, scale="row",symbreaks=TRUE, trace="none", dendrogram="column", key=FALSE, Rowv=FALSE, Colv=TRUE,lwid=lwid, lhei=lhei,lmat=lmat, hclustfun=function(x){hclust(x,method="ward.D2")}))
	}
	# Custom key
	par(fig = c(0, 1/(5.2),4/(5.2), 1), mar=c(4,1,1,1), new=TRUE)
	scale01 <- function(x, low = min(x), high = max(x)) {
        	x <- (x - low)/(high - low)
        	x
    	}
	par(mar=c(5,1,1,1))
	par(cex=0.75)
	par(mgp=c(2,1,0))
	key_breaks <- seq(-2,2,length=10)
	key_col <- heatcolours[2:(length(heatcolours)-1)]
	z <- seq(min(key_breaks),max(key_breaks), by=min(diff(key_breaks)/4))
	image(z=matrix(z,ncol=1),col=key_col,breaks=key_breaks,xaxt="n",yaxt="n")
	par(usr = c(0, 1, 0, 1))
	lv <- pretty(key_breaks)
        xv <- scale01(as.numeric(lv), min(key_breaks),max(key_breaks))
        xargs <- list(at = xv, labels = lv)
	xargs$side <- 1
	do.call(axis, xargs)
	mtext(side = 1, "Z-Score", line = par("mgp")[1], padj = 0.5, 
                cex <- par("cex") * par("cex.lab"))

	# Legend
	par(fig = c(0/5.2, 1/(5.2),0/(5.2), 4/5.2), mar=c(0,0,0,0), new=TRUE)
	par(mar=c(0,0,0,0))
	if (!is.na(cell_labels[1])) {
		legend("left", mylegend$names, pt.bg = mylegend$fill,bg="white",col="black", pch=22, pt.cex=2.5, cex=1.25, bty="n",y.intersp = 2);
	}
	suppressWarnings(par(par_orig))
	invisible(heatmap_output);
}

M3DropExpressionHeatmap <- function(genes, expr_mat, cell_labels=NA, interesting_genes=NA, key_genes=genes, key_cells=NA) {
	# Converted known DE genes into heatmap labels 
	gene_labels <- rep(1, times = length(genes));
	if (is.na(interesting_genes[1])) {
		gene_labels<-NA
	}
 	if (is.list(interesting_genes)) {
                for (i in 1:length(interesting_genes)) {
                        gene_labels[genes %in% interesting_genes[[i]]] <- i+1;
                }
        } else {
                gene_labels[genes %in% interesting_genes] <- 2;
        }
	if (is.numeric(key_genes) | is.logical(key_genes)) {
		key_genes <- rownames(expr_mat)[key_genes];
	}
	if (is.numeric(key_cells) | is.logical(key_cells)) {
		key_cells <- rownames(expr_mat)[key_cells];
	}
	if (is.factor(genes)) {
		genes <- as.character(genes);
	}
	if (!is.vector(genes)) {
		is.gene <- grepl("gene",colnames(genes), ignore.case=TRUE)
		if (sum(is.gene) == 1) {
			genes <- unlist(genes[,is.gene]);
		} else {
			stop("Error: please provide a vector of gene names not a table.")
		}
	}
	heatmap_output <- bg__expression_heatmap(genes, expr_mat, cell_labels=cell_labels, gene_labels=as.numeric(gene_labels), key_genes=as.character(key_genes), key_cells=key_cells);
	invisible(heatmap_output);
}

M3DropGetHeatmapClusters <- function (heatout, k, type="cell") {
	if (grepl("gene",type) | grepl("row",type)) {
        	dendro<-heatout$rowDendrogram
	} else if (grepl("cell",type) | grepl("col",type)) {
        	dendro<-heatout$colDendrogram
	}
        curr_k <- 1;
        dendro_list <- list(dendro)
        dendro_heights <- attr(dendro, "height")
        while( curr_k < k ){
                to_split <- which(dendro_heights == max(dendro_heights))
                to_split_dendro <- dendro_list[[to_split]]
                to_split_height <-  dendro_heights[to_split]

                children <- as.list(to_split_dendro)
                for (i in 1:length(children)) {
                        dendro_heights <- c(dendro_heights,attr(children[[i]],"height"))
                        dendro_list[[length(dendro_list)+1]] <- children[[i]]
                }
                # Remove to split
                dendro_list[to_split] <- NULL
                dendro_heights <- dendro_heights[-to_split]
                curr_k <- curr_k-1+length(children)
        }
        # Make group vector
	if (grepl("gene",type) | grepl("row",type)) {
	        names_orig_order <- labels(dendro)[order(heatout$rowInd)]
	} else if (grepl("cell",type) | grepl("col",type)) {
		names_orig_order <- labels(dendro)[order(heatout$colInd)]
	}
        groups <- rep(0, times=length(names_orig_order))
        for (i in 1:length(dendro_list)) {
                groups[names_orig_order %in% labels(dendro_list[[i]])] <- i
        }
	names(groups) <- names_orig_order;
        return(groups);
}

M3DropGetHeatmapNames <- function (heatout, type="cell") {
	if (grepl("gene",type) | grepl("row",type)) {
        	dendro<-heatout$rowDendrogram
	} else if (grepl("cell",type) | grepl("col",type)) {
        	dendro<-heatout$colDendrogram
	}
	return(labels(dendro))
}

hidden_m3dropImputeZeros <- function(expr_mat) {
	BasePlot <- bg__dropout_plot_base(expr_mat, xlim = NA, suppress.plot=TRUE)
	MM <- bg__fit_MM(BasePlot$gene_info$p, BasePlot$gene_info$s);

	no_zero_p <- BasePlot$gene_info$p
	no_zero_p[no_zero_p==0] <- 1/(length(expr_mat[1,])*2);
	Expected_S <- MM$K*(1/no_zero_p -1);
#	Expected_S <- MM$K*(1/BasePlot$gene_info$p -1);
#	Expected_S[BasePlot$gene_info$p==0] = BasePlot$gene_info$s[BasePlot$gene_info$p==0];
#	new_mat <- (expr_mat+Expected_S)
	n_cells <- ncol(expr_mat)
	expect_mat <- matrix(rep(Expected_S, times=n_cells), ncol=n_cells, byrow=F)
	imp_mat <- (expect_mat+expr_mat)/2
	new_mat <- expr_mat
	new_mat[expr_mat < expect_mat] <- imp_mat[expr_mat < expect_mat]
	
	return(new_mat)
}

#Copyright (c) 2015, 2016 Genome Research Ltd .
#Author : Tallulah Andrews <tallulandrews@gmail.com>
#This file is part of M3Drop.

#M3Drop is free software : you can redistribute it and/or modify it under
#the terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.

#This program is distributed in the hope that it will be useful, but WITHOUT
#ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

#You should have received a copy of the GNU General Public License along with
#this program . If not , see <http://www.gnu.org/licenses/>.

M3DropConvertData <- function(input, is.log=FALSE, is.counts=FALSE, pseudocount=1) {
	remove_undetected_genes <- function(mat) {
		no_detect <- Matrix::rowSums(mat > 0, na.rm=T) == 0;
		print(paste("Removing ",sum(no_detect), "undetected genes."))
		return(mat[!no_detect,])
	}
	type <- class(input)[1]
	lognorm <- NULL
	counts <- NULL
	if (type == "SingleCellExperiment") {
		# New scater
		c <- which(names(assays(input)) == "counts")
		ln <- which(names(assays(input)) == "logcounts")
		norm <- which(names(assays(input)) == "normcounts")
		if (length(norm) > 0) {
			return(remove_undetected_genes(input));
		} else if (length(ln) > 0) {
			lognorm <- assays(input)[[ln]]
		} else if (length(c) > 0) {
			counts <- assays(input)[[c]]
		} else {
			stop("Error: Recognized SingleCellExperiment object but cannot find either counts or lognorm expression.")
		}
	} else if (type == "ExpressionSet") {
		# monocle
		if (is.log) {
			lognorm <- Biobase::exprs(input)
		} else {
			counts <- Biobase::exprs(input)
		}
	} else if (type == "seurat") {
		# Seurat
		counts <- input@raw.data
	} else if (type == "matrix" | 
		   type == "data.frame" | 
		   type == "dgCMatrix" | 
		   type == "data.table" |
		   type == "DataTable" |
		   type == "DataFrame" |
		   type == "AnnotatedDataFrame") {
		if (type != "dgCMatrix") {
			input <- as.matrix(input)
		}

		if (is.log) {
			lognorm <- input;
		} else if (is.counts) {
			counts <- input
		} else {
			return(remove_undetected_genes(input));
		}
	} else {
		stop(paste("Error: Unrecognized input format :", type))
	}

	# Prefer log-norm to raw counts
	if (!is.null(dim(lognorm))) {
		norm <- 2^lognorm-pseudocount
		norm <- remove_undetected_genes(norm);
		return(norm)
	} 

	# CPM transform raw counts
	if (!is.null(dim(counts))) {
		sf <- Matrix::colSums(counts)
		norm <- t( t(counts)/sf * median(sf) )
		norm <- remove_undetected_genes(norm)
		return( norm )
	}
}

bg__calc_variables <- function(expr_mat) {
    if (class(expr_mat)[1] != "matrix" & class(expr_mat)[1] != "dgCMatrix" & class(expr_mat)[1] != "Matrix") {
	warning("Warning: not a recognized matrix class, coercing to 'matrix'.")
	expr_mat <- as.matrix(expr_mat)
    }
    if (sum(is.na(expr_mat)) > 0) {
	stop("Error: Expression matrix contain NA values.");
    }
    # Calc variables
#    sum_neg <- sum(expr_mat < 0)
     sum_zero <- prod(dim(expr_mat)) - sum(expr_mat > 0)
#    sum_pos <- sum(expr_mat >= 0)
#    if (is.log) {
#	expr_mat <- is.log^expr_mat-1;
#    }

    lowest <- min(expr_mat)
    if (lowest < 0) {stop("Error: Expression matrix cannot contains negative values! Has the matrix been log-transformed?")}
    
    # Deal with strangely normalized data
    if (lowest > 0) {
        warning("Warning: No zero values (dropouts) detected will use minimum expression value instead.")
        # If no zeros in expression matrix convert minimum value into zero
        #expr_mat <- round(expr_mat, digits=2) # Round to accomodate errors
        min_val <- lowest+0.05 # Instead of rounding to accomodate errors
        expr_mat[expr_mat == min_val] <- 0;
    }
    if (sum_zero < 0.1*prod(dim(expr_mat))) {
        # Less than 10% zeros
        warning("Warning: Expression matrix contains few zero values (dropouts) this may lead to poor performance.")
    }
    
    p <- 1 - Matrix::rowSums(expr_mat > 0)/ncol(expr_mat)
    if (sum(p == 1) > 0) {
	warning(paste("Warning: Removing", sum(p==1),"undetected genes."))
	expr_mat <- expr_mat[p < 1,]
        p <- 1-Matrix::rowSums(expr_mat > 0)/ncol(expr_mat)
    }

    p_stderr <- sqrt(p*(1-p)/ncol(expr_mat))
    s <- rowMeans(expr_mat)
    s_stderr <- sqrt( (rowMeans(expr_mat^2) - s^2)/ncol(expr_mat) ) #sparse matrix friendly
#    s_stderr <- rowSds(expr_mat)/sqrt(ncol(expr_mat))
    names(s_stderr) <- rownames(expr_mat)
    return(list(s = s, p = p, s_stderr = s_stderr, p_stderr = p_stderr))
}

hidden__invert_MM <- function (K, p) {K*(1-p)/(p)}
bg__horizontal_residuals_MM_log10 <- function (K, p, s) {log(s)/log(10) - log(hidden__invert_MM(K,p))/log(10)}

hidden_getAUC <- function(gene, labels) {
	labels <- unlist(labels);
        ranked <- rank(gene);
	#MAT <- as.matrix(ranked)
        #x <- split(seq(ncol(MAT)), labels)
        #ms <- sapply(x, function(a) rowMeans(MAT[,a]))

        ms <- aggregate(ranked~unlist(labels),FUN=mean); #Get average score for each cluster
        posgroup <- as.character(unlist(ms[which(ms[,2]==max(ms[,2])),1])); #Get cluster with highest average score
        if (length(posgroup) > 1) {return (c(-1,-1,-1))} # Return negatives if there is a tie for cluster with highest average score (by definition this is not cluster specific)

        # Create 1/0 vector of truths for predictions, cluster with highest average score vs everything else
        truth <- labels == posgroup

        #Make predictions & get auc using RCOR package.
        pred <- ROCR::prediction(ranked,as.numeric(truth))
        val <- unlist(ROCR::performance(pred,"auc")@y.values)
        pval <- wilcox.test(gene[truth],gene[!truth])$p.value
        if (!exists("pval")) {pval <- 1}

        return(c(val,posgroup,pval))
}

hidden_fast_AUC_m3drop <- function(expression_vec, labels) {
	R = rank(expression_vec);
	labels <- unlist(labels);
	ms <- aggregate(R~unlist(labels),FUN=mean); #Get average score for each cluster
	posgroup <- as.character(unlist(ms[which(ms[,2]==max(ms[,2])),1])); #Get cluster with highest average score
	if (length(posgroup) > 1) {return (c(-1,-1,-1))} # Return negatives if there is a tie for cluster with highest average score (by definition this is not cluster specific)
	truth <- labels == posgroup

	out <- wilcox.test(expression_vec[truth], expression_vec[!truth], alternative="two.sided")
	
        N1 = sum(truth)
        N2 = sum(!truth);
        #U1 = sum(R[truth])-N1*(N1+1)/2
        U2 = sum(R[!truth])-N2*(N2+1)/2
        if (N1 == 0) {return(c(0,0,0))}
        if (N2 == 0) {return(c(1,1,1))}
        AUC = 1-U2/(N1*N2);  # smaller valued ranks (i.e. lower expression values for !true).
        # assumes large sample size
        #  https://ncss-wpengine.netdna-ssl.com/wp-content/themes/ncss/pdf/Procedures/PASS/Confidence_Intervals_for_the_Area_Under_an_ROC_Curve.pdf
        # originally (Hanley and McNeil 1982)
	
        return(c(AUC, posgroup, out$p.value));
}

M3DropGetMarkers <- function(expr_mat, labels) {
	if (length(labels) != length(expr_mat[1,])) {
		stop("Length of labels does not match number of cells.")
	}
        aucs <- apply(expr_mat,1,hidden_fast_AUC_m3drop,labels=labels)
        auc_df <- data.frame(matrix(unlist(aucs), ncol=3, byrow=TRUE))
        rownames(auc_df) <- rownames(expr_mat)
        colnames(auc_df) <- c("AUC","Group", "pval")
	auc_df$Group <- as.character(auc_df$Group)
	if(sum(auc_df$Group == "-1") > 0) {
		auc_df$Group[auc_df$Group == "-1"] <- "Ambiguous";
	}
        auc_df[,1] <- as.numeric(as.character(auc_df[,1]))
        auc_df[,3] <- as.numeric(as.character(auc_df[,3]))
        auc_df <- auc_df[auc_df[,1] > 0,]
	auc_df <- auc_df[order(-auc_df$AUC),]
        return(auc_df);
}

\documentclass[a4paper]{article}
\begin{document}

\title{Dropout-based Feature Selection with M3Drop and NBumi}
\author{Tallulah Andrews}
%\VignetteIndexEntry{Introduction to M3Drop}

\maketitle

\section*{Introduction}

Feature selection is a key step in the analysis of single-cell RNASeq data. 
Feature selection aims to identify and remove genes whose expression is 
dominated by technical noise, thus reducing the effect of batch-effects and 
other technical counfounders while also reducing the curse of dimensionality. 

A common way to select features is to identify genes that follow a different 
expression distribution than a set of control genes. These control genes may 
be spike-ins but more commonly one assumes that the majority of genes don't
differ from control genes and use all genes to approximate the expression 
pattern of expression of controls.

Often the variance relative to the mean expression is used to characterize the
expression distribution of a gene, and feature selection proceeds by identifying
highly variable genes. In contrast, this package provides two methods which 
characterize the expression distribution of a gene using the dropout-rate and 
mean expression. This takes advantage of the fact than at least 50\% of entries
in a single-cell RNASeq expression matrix are dropouts, and that dropout rate
is less sensitive to sampling errors than variance.

\section*{M3Drop}

Single-cell RNA sequencing is able to quantify the whole transcriptome from the
small amount of RNA present in individual cells. However, a consequence of 
reverse-transcribing and amplifying small quantities of RNA is a large number 
of dropouts, genes with zero expression in particular cells. The frequency of 
dropout events is strongly non-linearly related to the measured expression 
levels of the respective genes. M3Drop posits that these dropouts are due to 
failures of reverse transcription, a simple enzyme reaction, thus should be 
modelled using the Michaelis-Menten equation as follows:

$$P_{i} = 1 - \frac{S_{i}}{S_{i}+K}$$

Where $P_{i}$ is the proportion of cells where gene $i$ dropouts out, 
$S_{i}$ is the mean expression of gene $i$ and $K$ is the Michaelis constant. 
This model fits observed single-cell RNASeq data which have been sequenced to 
near saturation (i.e. >100,000 reads per cell). 

Note: This model works well for Smartseq2 and other single-cell datasets that do not use unique molecular identifiers (UMIs). For 10X Chromium data or other UMI tagged data see the NBumi model below.

We'll be using a portion of the Deng et al. (2014) dataset in this example. You can download the R-package containing this data (M3DExampleData) from Bioconductor using biocLite().
<<results=hide>>=
library(M3Drop)
library(M3DExampleData)
@

\subsection*{Prepare Data}
The first step is to perform quality control on cells. Any existing tools can
be used for this. Here we will simply remove cells with fewer than 2000 detected
genes.
<<>>=
counts <- Mmus_example_list$data
labels <- Mmus_example_list$labels
total_features <- colSums(counts >= 0)
counts <- counts[,total_features >= 2000]
labels <- labels[total_features >= 2000]
@

The second is to extract and correctly format data for M3Drop. To support 
quality control tools in various other single-cell R packages, we have 
provided a function to extract appropriate data from objects used by scater, 
monocle, and Seurat. This function also removes undetected genes, and if necessary applies a counts-per-million normalization to raw count data. 

<<>>=
norm <- M3DropConvertData(counts, is.counts=TRUE)
@

M3Drop requires a normalized but not log-transformed expression matrix, thus the above function can optionally de-log a log2-normalized expression matrix from any normalization method.

<<>>=
norm <- M3DropConvertData(log2(norm+1), is.log=TRUE, pseudocount=1)
@

Any normalization method which preserves dropouts (i.e zeros) is compatible 
with M3Drop.

\subsection*{Feature Selection}

Since the Michaelis-Menten equation is convex, averaging across a mixed population forces differentially expressed genes to be shifted to the right of the Michaelis-Menten curve (Figure 1).

\begin{center}
<<fig=TRUE>>=
K <- 49
S_sim <- 10^seq(from=-3, to=4, by=0.05)
MM <- 1-S_sim/(K+S_sim)
plot(S_sim, MM, type="l", lwd=3, xlab="Expression", ylab="Dropout Rate", 
	xlim=c(1,1000))
S1 <- 10; P1 <- 1-S1/(K+S1);
S2 <- 750; P2 <- 1-S2/(K+S2);
points(c(S1,S2), c(P1,P2), pch=16, col="grey85", cex=3)
lines(c(S1, S2), c(P1,P2), lwd=2.5, lty=2, col="grey35")
mix <- 0.5;
points(S1*mix+S2*(1-mix), P1*mix+P2*(1-mix), pch=16, col="grey35", cex=3)
@ 
\\
\textbf{Figure 1}: Michaelis-Menten is convex which leads to DE genes being outliers to the right/above the curve.
\end{center}

Feature selection for DE genes are identified by comparing the local $K$ 
calculated for a specific gene to the global $K$ fitted across all genes using 
a Z-test followed by multiple-testing correction. Here we find 1,248 DE genes 
at 1\% FDR.

\begin{center}
<<fig=TRUE>>=
M3Drop_genes <- M3DropFeatureSelection(norm, mt_method="fdr", mt_threshold=0.01)
@
\\
\textbf{Figure 2}: Differentially expressed genes at 1\% FDR (purple).
\end{center}

\section*{NBumi}

The Michaelis-Menten equation fits full-transcript single-cell RNASeq data 
well, but often struggles to fit data tagged with unique molecular identifiers 
(UMIs). This is a result of UMI datasets typically not being sequenced to 
saturation, thus many dropouts are a result of low sequencing coverage rather 
than a failure of reverse transcription. 

To account for zeros resulting from insufficient sequencing depth, the M3Drop 
packaged includes a depth-adjusted negative binomial model (DANB). DANB models 
each observation as a negative binomial distribution with mean proportional to
the mean expression of the respective gene and the relative sequencing depth 
of the respective cell. The dispersion parameter of the negative binomial is 
fit to the variance of each gene. The equations for calculating these 
parameters are:

Observation specific means:
$$\mu_{ij} = \frac{\sum\limits_{i} t_{ij} * \sum\limits_{j} t_{ij}}{\sum\limits_{ij} t_{ij}}$$


Gene specific dispersion (solved for $r_{i}$):
$$\sum_{j}(t_{ij}-\mu_{ij})^2 = \sum_{j}(\mu_{ij}+r_{i}\mu_{ij}^2)$$

Where $\mu_{ij}$ is the observation specific mean for the negative binomial for the observed molecule counts, $t_{ij}$ of the $i$th gene in the $j$th of cell and $r_{i}$ is the dispersion parameter (the "size" paramter of R's nbinom functions) for gene $i$.

Functions relating to the DANB model are tagged with the "NBumi" prefix. We will continue using the example data despite it not using UMIs for demonstration purposes. Similar to M3Drop we have provided a function to extract the relevant 
expression matrix from objects used by popular single-cell RNASeq analysis 
packages : scater, monocle, and Seurat. This is a separate function because
DANB requires raw counts as opposed to the normalized expression matrix used
by M3Drop. 

<<>>=
count_mat <- NBumiConvertData(Mmus_example_list$data, is.counts=TRUE)
@

This function can also de-log and de-normalize a provided expression matrix, 
but raw counts are preferable if they are available.

Next we fit the DANB model to this count matrix, and check it fits the data:

\begin{center}
<<fig=TRUE>>=
DANB_fit <- NBumiFitModel(count_mat)

# Smoothed gene-specific variances
par(mfrow=c(1,2))
stats <- NBumiCheckFitFS(count_mat, DANB_fit)
print(c(stats$gene_error,stats$cell_error))
@
\\
\textbf{Figure 3}: Fits of the depth-adjusted negative binomial.
\end{center}

Since this is a full-transcript dataset that we have converted from normalized values the model 
doesn't fit well as can be seen above. We will continue with this data for demonstration purposes only.

We use a binomial test to evaluable the significance of features under the DANB
model:

\begin{center}
<<fig=TRUE>>=
NBDropFS <- NBumiFeatureSelectionCombinedDrop(DANB_fit, method="fdr", qval.thres=0.01, suppress.plot=FALSE)
@
\textbf{Figure 4}: Dropout-based feature selection using DANB
\end{center}

\subsection*{Pearson Residuals}
Pearson residuals have recently been proposed as an alternative normalization approach for UMI tagged single-cell RNAseq data. (https://genomebiology.biomedcentral.com/articles/10.1186/s13059-021-02451-7)

We have added two option for calculating Pearson residuals using the DANB model presented here:

<<>>=
pearson1 <- NBumiPearsonResiduals(count_mat, DANB_fit)
pearson2 <- NBumiPearsonResidualsApprox(count_mat, DANB_fit)
@

If you have not fit the DANB model to the data, you can run each of these functions directly on the count matrix alone, and the required model fitting will be performed as part of the function. In this case the NBumiPearsonResidualsApprox function will be much quicker since it does not fit the dispersion parameter of the DANB model, only the mean, and approximates the Pearson residuals as if the data was Poisson distributed:

$$p_{approx} = \frac{x_{ij}-mu_{ij}}{\sqrt{mu_{ij}}}$$

Where $x_{ij}$ is the observed umi counts.

Whereas the NBumiPearsonResiduals function calculates the Pearson residuals using the full DANB model as:

$$p_{exact} = \frac{x_{ij}-mu_{ij}}{\sqrt{mu_{ij} + \frac{mu_{ij}^2}{r_i}}}$$

We do not perform any trimming or clipping of the residuals. To perform $\sqrt{n}$ clipping as per Hafemeister and Satija, simple run the following additional steps:

<<>>=
pearson1[pearson1 > sqrt(ncol(count_mat))] <- sqrt(ncol(count_mat))
pearson1[pearson1 < -sqrt(ncol(count_mat))] <- -1*sqrt(ncol(count_mat))
@


\section*{Other Feature Selection Methods}

For comparison purposes we have provided functions for other feature selection
methods, including: 
BrenneckeGetVariableGenes from Brennecke et al. (2013) modified to optionally use all genes to fit the function between CV2 and mean expression.
giniFS based on GiniClust (REF)
pcaFS based on monocle (REF) using irlba and sparse-matricies.
corFS which uses gene-gene correlations.
and ConsensusFS which uses all the available feature selection methods and takes the average rank of genes across all methods.

Only the Brennecke highly variable gene method also provides a significance 
test and runs in linear time, thus is the only one we will demostrate here.


\begin{center}
<<fig=TRUE>>=
HVG <- BrenneckeGetVariableGenes(norm)
@
\textbf{Figure 5}: Brennecke highly variable genes.
\end{center}

\section*{Examining Features and Identifying Subpopulations of Cells}

To check that the identified genes are truly differentially expressed we can plot the normalised expression of the genes across cells.

\begin{center}
<<fig=TRUE>>=
heat_out <- M3DropExpressionHeatmap(M3Drop_genes$Gene, norm, 
		cell_labels = labels)
@
\\
\textbf{Figure 6}: Heatmap of expression of M3Drop features.
\end{center}

The heatmap (Figure 6) shows that the genes identified by M3Drop are 
differentially expressed across timepoints. Furthermore, it shows that the 
blastocysts cluster into two different groups based on the expression of 
these genes. We can extract these groups and identify marker genes for them 
as follows:

<<>>=
cell_populations <- M3DropGetHeatmapClusters(heat_out, k=4, type="cell")
library("ROCR") 
marker_genes <- M3DropGetMarkers(norm, cell_populations)
@

The first function cuts the dendrogram from the heatmap to produce k clusters 
of cells. These labels are stored in cell\_populations. The second function 
tests all genes as marker genes for the provided clusters.

Marker genes are ranked by the area-under the ROC curve (AUC) for predicting 
the population with the highest expression of the gene from the other groups. 
Significance is calculated using a Wilcox-rank-sum test. Now we can examine 
the marker genes of the two clusters of blastocyst cells more closely.

<<>>=
head(marker_genes[marker_genes$Group==4,],20) 
marker_genes[rownames(marker_genes)=="Cdx2",] 
@

This shows that the inner cell mass (ICM) marker Sox2 is one of the top 20 
markers for group 4 and that the trophectoderm (TE) marker Cdx2 is a marker 
for group 3, suggesting these two clusters coorespond to ICM and TE cells 
within each blastocyst.

\section*{Comparing to Other Methods}

The DANB features look similar to M3Drop though identifies more genes due to 
the simplified noise model it uses, which is appropriate for UMI-tagged data 
but too permissive for data with high amplifcation noise.

\begin{center}
<<fig=TRUE>>=
heat_out <- M3DropExpressionHeatmap(NBDropFS$Gene, norm, 
		cell_labels = labels)
@
\\
\textbf{Figure 7}: Heatmap of expression of DANB features.
\end{center}

The HVG method is more sensitive to lowly expressed genes. In addition, the 
quadratic model it uses frequenly over estimates the expected variability of 
highly expressed genes. This is in contrast with M3Drop which recognizes the 
low information available for lowly expressed genes. 

This difference can be seen by comparing the heatmaps for the respective genes.
The highly variable genes contains many more genes exhibiting just noisy 
expression, whereas nearly all genes detected by M3Drop are clearly 
differentially expressed across the different cell populations. 

\begin{center}
<<fig=TRUE>>=
heat_out <- M3DropExpressionHeatmap(HVG, norm, 
		cell_labels = labels)
@
\\
\textbf{Figure 8}: Heatmap of expression of highly variable genes across cells.
\end{center}

\section*{References}

Tallulah Andrews, Martin Hemberg. Modelling dropouts allows for unbiased identification of marker genes in scRNASeq experiments. \textit{bioRxiv}, 2016. doi:10.1101/065094

Philip Brennecke, Simon Anders, Jong Kyoung Kim, Aleksandra A Kolodziejczyk, Xiuwei Zhang, Valentina Proserpio, Bianka Baying, Vladimir Benes, Sarah A Teichmann, John C Marioni and Marcus G Heisler. Accounting for technical noise in single-cell RNA-seq experiments. \textit{Nature Methods}, 10:1093-1095, 2013. doi:10.1038/nmeth.2645, PMID: 24056876

\end{document}

\name{M3DropGetMarkers}
\alias{M3DropGetMarkers}
\title{Identify marker genes}
\description{
	Calculates area under the ROC curve for each gene to predict the best group of cells from all other cells.
}
\usage{
  M3DropGetMarkers(expr_mat, labels)
}
\arguments{
  \item{expr_mat}{a numeric matrix of normalized expression values, columns = samples, rows = genes.}
  \item{labels}{a vector of group ids for each cell/sample.}
}
\details{Uses the ROCR package to calculate the AUC for each gene for the group with the highest average rank. Significant is calculated using a Wilcox rank-sum test. 
}
\value{
  A dataframe with a row for each gene and columns: AUC, Group (which label this gene had the highest average rank for), and pval (uncorrected p-value of prediction).
}
\examples{
  library(M3DExampleData)
  norm <- M3DropConvertData(Mmus_example_list$data, is.counts=TRUE)
  marker_gene_table <- M3DropGetMarkers(norm, Mmus_example_list$labels)
}
\keyword{markers}

\name{M3DropThreeSetVenn}
\alias{M3DropThreeSetVenn}
\title{Three-way Venn Diagram}
\description{
	Plot an area-proportional three-set Venn Diagram with labels.
}
\usage{
  M3DropThreeSetVenn(set1, set2, set3, names)
}
\arguments{
  \item{set1}{a vector of items in the first set.}
  \item{set2}{a vector of items in the second set.}
  \item{set3}{a vector of items in the third set.}
  \item{names}{a vector of names of each set}
}
\details{Approximates area-proportional three-set Venn Diagram with code by David J. States (available at: http://tolstoy.newcastle.edu.au/R/help/03a/1115.html). Then places labels within each circle and overlap-region using code by Tallulah Andrews. 
}
\value{None}
\examples{
  SetA <- c(1:20)
  SetB <- c(15:30)
  SetC <- c(5,10,15,20,25,30,35,40,45,50,55,60)
  M3DropThreeSetVenn(SetA, SetB, SetC, names=c("A","B","C"))
}
\keyword{Venn Diagram}

\name{M3DropTraditionalDE}
\alias{bg__m3dropTraditionalDE}
\alias{bg__m3dropTraditionalDEShiftDisp}
\alias{bg__fitdispersion}
\alias{bg__get_mean2disp}
\title{Perform Traditional Differential Expression}
\description{
	Performs traditional (i.e. compare defined groups) differential expression using a negative binomial model with MM zero-inflation. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
	bg__get_mean2disp(expr_mat)
	bg__fitdispersion(expr_mat)
}
\arguments{
  \item{expr_mat}{a numeric matrix of library-size normalized expression values, columns = samples, rows = genes.}
}
\details{
THESE FUNCTIONS SHOULD NOT BE USED.

\code{unfinished__m3dropTraditionalDE} : Uses a log-likelihood ratio test to perform model selection between a model of constant mean expression vs a model of different mean expression across the biological groups. 
Probabilities of observing the data given the model are calculated using a zero-inflated negative binomial distribution. Global relationships between mean and dispersion (power-law) as well as mean and 
dropouts (Michaelis-Menten) for genes are fit independently for each batch. Dispersions are fixed for each batch and calculated using the fitted power-law using the global mean expression of each gene.
Significance is evaluated using the chi-square distribution.

\code{unfinished__m3dropTraditionalDEShiftDisp}: Uses a log-likelihood ratio test to perform model selection between a model of constant mean expression vs a model of different mean expression across the biological groups. 
Probabilities of observing the data given the model are calculated using a zero-inflated negative binomial distribution. Global relationships between mean and dispersion (power-law) as well as mean and 
dropouts (Michaelis-Menten) for genes are fit independently for each batch. Dispersions are shifted from the global variance according to the mean expression for each biological group, using batch-specific power-laws.
Significance is evaluated using the chi-square distribution.


\code{broken__m3dropCTraditionalDE} : Uses a log-likelihood ratio test to perform model selection between a model of constant mean expression vs a model of different mean expression across the biological groups. 
Probabilities of observing the data given the model are calculated using a zero-inflated negative binomial distribution. Global relationships between mean and dispersion (power-law) as well as mean and 
dropouts (Michaelis-Menten) for genes are fit to the full dataset. Significance is evaluated using the chi-square distribution.

\code{bg__get_mean2disp} fits a power-law relationship between the squared coefficient of variation and mean expression of each gene, which is used to calculate the expected dispersion parameter for the negative binomial distribution from a given mean expression value.

\code{bg__fitdispersion} estimates gene-specific dispersions from the mean and variance of gene expression values \deqn{r = \frac{\mu^2}{\sigma^2-\mu}}{r = mu^2/(var-mu)}. Then fits a power-law relationship between the estimated dispersion and mean exprssion.
}

\value{
	\code{bg__m3dropTraditionalDE} : a table of observed mean expression levels for each biological group and each batch with raw p-values and FDR corrected p-values for each gene.
	\code{bg__m3dropTraditionalDEShiftDisp} : a table of observed mean expression levels for each biological group and each batch with raw p-values and FDR corrected p-values for each gene.
	\code{broken__m3dropCTraditionalDE} : a table of observed mean expression levels for each biological group with raw p-values and FDR corrected p-values for each gene.
	\code{bg__get_mean2disp} : a function which calculates the expected dispersion given the mean expression.
	\code{bg__fitdispersion} : exponent of the power-law relationship between dispersion and mean expression.
}
\examples{
library(M3DExampleData)
#Normalized_data <- M3DropCleanData(Mmus_example_list$data,
#                  labels = Mmus_example_list$labels,
#                  is.counts=TRUE, min_detected_genes=2000)
#DE_output <- bg__m3dropTraditionalDE(Normalized_data$data[1:100,], Normalized_data$labels)
#DE_shifted_output <- bg__m3dropTraditionalDEShiftDisp(Normalized_data$data[1:100,], Normalized_data$labels)
#DE_output_batches <- bg__m3dropTraditionalDE(Normalized_data$data[1:100,], Normalized_data$labels, batches=Normalized_data$labels) # each biological condition was performed separately
}
\keyword{differential expression}

\name{bg__fit_size_to_var}
\alias{bg__fit_size_to_var}
\title{Fit gene-specific dispersion}
\description{
	Fits dispersion for a specific gene for the depth-adjusted negative binomial. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
	bg__fit_size_to_var(obs, mu_vec, max_size, min_size=10^-10, convergence=0.001)
}
\arguments{
  \item{obs}{observed variance corrected for library size.}
  \item{mu_vec}{expectation of depth-adjusted negative-binomial for each observation}
  \item{max_size}{maximum dispersion for genes that exhibit Poissonian expression.}
  \item{min_size}{initial dispersion estimate.}
  \item{convergence}{acceptable error on dispersion estimate.}
}
\details{
	Minimized the difference between the mean of expected variances for each observation based on the depth-adjust negative binomial(DANB) and 
	the observed variance. The observed variance is corrected for library size by subtracting the observation specific mean as per the DANB from 
	the observed UMI counts prior to calculating the variance. The optimization is done using an iterative method similar to Newton's method, which ends when the difference between estimates is less than "convergence" or if the estimated dispersion exceeds "max_size" or after 1000 iterations. 
}
\value{
	Estimated dispersion.
}
\examples{
#	library(M3DExampleData)
#	counts <- as.matrix(Mmus_example_list$data);
#	counts <- counts[rowSums(counts) > 0,];
#	fit <- NBumiFitModel(counts);
#	coeffs <- NBumiFitDispVsMean(fit, suppress.plot=TRUE);
}

\name{bg__get_stats}
\alias{bg__get_stats}
\title{Calculate Simulation Statistics}
\description{
	Given significant genes and a ground truth calculates the false discovery rate and false negative rate of the significance calls.
}
\usage{	bg__get_stats(sig, TP, ngenes)}
\arguments{
  \item{sig}{vector of gene IDs/names which were called as significant by some method.}
  \item{TP}{vector of gene IDs/names which known to be positives (ground truth).}
  \item{ngenes}{number of total genes under consideration.}
}
\details{False discovery rate (FDR) is the proportion of all genes considered significant which are not in the ground truth. False negative rate (FNR) is the proportion of all ground truth genes which are not considered significant.
}
\value{vector of two elements: (FDR, FNR).}
\examples{
#	Calls <- c(1,3,5,8,10)
#	Truth <- c(2,3,5,9)
#	total=10
#	stats <- bg__get_stats(Calls, Truth, total)
}
\keyword{simulations, statistics}

\name{bg__default_mean2disp}
\alias{bg__default_mean2disp}
\title{Mean to Dispersion}
\description{
	Default function to calculate the negative binomial dispersion parameter given the mean expression of the gene.
}
\usage{
	bg__default_mean2disp(mu, coeffs=c(3.967816,-1.855054))
}
\arguments{
  \item{mu}{mean expression of the gene(s)}
  \item{coeffs}{coefficients of linear regression between log(cv2) and log10(mean expression) for observed data.}
}
\details{
Given the coefficients of a linear regression between the natural logarithm of the squared coefficient of variation and log base 10 of mean expression, calculates the dispersion parameter (1/size) of the negative binomial distribution for a given expression level.

Default coefficients were fit to the Buettner at al.[1] single cell mESC data.
}
\value{the dispersion parameter (1/size) for the (mu,size) paramterization of the \code{rnbinom} function.
}
\examples{
# mu <- 100
# r <- bg__default_mean2disp(mu)
# n_cells <- 50
# sim <- rnbinom(n_cells, size=1/r, mu=mu)
}
\keyword{simulations}
\references{
[1] Buettner et al. (2015) Computational analysis of cell-to-cell heterogeneity in single-cell RNA-sequencing data reveals hidden subpopulations of cells. Nature Biotechnology 33 : 155-160.
}

\name{NBumiConvertToInteger}
\alias{NBumiConvertToInteger}
\title{Turn a matrix of expression values into integer counts}
\description{
	Reformats a provided expression matrix to be compatible with NBumi modelling. 
}
\usage{
	NBumiConvertToInteger(mat)
}
\arguments{
  \item{mat}{a numeric matrix of expression values (ideally raw UMI counts), columns = samples, rows = genes.}
}
\details{
	Coerces the provided data to a matrix then rounds all values up (ceiling) to integers and removes all rows where all values are zero.	
}
\value{
	Rounded, integer matrix of the original data.
}
\examples{
mat <- matrix(rgamma(1000, shape=10, scale=20), ncol=10)
mat_int <- NBumiConvertToInteger(mat)
is.integer(mat_int)
}
\keyword{Depth-Adjusted Negative Binomial}

\name{NBumiFeatureSelectionHighVar}
\alias{NBumiFeatureSelectionHighVar}
\title{Variance-based Feature Selection}
\description{
	Ranks genes by residual dispersion from mean-dispersion power-law relationship.
}
\usage{
	NBumiFeatureSelectionHighVar(fit)
	
}
\arguments{
  \item{fit}{output from NBumiFitModel or NBumiFitBasicModel.}
}
\details{
	Uses linear regression on log-transformed mean expression and fitted dispersions. Ranks genes by the residuals of this fit, negative = high variance, positive = low variance.
}
\value{
	Sorted vector of residuals.
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	fit <- NBumiFitModel(counts);
	HVGs <- names(NBumiFeatureSelectionHighVar(fit)[1:2000]);
}
\keyword{single cell}
\keyword{feature selection}
\keyword{depth-adjusted negative binomial}

\name{Other Feature Selection Methods}
\alias{irlbaPcaFS}
\alias{giniFS}
\alias{corFS}
\title{Other Feature Selection Methods}
\description{
	Other feature selection methods which only rank genes.
}
\usage{
	irlbaPcaFS(expr_mat, pcs=c(2,3))
	giniFS(expr_mat, suppress.plot=TRUE)
	corFS(expr_mat, dir=c("both", "pos", "neg"), fdr=NULL)
}
\arguments{
  \item{expr_mat}{normalized but not log-transformed gene expression matrix, rows=genes, cols=cells.}
  \item{pcs}{which principle components to use to score genes.}
  \item{suppress.plot}{whether to plot the gene expression vs Gini score.}
  \item{dir}{direction of correlation to consider.}
  \item{fdr}{apply an fdr threshold for significant features.}
}
\details{

	\code{irlbaPcaFS} Features are ranked by the sum of the magnitude of the loadings for the specified principle components. PCA is performed using the irlba package using sparse matricies for speed.
	\code{giniFS} Fits a loess curve between the maximum expression value and gini-index of each gene. Genes are ranked by p-value from a normal distribution fit to the residuals of the curve. As proposed by GiniClust [1].
	\code{corFS} Calculates all gene-gene correlations then ranks genes by the magnitude of the most positive or negative correlation. "both" will rank by the average of the magnitudes of the most positive & negative correlation.
}
\value{
	Sorted vector of scores for each gene from best features to worst features.
}
\examples{
library(M3DExampleData)
norm <- M3DropConvertData(Mmus_example_list$data[1:500,]);
Features_gini <- giniFS(norm, suppress.plot=FALSE);
Features_cor <- corFS(norm);
Features_pca <- irlbaPcaFS(norm);
}
\keyword{single cell}
\keyword{feature selection}

\name{M3DropCleanData}
\alias{M3DropCleanData}
\alias{bg__filter_cells}
\title{Filter Expression Data}
\description{
	Filters and normalizes a given expression matrix. Removes low quality cells and undetected genes, and normalizes counts to counts per million. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
  M3DropCleanData(expr_mat, labels = NA, is.counts = TRUE, suppress.plot = FALSE, pseudo_genes = NA, min_detected_genes = NA)
  bg__filter_cells(expr_mat, labels = NA, suppress.plot = FALSE, min_detected_genes = NA)
}
\arguments{
  \item{expr_mat}{a numeric matrix of raw or normalized (not log-transformed) expression values, columns = samples/cells, rows = genes.}
  \item{labels}{a vector of length equal to the number of columns of expr_mat with names or group IDs for each cell.}
  \item{is.counts}{logical, whether the provided data is unnormalized read/fragment counts.}
  \item{suppress.plot}{logical, whether to plot the distribution of number of detected genes per cell.}
  \item{pseudo_genes}{a vector of gene names of known pseudogenes which will be removed from the cleaned data.}
  \item{min_detected_genes}{minimum number of genes/cell for a cell to be included in the cleaned data.}
}
\details{Retains genes detected (expression>0) in more than 3 cells and with mean normalized expression >= 10^-5. If min_detected_genes is defined all cells not reaching the threshold are removed. Otherwise, fits a normal distribution to the distribution of detected genes/cell and removes those cells with significantly few detected genes (FDR 5\%). This fit is plotted for visual inspection. If is.counts==TRUE then each column is converted to counts per million (ignoring ERCC spike-ins if present).}
\value{A list with elements: data, the normalized filtered expression matrix; and labels, labels of the remaining cells.}
\examples{
  library(M3DExampleData)
  # Remove all cells with < 2000 detected genes and convert to cpm 
  cpm <- M3DropCleanData(Mmus_example_list$data, Mmus_example_list$labels, 
			is.counts=TRUE, min_detected_genes=2000)
  # Removes cells with significantly few detected genes (FDR=5%)
  filtered_only <- M3DropCleanData(Mmus_example_list$data, Mmus_example_list$labels, 
			is.counts=FALSE)
#  QCed <- bg__filter_cells(Mmus_example_list$data[,1:10], Mmus_example_list$labels[1:10], suppress.plot=TRUE)
}
\keyword{normalization, quality control}

\name{NBumiConvertData}
\alias{NBumiConvertData}
\title{Convert Data to be suitable for NBumi}
\description{
	Recognizes a variety of R objects/classes and extracts expression 
	matrices from them then converts that to a count matrix for input 
	into NBumi functions.
}
\usage{
  NBumiConvertData(input, is.log=FALSE, is.counts=FALSE, pseudocount=1)
}
\arguments{
  \item{input}{ a matrix, data.frame or object }
  \item{is.log}{ has the data been log-transformed? (assumes log-base 2 with pseudocount of 1) }
  \item{is.counts}{ is the data raw unnormalized counts? (raw counts will be CPM normalized) }
  \item{pseudocount}{ pseudocount added before log-transformation }
}
\details{
	You must have loaded the respective packages (in parentheses) into your namespace before running this function on the respective objects. Note that to maintain scalability sparse matrices will remain as such.

	Supported classes/objects:
		\describe{

		\item{SCESet (scater <= 1.4.0)}{uses "counts" or if unavailable then "exprs"}

		\item{SingleCellExperiment (scater >= 1.6.0)}{uses "counts", if unavailable then "logcounts", which is assumed to be log-normalized.}

		\item{ExpressionSet (Biobase)}{uses "exprs", specify log/counts using arguments}

		\item{seurat (Seurat)}{uses "raw.data" as counts.}
		}
		Matrix/Dataframe classes :
		\describe{
		\item{dgCMatrix (Matrix)}{specify log/counts using arguments}

		\item{data.table (data.table)}{specify log/counts using arguments}

		\item{DataTable (S4Vectors)}{specify log/counts using arguments}

		\item{DataFrame (S4Vectors)}{specify log/counts using arguments}

		\item{AnnotatedDataFrame (Biobase)}{specify log/counts using arguments}

		\item{matrix (base-r)}{specify log/counts using arguments}

		\item{data.frame (base-r)}{specify log/counts using arguments}
		}

	Counts are rounded up to integers if necessary, if counts are 
	unavailable then this will attempt to convert log2 normalized expression
	to counts by de-logging, subtracting the pseudocount, and then 
	un-normalizing by rescaling cells based on their relative number of 
	detected genes, finally expression is rounded up to integers for use 
	as counts. 

}
\value{A count matrix appropriate for input into NBumi functions.}
\examples{
	# Simulated raw count matrix:
	set.seed(42)
	counts <- matrix(rpois(200, lambda=3), ncol=10)
	input_counts <- NBumiConvertData(counts, is.counts=TRUE)

	# log normalized data frame
	lognorm <-log2( t(t(counts)/colSums(counts)*100)+1 )
	lognorm <- as.data.frame(lognorm)
	input_counts <- NBumiConvertData(lognorm)

	# Sparse matrix
	require("Matrix")
	counts <- Matrix(counts, sparse=TRUE)
	input_counts <- NBumiConvertData(counts, is.counts=TRUE)

	# SingleCellExperiment Object
	require("SingleCellExperiment")
	SCE <- SingleCellExperiment(assays=list(counts=counts))
	input_counts <- NBumiConvertData(SCE)
}
\keyword{normalization}

\name{NBumiFeatureSelectionCombinedDrop}
\alias{NBumiFeatureSelectionCombinedDrop}
\title{Dropout-based Feature Selection}
\description{
	Ranks genes by significance of increase in dropouts compared to expectation.
}
\usage{
	NBumiFeatureSelectionCombinedDrop(fit, ntop=NULL, method="fdr", qval.thresh=2, suppress.plot=TRUE)
	
}
\arguments{
  \item{fit}{output from NBumiFitModel or NBumiFitBasicModel.}
  \item{ntop}{number of top ranked genes to return}
  \item{method}{correction method for multiple comparisons (check ?p.adjust.methods for more details)}
  \item{qval.thresh}{significant threshold}
  \item{suppress.plot}{logical, whether to plot the fitted curve and highlight selected features}
}
\details{
	Calculates dropout probability for each observation using depth-adjusted negative binomial means and dispersions calculated 
	from a fitted power-law relationship between mean and dispersion. Total dropouts per gene are modelled using the normal 
	approximation of the sum of bernoulli variables. And significance is evaluated using a Z-test.

	If provided, ntop will overrule the significance threshold.
}
\value{
	dataframe with columns: 
		Gene 
		effect_size (difference between observed and expected dropout rate)
		p.value
		q.value (corrected by adjustment method specifed by the method argument)
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	fit <- NBumiFitModel(counts);
	Drop_features <- names(NBumiFeatureSelectionCombinedDrop(fit, qval.thresh=0.05));
}
\keyword{single cell}
\keyword{feature selection}
\keyword{depth-adjusted negative binomial}

\name{bg__var_vs_drop}
\alias{bg__var_vs_drop}
\title{Variance vs Dropout Rate}
\description{
	Fixes the mean expression level and simulates a single gene with different fold changes across two equally sized subpopulations. 
}
\usage{
	bg__var_vs_drop(pop_size, fixed_mean, K=10.3, dispersion_from_mean=bg__default_mean2disp, suppress.plot=TRUE)
}
\arguments{
  \item{pop_size}{size of each subpopulation, i.e. 1/2 the total number of cells.}
  \item{fixed_mean}{expected mean expression across all cells.}
  \item{K}{K of the Michaelis-Menten function.}
  \item{dispersion_from_mean}{function which calculates the dispersion parameter of the negative binomial distribution given the mean expression of the gene.}
  \item{suppress.plot}{Whether to plot variance vs fold change and dropout rate vs fold change plots.}
}
\details{
	Simulates a single gene that is differentially expressed across two equally sized cell populations (\code{pop_size}) with a constant expected mean expression across all cells. Simulations are performed for fold changes of every integer value from 1 (no DE) to 100. Simulations use a negative binomial distribution inflated with zeros according to a Michaelis-Menten function. 

	Default mean2disp relationship and K were fit to the Buettner et al. [1] mESC data.

	Calculates pearson correlations between observed sample variance and dropout rate and fold change of the differential expression. Sample variance is decomposed into between subpopulation and within subpopulation variance ANOVA. Optionally plots the relationship between observed sample variance and dropout rate and the fold change. 
}
\value{
Name list of values:
	var_r = pearson correlation between observed sample variance and fold change.
	drop_r = pearson correlation between observed dropout rate and fold change.
	vars = vector of observed sample variances
	drops = vector of observed dropout rates
	fc = vector of expected fold changes
	Vbtw = vector of between subpopulation variances
	Vwithin = vector of within subpopulation variances
}
\examples{
#	mu100_N200 <- bg__var_vs_drop(100,100, suppress.plot=FALSE)
#	mu100_N2000 <- bg__var_vs_drop(1000,100)
#	mu1000_N200 <- bg__var_vs_drop(1000,100)
#	c(mu100_N200$var_r,mu100_N2000$var_r,mu1000_N200$var_r)
}
\keyword{simulations, variance, dropouts, differential expression}
\references{
[1] Buettner et al. (2015) Computational analysis of cell-to-cell heterogeneity in single-cell RNA-sequencing data reveals hidden subpopulations of cells. Nature Biotechnology 33 : 155-160.
}

\name{bg__nbumiGroupDE}
\alias{bg__nbumiGroupDE}
\title{Perform Traditional Differential Expression}
\description{
	Performs traditional (i.e. compare defined groups) differential expression using a negative binomial model with MM zero-inflation. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
}
\arguments{
  \item{counts}{a numeric matrix of raw UMI counts, columns = samples, rows = genes.}
  \item{fit}{fit NB UMI model from NBumiFitModel}
  \item{groups}{a vector of biological group IDs for each cell(columns).}
}
\details{
THIS FUNCTION SHOULD NOT BE USED.

\code{unfinished__nbumiGroupDE} Uses a log-likelihood ratio test to perform model selection between a model of constant mean expression vs a model of different mean expression across the biological groups. Probabilities of observing the data given the model are calculated using a negative binomial distribution with means adjusted for the total molecules detected per cell and dispersion fit to observed variance and adjusted to the mean of each group based on a globally fit power-law relationship. Significance is evaluated using the chi-square distribution.

}
\value{
	A table of fold mean expression differences for each biological group relative to the global mean expression level with raw p-values and FDR corrected p-values for each gene.
}
\examples{
#library(M3DExampleData)
#counts <- as.matrix(Mmus_example_list$data);
#counts <- counts[rowSums(counts) > 0,];
#counts <- counts[1:1000,]
#fit <- NBumiFitModel(counts);
#DE_output <- bg__nbumiGroupDE(counts, fit, Mmus_example_list$labels)
}
\keyword{differential expression}

\name{M3DropPlottingFunctions}
\alias{bg__dropout_plot_base}
\alias{bg__add_model_to_plot}
\alias{bg__highlight_genes}
\title{Make M3Drop Plots}
\description{
	Background functions used for making M3Drop plots. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
	bg__dropout_plot_base(expr_mat, xlim=NA, suppress.plot=FALSE)
	bg__add_model_to_plot(fitted_model, base_plot, lty=1, lwd=1, col="dodgerblue", legend_loc="topright")
	bg__highlight_genes(base_plot, expr_mat, genes, col="darkorange", pch=16)
}
\arguments{
  \item{expr_mat}{a numeric matrix of normalized (not log-transformed) expression values, columns = samples, rows = genes.}
  \item{xlim}{limits of x-axis of plot, for comparing distributions across many datasets.}
  \item{suppress.plot}{Whether to make the plot or just return values.}
  \item{fitted_model}{output from fitting a model to dropout rate vs mean expression, see \code{bg__fit_MM},\code{bg__fit_ZIFA},\code{bg__fit_logistic}}
  \item{base_plot}{output from \code{bg__dropout_plot_base}}
  \item{legend_loc}{coordinates of top-right corner of the legend.}
  \item{genes}{list of genes to be highlighted.}
  \item{col,lty,lwd,pch}{Graphical parameters passed to plotted points & lines respectively.}
}
\details{ 
\code{bg__dropout_plot_base} plots grey-scale distribution of dropout-rate vs log10(mean expression) for each each. Colour indicates density of points light-grey = low, black = high.

\code{bg__add_model_to_plot} adds a line based on the predicted dropout rate for each gene from a particular model.

\code{bg__highlight_genes} highlights specified genes in the given colour.

}
\value{
\code{bg__dropout_plot_base} A named list of output:
	p = dropout rate for each gene
	s = mean expression of each gene
	xes = log10 transformed mean expression of each gene
	data = original expression matrix
	order = ordering of xes from smallest to largest
\code{bg__add_model_to_plot} Invisibly location of the plotted legend.
\code{bg__highlight_genes} Invisibly the coordinates of the highlighted genes.
}
\keyword{plotting, background}

\name{PoissonUMIFeatureSelectionDropouts}
\alias{PoissonUMIFeatureSelectionDropouts}
\title{Dropout-based Feature Selection}
\description{
	Ranks genes by significance of increase in dropouts compared to expectation.
}
\usage{
	PoissonUMIFeatureSelectionDropouts(fit)
	
}
\arguments{
  \item{fit}{output from NBumiFitModel or NBumiFitBasicModel.}
}
\details{
	Calculates dropout probability for each observation using depth-adjusted negative binomial means and dispersions equal to the mean (Poisson). Total dropouts per gene are modelled using the normal 
	approximation of the sum of bernoulli variables. And significance is evaluated using a Z-test.
}
\value{
	Sorted vector of p-values
}
\examples{
library(M3DExampleData)
counts <- as.matrix(Mmus_example_list$data);
counts <- counts[rowSums(counts) > 0,];
fit <- NBumiFitModel(counts);
Dropout_features <- names(PoissonUMIFeatureSelectionDropouts(fit)[1:2000]);
}
\keyword{single cell}
\keyword{feature selection}
\keyword{depth-adjusted negative binomial}

\name{NBumiFeatureSelectionOther}
\alias{obsolete__nbumiFeatureSelectionDropouts}
\alias{obsolete__nbumiFeatureSelectionHighVarDist2Med}
\title{Other Feature Selection Methods}
\description{
	Ineffective alternative feature selection methods based on the depth-adjusted negative binomial model. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
	
}
\arguments{
  \item{fit}{output from NBumiFitModel or NBumiFitBasicModel.}
  \item{window_size}{window for calculating the moving median.}
}
\details{

	Calculates dropout probability for each observation using depth-adjusted negative binomial means and dispersions calculated 
	from a fitted power-law relationship between mean and dispersion. Total dropouts per gene are modelled using the normal 
	approximation of the sum of bernoulli variables. And significance is evaluated using a Z-test.

	\code{obsolete__nbumiFeatureSelectionDropouts} Ranks genes by significance of increase in dropouts compared to expectation allowing for gene-specific dispersions.
	\code{obsolete__nbumiFeatureSelectionHighVarDist2Med} Ranks genes by the distance to median of log-transformed estimated dispersions.
}
\value{
	Sorted vector of p-values/distances.
}
\examples{
library(M3DExampleData)
#counts <- as.matrix(Mmus_example_list$data);
#counts <- counts[rowSums(counts) > 0,];
#fit <- NBumiFitModel(counts);
#Dropout_features <- names(obsolete__nbumiFeatureSelectionDropouts(fit)[1:2000]);
#dist2med_features <- names(obsolete__nbumiFeatureSelectionHighVarDist2Med(fit)[1:2000]);
}
\keyword{single cell}
\keyword{feature selection}
\keyword{depth-adjusted negative binomial}

\name{bg__shift_size}
\alias{bg__shift_size}
\title{Shift Size Parameter}
\description{
	Shifts a fitted size paramter according to the a power-law relationship.
}
\usage{
	bg__shift_size(mu_all, size_all, mu_group, coeffs) 
}
\arguments{
	\item{mu_all}{vector of original mus}
	\item{size_all}{vector of original sizes}
	\item{mu_group}{vector of new mus}
	\item{coeffs}{coefficients of the power-law fit}
}
\details{
	Wrapper of short function which converts values to their logs then shifts them using the parameters of a linear fit before un-logging them.
}
\value{vector of new size parameters}
\examples{
#require("M3DExampleData")
#counts <- NBumiConvertData(Mmus_example_list$data, is.counts=TRUE);
#fit <- NBumiFitModel(counts);
#ceoffs <- NBumiFitDispVsMean(fit, suppress.plot=TRUE)
#new_size <- bg__shift_size(rowMeans(counts), fit$size, rowMeans(counts)*2, coeffs)
}
\keyword{simulations}

\name{bg__horizontal_residuals_MM_log10}
\alias{bg__horizontal_residuals_MM_log10}
\title{Calculate Horizontal Residuals}
\description{
	Calculates horizontal residuals from the Michaelis-Menten Function. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
	bg__horizontal_residuals_MM_log10(K, p, s)
}
\arguments{
  \item{K}{fitted Michaelis-Menten constant.}
  \item{p}{Observed dropout rate.}
  \item{s}{Observed mean expression.}
}
\details{ Calculates the log-transformed horizontal residuals from the Michaelis-Menten function. Input values may be single values or vectors.

\deqn{r = log_{10}(S) - \frac{(1-P)K}{P}}{log10(S) - K*(1-P)/P}
	
}
\value{ Value of horizontal residual.}
\examples{
#  s <- c(10,100,100)
#  p <- c(0.9, 0.5, 0.1)
#  res <- bg__horizontal_residuals_MM_log10(10, p, s)
}
\keyword{background}

\name{M3DropDropoutModels}
\alias{M3DropDropoutModels}
\title{Fit functions to the dropouts vs expression distribution.}
\description{
	Fits the modified Michaelis-Menten equation (MM), a logistic regession (logistic), or a double exponential (ZIFA) function to the relationship between mean expression and dropout-rate (proportion of zero values).
}
\usage{M3DropDropoutModels(expr_mat, xlim=NA, suppress.plot=FALSE)}
\arguments{
  \item{expr_mat}{a numeric matrix of normalized (not log-transformed) expression values, columns = samples, rows = genes.}
  \item{xlim}{limits for x-axis of plot.}
  \item{suppress.plot}{logical, whether to plot fit curves or not.}
}
\details{Plots the dropout-rate (P) vs average gene expression (S) for all genes. Fits three different models and adds the fitted curves to the plot. The three models are: 
	MMfit : the Michaelis-Menten function \deqn{P=1-\frac{S}{S+K}}{P = 1 - S/(K+S)} (see: [1]).
	LogiFit : a logistic regression between P and log base 10 of S (used by [2]).
	ExpoFit : a double exponential \deqn{P = e^{\lambda S^2}}{P = e^(-lambda*S^2)} (used by [3]).
}
\value{Invisibly, a list of output from each fit (MMfit, LogiFit, ExpoFit).}
\examples{
  library(M3DExampleData)
  norm <- M3DropConvertData(Mmus_example_list$data, is.counts=TRUE)
  M3DropDropoutModels(norm)
} 

\references{
  [1] Keener, J.; Sneyd, J. (2008). Mathematical Physiology: I: Cellular Physiology (2 ed.). Springer. ISBN 978-0-387-75846-6 
  [2] Kharchenko, PV; Silberstein, L; Scadden, DT. (2014) Bayesian approach to single-cell differential expression analysis. Nature Methods. 11:740-742 
  [3] Pierson, E; Yau, C. (2015) ZIFA: Dimensionality reduction for zero-inflated single-cell gene expression analysis. Genome Biology. 16:241 doi:10.1186/s13059-015-0805-z}
\keyword{Michaelis Menten, model fitting}

\name{NBumiFitDispVsMean}
\alias{NBumiFitDispVsMean}
\title{Fit function between mean and dispersion}
\description{
	Fits a power-law relationship between mean expression and dispersion.
}
\usage{
	NBumiFitDispVsMean(fit, suppress.plot=TRUE)
}
\arguments{
  \item{fit}{output from NBumiFitModel or NBumiFitBasicModel.}
  \item{suppress.plot}{Whether to plot the calculated fit.}
}
\details{
	Fits a power-law relationship between mean expression and fitted gene-specific dispersions, using linear regression on ln transformed values. 
	Lowly expressed genes, mean expression < 2^4, are excluded as long as at least 2000 genes remain for fitting.
}
\value{
	Coefficients of linear regression.
}
\examples{
	library(M3DExampleData)
	counts <- as.matrix(Mmus_example_list$data);
	counts <- counts[rowSums(counts) > 0,];
	fit <- NBumiFitModel(counts);
	coeffs <- NBumiFitDispVsMean(fit, suppress.plot=TRUE);
}

\name{NBumiHVG}
\alias{NBumiHVG}
\title{Variance-based Feature Selection Accounting for Library Size and Sample Variance}
\description{
	Tests for significantly high variability in droplet-based datasets.
}
\usage{
	NBumiHVG(counts, fit, fdr_thresh=0.05, suppress.plot=FALSE, method=c("DANB", "basic"))
}
\arguments{
  \item{counts}{raw count matrix (e.g. from NBumiConvertData).}
  \item{fit}{output from NBumiFitModel or NBumiFitBasicModel.}
  \item{fdr_thresh}{multiple testing correction threshold to apply to filter output.}
  \item{suppress.plot}{whether to plot mean vs variance & selected features.}
  \item{method}{whether to use DANB dispersions or raw sample variances.}
}
\details{
	Assumes a constant dispersion parameter due to technical noise (see: [1]), which is
	estimated using linear regression. Gene-specific observed sample variances are 
	compared to their respective expected variances. Significance is evaluated using
	a Z-test with the expected variance of the sample variance for a Negative Binomial
	(see: [2]). 

	The method argument controls whether the expected and observed variances are adjusted
	to account for uneven library sizes between cells. The default "DANB" option does
	the adjustment.
}
\value{
	a data.frame with columns: Gene, effect.size, p.value, q.value
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	fit <- NBumiFitModel(counts);
	HVGs <- NBumiHVG(counts, fit, suppress.plot=TRUE);
	HVGs_uncorrected <- NBumiHVG(counts, fit, suppress.plot=TRUE, method="basic");
}
\keyword{single cell}
\keyword{feature selection}
\keyword{depth-adjusted negative binomial}
\references{
	[1] Svensson, V. (2019) Droplet scRNA-seq is not zero-inflated. bioRxiv. doi: https://doi.org/10.1101/582064
	[2] Rose, C. and Smith, M. D. Mathematical Statistics with Mathematica. New York: Springer-Verlag, 2002. p264
}

\name{M3DropGetExtremes}
\alias{M3DropGetExtremes}
\alias{bg__get_extreme_residuals}
\title{Get outliers from MM curve.}
\description{
	Identifies outliers left and right of a fitted Michaelis-Menten curve. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
  M3DropGetExtremes(expr_mat, fdr_threshold=0.1, percent=NA, v_threshold=c(0.05,0.95), suppress.plot=FALSE)
  bg__get_extreme_residuals(expr_mat, fit=NA, fdr_threshold=0.1, percent=NA, v_threshold=c(0.05, 0.95), direction="right", suppress.plot=FALSE)
}
\arguments{
  \item{expr_mat}{a numeric matrix of normalized (not log-transformed) expression values, columns = samples, rows = genes.}
  \item{fit}{output from fitting the Michaelis-Menten equation, see \code{bg__fit_MM}}
  \item{fdr_threshold}{the threshold for identifying significant outliers after multiple testing correction.}
  \item{percent}{identify this percentage of data that is most extreme in each direction.}
  \item{v_threshold}{restrict to this range of dropout rates to avoid poorly fit regions of the data.}
  \item{direction}{"left" or "right", which tail of outliers to examine.}
  \item{suppress.plot}{logical, whether to plot the fitted Michaelis-Menten curve and highlight to identified most extreme outliers.}
}
\details{Fits a Michaelis-Menten function to the dropout-rate of the provided data, then identifies the most extreme left and/or right outliers from the curve. Horizontal residuals are calculated as : \deqn{\log_{10} S - \log_{10} \frac{K*(1-P)}{P}}{log_10(S) - log_10( (K * (1 - P)) / P )}. Extreme left[right] outliers are identified either as the \code{percent} smallest[largest] horizontal residuals. If \code{percent} is undefined (default) a normal distribution is fitted to the horizontal residuals and a Z-test is used to identify significant outliers after FDR multiple testing correction. 

Only genes with dropout rates within v_threshold will be considered to avoid the skewing of residuals due to the exponential parts of the MM curve near P = 0 & P = 1.

\code{M3DropGetExtremes} identifies both left and right residuals using the provided thresholds in each direction. Eg. will return the \code{percent} smallest and \code{percent} largest residuals. It also plots the fitted MM curve and highlights the left and right extreme outliers unless suppress.plot=TRUE .
\code{bg__get_extreme_residuals} identifies one-sided extreme outliers.
}
\value{
  \code{M3DropGetExtremes} List containing elements left and right, vectors of the names of the extreme genes to the left and right of the curve respectively.
  \code{bg__get_extreme_residuals} A vector of names of the extreme genes in the specified direction.
}
\examples{
  library(M3DExampleData)
  norm <- M3DropConvertData(Mmus_example_list$data, is.counts=TRUE)
  extreme_gene_lists <- M3DropGetExtremes(norm, fdr_threshold=0.1)
  extreme_gene_lists <- M3DropGetExtremes(norm, percent=0.01)
#  Lextremes <- bg__get_extreme_residuals(norm, fdr_threshold=0.1, direction="left")
}
\keyword{extremes, outliers, residuals}

\name{NBumiPearsonResiduals}
\alias{NBumiPearsonResiduals}
\alias{NBumiPearsonResidualsApprox}
\title{Calculate Pearson Residuals}
\description{
	Uses the NBumi depth-adjusted negative binomial model to calculate Pearson Residuals as an approach to normalizing 10X/UMI tagged data.
}
\usage{
	NBumiPearsonResiduals(counts, fits=NULL)
	NBumiPearsonResidualsApprox(counts, fits=NULL)
}
\arguments{
  \item{counts}{a numeric matrix of raw UMI counts, columns = samples/cells, rows = genes.}
  \item{fits}{the output from NBumiFitModel.}
}
\details{
Calculates a unique expectation for each gene expression observation based on the depth-adjusted negative binomial model (see: NBumiFitModel for details). This expection (mu) is equal to t_i*t_j/T, where t_i is the total counts for sample i, t_j is the total counts for gene j and T is the total counts.

\code{NBumiPearsonResidualApprox} Pearson residuals are approximated as: (counts - expectation) / sqrt(expectation). This assumes a Poisson-distribution of counts.

\code{NBumiPearsonResiduals} Pearson residuals are approximated as: (counts - mu) / sqrt(mu + mu^2/size). This uses a negative-binomial distribution of counts.

}
\value{
	a matrix of pearson residuals of equal dimension as your original matrix.
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	fit <- NBumiFitModel(counts);
	pearson1 <- NBumiPearsonResiduals(counts,fit)
	pearson2 <- NBumiPearsonResidualsApprox(counts,fit)
}
\keyword{single cell}
\seealso{NBumiFitModel}

\name{M3DropGetHeatmapClusters}
\alias{M3DropGetHeatmapClusters}
\alias{M3DropGetHeatmapNames}
\alias{M3DropGetHeatmapClusters}
\title{Extracts clusters/ordered names from heatmap output}
\description{Extracts the groupings correponding to the given number of clusters from heatmap output.}
\usage{M3DropGetHeatmapClusters(heatout, k, type="cell")
	M3DropGetHeatmapNames(heatout, type="cell")}
\arguments{
  \item{heatout}{Output from a gene-expression heatmap.}
  \item{k}{Number of clusters.}
  \item{type}{One of "cell" or "gene" indicating whether to get clusters or names of cells(columns) or genes(rows).}
}
\details{\code{M3DropGetHeatmapClusters} : Traverses down the row or column dendrogram and cuts at the first point where there are at least k clusters.
	 \code{M3DropGetHeatmapNames}: gets the names of the cells/genes in the order that they appear on the dendrogram.
}
\value{A vector of cluster labels for each cell.}
\examples{
  library(M3DExampleData)
  genes <- rownames(Mmus_example_list$data)[1:20]
  heatmap_out <- M3DropExpressionHeatmap(genes, Mmus_example_list$data)
  clusters <- M3DropGetHeatmapClusters(heatmap_out, k=5)
  heatmap_gene_labels <- M3DropGetHeatmapNames(heatmap_out, type="gene")
}
\keyword{heatmap, cluster}

\name{M3DropExpressionHeatmap}
\alias{M3DropExpressionHeatmap}
\alias{bg__expression_heatmap}
\title{Plot Heatmap of Gene Expression}
\description{
	Plots a customized heatmap of scaled log expression values. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
  M3DropExpressionHeatmap(genes, expr_mat, cell_labels=NA, interesting_genes=NA, key_genes=genes, key_cells=NA)
  bg__expression_heatmap(genes, expr_mat, cell_labels=NA, gene_labels=NA, key_genes=genes, key_cells=NA)
}
\arguments{
  \item{genes}{a character vector of gene names to be plot.}
  \item{expr_mat}{a numeric matrix of normalized (not log-transformed) expression values, columns = samples, rows = genes.}
  \item{cell_labels}{factor of labels for each cell in the expression matrix that will be used to colour the bar on the top of the heatmap.}
  \item{interesting_genes}{list of vectors of gene names that will be used to colour the bar to the left of the heatmap.}
  \item{key_genes}{a character vector of gene names to be labelled on the heatmap.}
  \item{key_cells}{a character vector of cells to be labelled on the heatmap. Unlabelled cells will be assigned a numerical index}
  \item{gene_labels}{factor of labels for each gene that will be used to colour the bar on the left of the heatmap.}
}
\details{Modifies the gplots function heatmap.2 to replace the row dendrogram with a legend of the colours used in the columns colour bar (cell_labels) and use a custom colour scalling. Expression is displayed as Z-scores of log transformed expression (adding a pseudocount of 1) coloured blue-white-red centered at 0 and binned in the range [-2,2].

\code{M3DropExpressionHeatmap} is a wrapper for \code{bg__expression_heatmap} that checks and reformats provided arguments. 
}
\value{Invisibly, output from heatmap.2 call.}
\examples{
  library(M3DExampleData)
  M3DropExpressionHeatmap(head(rownames(Mmus_example_list$data),20),Mmus_example_list$data, cell_labels = Mmus_example_list$labels)
#  bg__expression_heatmap(head(rownames(Mmus_example_list$data),20),Mmus_example_list$data, cell_labels = Mmus_example_list$labels)
}
\keyword{heatmap}

\name{Fitting_Dropout_Models}
\alias{bg__fit_MM}
\alias{bg__fit_ZIFA}
\alias{bg__fit_logistic}
\title{Fit functions to the dropouts vs expression distribution.}
\description{
	Fits the modified Michaelis-Menten equation (MM), a logistic regession (logistic), or a double exponential (ZIFA) function to the relationship between mean expression and dropout-rate (proportion of zero values).
}
\usage{
  bg__fit_MM(p, s)
  bg__fit_logistic(p, s)
  bg__fit_ZIFA(p, s)
}
\arguments{
  \item{p}{a vector of dropout rates for each gene.}
  \item{s}{a vector of mean expression values for each gene. Must be the same order & length as p.}
}
\details{Fits one of different models to the relationship between dropout rate and mean expression. The three models are: 
		\code{bg__fit_MM} : the Michaelis-Menten function \deqn{P=1-\frac{S}{S+K}}{P = 1 - S/(K+S)} (see: [1]). Fit using \code{mle2} using normally distributed error.
		\code{bg__fit_logistic} : a logistic regression between P and log base 10 of S (used by [2]). Fit using \code{glm} (excludes genes where S == 0).
		\code{bg__fit_ZIFA} : a double exponential \deqn{P = e^{\lambda S^2}}{P = e^(-lambda*S^2)} (used by [3]). Fit using \code{lm} after log-transformation (genes were P == 0 are assigned a value of one tenth of the smallest P which is not 0).
}
\value{Named list including:
	K,fitted_err/B0,B1/lambda,fitted_err : the fitted parameters
	predictions : predicted values of p for each gene
	SSr/SAr :  sum of squared/absolute residuals
	model : vector of string descriptors of the fit
}
\examples{
#  library(M3DExampleData)
#  gene_info = bg__calc_variables(Mmus_example_list$data)
#  MM_fit = bg__fit_MM(gene_info$p, gene_info$s)
#  logistic_fit = bg__fit_logistic(gene_info$p, gene_info$s)
#  ZIFA_fit = bg__fit_ZIFA(gene_info$p, gene_info$s) 
}
\references{
  [1] Keener, J.; Sneyd, J. (2008). Mathematical Physiology: I: Cellular Physiology (2 ed.). Springer. ISBN 978-0-387-75846-6 
  [2] Kharchenko, PV; Silberstein, L; Scadden, DT. (2014) Bayesian approach to single-cell differential expression analysis. Nature Methods. 11:740-742 
  [3] Pierson, E; Yau, C. (2015) ZIFA: Dimensionality reduction for zero-inflated single-cell gene expression analysis. Genome Biology. 16:241 doi:10.1186/s13059-015-0805-z}
\keyword{Michaelis Menten, model fitting}

\name{bg__calc_variables}
\alias{bg__calc_variables}
\title{Calculate Gene Variables}
\description{
	Calculates a suite of gene-specific variables including: mean, dropout rate, and their standard errors.
}
\usage{
	bg__calc_variables(expr_mat)
}
\arguments{
  \item{expr_mat}{a numeric matrix of normalized (not log-transformed) expression values, columns = samples, rows = genes.}
}
\details{ Calculates mean expression and its standard error ( sd/sqrt(n) ), and the dropout rate, proportion of observations that are zero, and its standard error ( sqrt(p*(1-p)/n) ) for each gene in the dataset. Performs various checks to ensure expression matrix is of suitable format for M3Drop. Removes undetected genes if they are present.
}
\value{
	Named list of calculated values:
	s = vector of mean expression for each gene
	s_stderr = vector of mean expression standard error for each gene
	p = vector of dropout rate for each gene
	p_stderr = vector of dropout rate standard error for each gene
}
\keyword{background}

\name{M3DropConvertData}
\alias{M3DropConvertData}
\title{Convert Data to be suitable for M3Drop}
\description{
	Recognizes a variety of R objects/classes and extracts expression 
	matrices from them then converts that to a normalized but 
	not-log transformed matrix appropriate for input into M3Drop functions.
}
\usage{
  M3DropConvertData(input, is.log=FALSE, is.counts=FALSE, pseudocount=1)
}
\arguments{
  \item{input}{ a matrix, data.frame or object }
  \item{is.log}{ has the data been log-transformed? (assumes log-base 2 with pseudocount of 1) }
  \item{is.counts}{ is the data raw unnormalized counts? (raw counts will be CPM normalized) }
  \item{pseudocount}{ pseudocount added before log-transformation }
}
\details{
	You must have loaded the respective packages (in parentheses) into your namespace before running this function on the respective objects. Note that to maintain scalability sparse matrices will remain as such.

	Supported classes/objects:
		\describe{

		\item{SCESet (scater <= 1.4.0)}{uses "exprs" or if unavailable then "counts"}

		\item{SingleCellExperiment (scater >= 1.6.0)}{uses "normcounts" if available, then "logcounts", which is assumed to be log-normalized, then "counts"}

		\item{ExpressionSet (Biobase)}{uses "exprs", specify log/counts using arguments}

		\item{seurat (Seurat)}{uses "raw.data" as counts.}
		}

		Matrix/Dataframe classes :

		\describe{
		\item{dgCMatrix (Matrix)}{specify log/counts using arguments}

		\item{data.table (data.table)}{specify log/counts using arguments}

		\item{DataTable (S4Vectors)}{specify log/counts using arguments}

		\item{DataFrame (S4Vectors)}{specify log/counts using arguments}

		\item{AnnotatedDataFrame (Biobase)}{specify log/counts using arguments}

		\item{matrix (base-r)}{specify log/counts using arguments}

		\item{data.frame (base-r)}{specify log/counts using arguments}
		}

}
\value{A normalized but not log-transformed matrix appropriate for input into M3Drop functions.}
\examples{
	# Simulated raw count matrix:
	set.seed(42)
	counts <- matrix(rpois(200, lambda=3), ncol=10)
	expr_mat <- M3DropConvertData(counts, is.counts=TRUE)

	# log normalized data frame
	lognorm <-log2( t(t(counts)/colSums(counts)*100)+1 )
	lognorm <- as.data.frame(lognorm)
	expr_mat <- M3DropConvertData(lognorm)

	# Sparse matrix
	require("Matrix")
	counts <- Matrix(counts, sparse=TRUE)
	expr_mat <- M3DropConvertData(counts, is.counts=TRUE)

	# SingleCellExperiment Object
	require("SingleCellExperiment")
	SCE <- SingleCellExperiment(assays=list(counts=counts))
	expr_mat <- M3DropConvertData(SCE)
}
\keyword{normalization}

\name{BrenneckeGetVariableGenes}
\alias{BrenneckeGetVariableGenes}
\title{Identify Highly Variable Genes}
\description{
	Implements the method of Brennecke et al. (2013) to identify highly variable genes.
}
\usage{BrenneckeGetVariableGenes(expr_mat, spikes=NA, suppress.plot=FALSE, fdr=0.1, minBiolDisp=0.5, fitMeanQuantile=0.8)}
\arguments{
  \item{expr_mat}{a numeric matrix of normalized or raw (not log-transformed) expression values, columns = samples, rows = genes.}
  \item{spikes}{a vector of gene names of row numbers of spike-in genes which are subject to only technical variance.}
  \item{suppress.plot}{Whether to make the plot or just calculate the requisite values.}
  \item{fdr}{Use FDR to identify significantly highly variable genes.}
  \item{minBiolDisp}{Minimum percentage of variance due to biological factors.}
  \item{fitMeanQuantile}{Threshold for genes to be used in fitting. May need to be decreased in low-depth/umi-tagged datasets to achieve good fit.}
}
\details{Identifies significantly highly variable genes as detailed in Brennecked et al [1]. If spike-ins are provided they are used fit a function to the relationship between gene expression and variance due to technical factors. If spike-ins are not provided then all genes are used in the fitting. }
\value{Vector of names of highly variable genes.}
\references{Brennecke et al. (2013) Accounting for technical noise in single-cell RNA-seq experiments. Nature Methods 10, 1093-1095. doi:10.1038/nmeth.2645}
\examples{
  library(M3DExampleData)
  HVG <- BrenneckeGetVariableGenes(Mmus_example_list$data)
  HVG_spike <- BrenneckeGetVariableGenes(Mmus_example_list$data, spikes=5550:5600)
}
\keyword{highly variable genes}

\name{NBumiCheckFit}
\alias{NBumiCheckFit}
\alias{NBumiCheckFitFS}
\title{Check Fit Quality}
\description{
	Checks the quality of the fit of the depth-adjusted negative binomial model.
}
\usage{
	NBumiCheckFit(counts, fit, suppress.plot=FALSE)
	NBumiCheckFitFS(counts, fit, suppress.plot=FALSE)
}
\arguments{
  \item{counts}{a numeric matrix of raw UMI counts, columns = samples, rows = genes.}
  \item{fit}{output from NBumiFitModel or NBumiFitBasicModel.}
  \item{suppress.plot}{Whether to plot the observed vs expected frequency of dropouts for genes & cells.}
}
\details{
	Calculates expected dropouts for genes and cells and compares to observed values. Optionally plots observed vs expected dropouts for both genes and cells.
	\code{NBumiCheckFit} uses depth-adjusted negative binomial with gene-specific dispersions.
	\code{NBumiCheckFitFS} uses depth-adjusted negative binomial with dispersions calculated from the power-law between gene-specific dispersion and mean expression (as is used for feature selection).
}
\value{
	Invisibly, named list of output: 
		gene_error = sum of squared error between observed and expected gene-specific total dropouts
		cell_error = sum of squared error between observed and expected cell-specific total dropouts
		exp_ps = gene by sample matrix of probability of a dropout for the negative binomial model fitted to each observation.
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	fit <- NBumiFitModel(counts);
	fit_quality <- NBumiCheckFitFS(counts, fit);
}

\name{NBumiCompareModels}
\alias{NBumiCompareModels}
\title{Compare negative binomial models}
\description{
	Compares the fit of the depth-adjusted negative binomial model and basic negative binomial model.
}
\usage{
	NBumiCompareModels(counts, size_factor=(Matrix::colSums(counts)/median(Matrix::colSums(counts))))
}
\arguments{
  \item{counts}{a numeric matrix of raw UMI counts, columns = samples, rows = genes.}
  \item{size_factor}{a calculated size factor for library size normalization.}
}
\details{
	Compares the fit of the depth-adjusted negative binomial model and basic negative binomial model. Depth
	-adjusted negative binomial is fit to raw molecule counts. Basic negative binomial is fit to library-size 
	normalized counts. The absolute error between observed gene-specific dropouts and expectations given each 
	model is calculated. And a plot of fitted and observed mean-expression vs dropouts is created.
}
\value{
	A named list containing: 
		errors : Vector of errors for each model.
		basic_fit : object for the basic negative binomial.
		adjusted_fit : object for the depth-adjusted negative binomial.
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	out <- NBumiCompareModels(counts);
	out$errors
}

\name{Imputation}
\title{Normalized Data using the DANB model}
\alias{NBumiImputeNorm}
\description{
	Normalizes data to a common library size, imputing zeros as needed.
}
\usage{
	NBumiImputeNorm(counts, fit, total_counts_per_cell=median(fit$vals$tis))
}
\arguments{
  \item{counts}{raw count matrix, rows=genes, cols=cells}
  \item{fit}{Output from \code{NBumiFitModel}}
  \item{total_counts_per_cell}{reference library size to normalize all cells to.}
}
\details{
	Converts raw counts into positions in the CDF for the depth-adjusted negative binomial model fit to each observation. Adjusts the DANB parameters (mean and size) for the new library size. Then calculate the normalized counts for the equivalent position in the CDF for the NB using the new parameters. 
}
\value{
	Normalized count matrix.
}

\examples{
library(M3DExampleData)
counts <- as.matrix(Mmus_example_list$data);
counts <- counts[rowSums(counts) > 0,];
fit <- NBumiFitModel(counts);
normed_counts <- NBumiImputeNorm(counts, fit, 1000000)
}
\keyword{single cell}
\keyword{normalization}

\name{M3DropTestShift}
\alias{M3DropTestShift}
\title{Test for horizontal shift.}
\description{
	Tests whether a given set of genes are significantly shifted to the left or right of the Michaelis-Menten curve.
}
\usage{
  M3DropTestShift(expr_mat, genes_to_test, name="", background=rownames(expr_mat), suppress.plot=FALSE)
}
\arguments{
  \item{expr_mat}{a numeric matrix of normalized (not log-transformed) expression values, columns = samples, rows = genes.}
  \item{genes_to_test}{vector of gene names to test.}
  \item{name}{string used to title the plot.}
  \item{background}{vector of gene names to test against. (default = all genes)}
  \item{suppress.plot}{logical, whether to the fitted Michaelis-Menten curve and highlight the given set of genes to test.}
}
\details{Fits a Michaelis-Menten function to the dropout-rate of the provided data, then tests whether a given set of genes (eg. pseudogenes) is significantly shifted left or right of the curve. Horizontal residuals are calculated as : \deqn{\log_{10} S - \log_{10} \frac{K*(1-P)}{P}}{log_10(S) - log_10( (K * (1 - P)) / P )}. Uses a Wilcox rank-sum test/Mann-Whitney U test to compare the residuals for the given genes to the residuals for all genes.
}
\value{
A one row dataframe with columns: 
    sample (median horizontal residual of genes in the test set), 
    pop (median horizontal residual of genes in the background set), 
    p.value
}
\examples{
  library(M3DExampleData)
  gene_set <- c("Dppa2","Tdgf1","Rnf130","Tet1","Uhrf1","Pttg1","Zfp600","Stat1")
  shift_output <- M3DropTestShift(Mmus_example_list$data, gene_set)
}
\keyword{residuals}

\name{M3DropFeatureSelection}
\alias{M3DropFeatureSelection}
\alias{bg__test_DE_K_equiv}
\title{Differentially Expressed Genes.}
\description{
	Use Michaelis-Menten curve to find differentially expressed (DE) genes. Functions tagged with "bg__" are not meant for direct usage and are not available in the Bioconductor release.
}
\usage{
  M3DropFeatureSelection(expr_mat, mt_method="bon", mt_threshold=0.05, suppress.plot=FALSE, xlim=NA)
  bg__test_DE_K_equiv(gene_info, fit=NA)
}
\arguments{
  \item{expr_mat}{a numeric matrix of normalized (not log-transformed) expression values, columns = samples, rows = genes.}
  \item{mt_method}{the multiple testing method used in p.adjust}
  \item{mt_threshold}{the threshold for identifying significantly DE genes.}
  \item{suppress.plot}{logical, whether to plot the fitted curve and highlight selected features.}
  \item{xlim}{specify the limits of the x-axis of the plot.}
  \item{fit}{Output from fitting the Michaelis-Menten equation (see: \code{bg__fit_MM}}
  \item{gene_info}{List of calculated gene-specific values output by \code{bg__calc_variables}}
}
\details{Fits a Michaelis-Menten function to the dropout-rate (if not provided) of the provided expression matrix. Identifies genes where the gene-specific K calculated as (S = mean expression, P = dropout rate): \deqn{K=\frac{S*P}{1-P}}{K = (S * P)/(1 - P)} is significantly larger than the K fitted to the entire dataset. Combines standard errors of the fitted K, the gene-specific dropout rate and the gene-specific average expression using error propagation rules. Determines the signficance of the gene-specific K using a Z-test of the log-transformed Ks with the propagated error then applies the specified multiple testing correction to identify DE genes. Plots the dropout rate vs gene expression with the fitted MM curve and highlights in purple the significantly DE genes.}
\value{ 
  \code{M3DropFeatureSelection} : a data.frame of significantly differentially expressed genes with columns: Gene, p.value, q.value
  \code{bg__test_DE_K_equiv} : a named list of containing: 
	pval : the significance of differential expression for each gene
	fold_change : ratio of gene-specific K to globally fit K
}
\examples{
	library(M3DExampleData)
	norm <- M3DropConvertData(Mmus_example_list$data, is.counts=TRUE)
	DE_genes <- M3DropFeatureSelection(norm, 
		mt_method="fdr", mt_threshold=0.01)
#  gene_info <- bg__calc_variables(Normalized_data$data[1:1000,])
#  DE_output <- bg__test_DE_K_equiv(gene_info)
}
\keyword{differential expression}

\name{bg__fit_gamma}
\alias{bg__fit_gamma}
\title{Fit Gamma Distribution}
\description{
	Fits the parameters of a gamma distribution to a set of observations
}
\usage{
	bg__fit_gamma(x)
}
\arguments{
  \item{x}{vector of observations to fit.}
}
\details{
	scale = var(x)/mean(x)
	shape = mean(x)/scale
}
\value{list with two entries: shape and size.}
\examples{
# dat <- rgamma(100, 0.1, 1)
# params <- bg__fit_gamma(dat)
}
\keyword{simulations}

\name{Simulation Trifecta}
\alias{M3DropSimulationTrifecta}
\alias{NBumiSimulationTrifecta}
\title{Make Simulated Data from a provided scRNASeq dataset.}
\description{
	Fits either a zero-inflated negative binomial model (M3Drop) or the depth-adjusted negative binomial model (NBumi) to a provided dataset then simulates data from that model including differentially expressed (DE), differentially variable (DV), or globally unusually variable (HV) genes. 
}
\usage{
	M3DropSimulationTrifecta(original_data, n_genes=25000, n_cells=250, sub_pop_prop=0.5)
	NBumiSimulationTrifecta(original_data, n_genes=25000, n_cells=250, sub_pop_prop=0.5)
}
\arguments{
  \item{original_data}{the expression matrix of a scRNASeq dataset to base the simulations on. Should be normalized (not log-transformed) values for M3Drop or raw counts for NBumi.}
  \item{n_genes}{number of genes to simulated for each expression matrix.}
  \item{n_cells}{number of cells to simulate for each condition (total columns of final matrices = 2*n_cells).}
  \item{sub_pop_prop}{proportion of cells in one of the sub-populations.}
}
\details{Generates simulated single-cell gene expression data based on an existing dataset. Three expression matrices are produced each with the same cell and gene-specific parameters but where the log2-fold change is applied to either the mean (DE), or variance (DV) in one half of the cells or is applied to the variance across all the cells (HV). 

Mean expression for each simulated gene is drawn from a log-normal distribution fit to the original dataset. These means are then bottom thresholded to ensure all genes have a mean expression >= 10^-5, and top thresholded to ensure no gene has a mean expression greater than the largest mean expression in the original dataset.

\code{M3DropSimulationTrifecta}

\code{NBumiSimulationTrifecta} : Cell-specific library sizes are drawn from a gamma distribution fit to the original data.

}
\value{
	a named list of output including:
		truth - the true log (base 2) fold changes in expression level or variability for each gene.
		groups - a vector specifying the group ID for each cell for the DE and DV genes (1 = control, 2 = different).
		de - the count matrix containing genes differentially expressed across the two groups.
		dv - the count matrix containing genes with differential variability across the two groups.
		hv - the count matrix containing genes with globally unusual variability.
		
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	norm <- M3DropConvertData(Mmus_example_list$data, is.log=FALSE, is.counts=TRUE)
	ZINB_sim <- M3DropSimulationTrifecta(norm)
	DANB_sim <- NBumiSimulationTrifecta(counts)
}
\keyword{simulations, differential expression, differential variance}

\name{NBumiCoexpression}
\alias{NBumiCoexpression}
\title{Variance-based Feature Selection}
\description{
	Ranks genes by residual dispersion from mean-dispersion power-law relationship.
}
\usage{
	NBumiCoexpression(counts, fit, gene_list=NULL, method=c("both", "on", "off"))
}
\arguments{
  \item{counts}{raw count matrix (e.g. from NBumiConvertData).}
  \item{fit}{output from NBumiFitModel or NBumiFitBasicModel.}
  \item{gene_list}{set of gene names to test coexpression of.}
  \item{method}{type of coexpression to test (see: Details).}
}
\details{
	Tests for co-expression using the normal approximation of a binomial test.
	Co-expression is defined according to the method argument as follows:
	\describe{
		\item{on}{two genes are both >0 in more cells than expected.}
		\item{off}{two genes are both 0 in more cells than expected.}
		\item{both}{two genes are either both >0 or both 0 in more cells than expected.}
	}
	In all cases the null expectation is calculated using the fit library-size adjusted negative binomial model.
	This remove spurious co-expression due to cells with differing numbers of detected genes.
}
\value{
	a matrix of Z-scores for each pair of genes in the provided gene list.
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	fit <- NBumiFitModel(counts);
	genes <- c("Sox2", "Eomes", "Zscan4d", "Obox1", "Obox3")
	co <- NBumiCoexpression(counts, fit, genes, method="both");
	on <- NBumiCoexpression(counts, fit, genes, method="on");
	off <- NBumiCoexpression(counts, fit, genes, method="off");
}
\keyword{single cell}
\keyword{feature selection}
\keyword{depth-adjusted negative binomial}

\name{NBumiFitModel}
\alias{NBumiFitModel}
\alias{NBumiFitBasicModel}
\title{Fit Depth-Adjusted Negative Binomial Model}
\description{
	Fits means and dispersions for depth-adjusted or basic negative binomial models to a read/UMI count matrix.
}
\usage{
	NBumiFitModel(counts)
	NBumiFitBasicModel(counts)
	
}
\arguments{
  \item{counts}{a numeric matrix of raw UMI counts, columns = samples, rows = genes.}
}
\details{

\code{NBumiFitModel} Fits a depth-adjusted negative binomial model for each expression observation. Each expression is modelled using a negative binomial distribution with mean equal to t_i*t_j/T, where t_i is the total counts for sample i, t_j is the total counts for gene j and T is the total counts. Dispersions (R size parameter) are fit such that: var_j(counts_ij-mu_ij) = sum(mu_ij+mu_ij^2/size_j). Cases where genes exhibit poissonian behavior (size->infinity) are assigned a size of 10^10. 
\code{NBumiFitBasicModel} Fits a basic negative binomial model for each expression observation. Each expression is modelled using a negative binomial distribution with mean equal to t_j/n, where t_j is the total counts for gene j and n is the number of cells. Dispersions (R size parameter) are fit such that: var_j(counts_ij) = sum(mu_j+mu_j^2/size_j). Cases where genes exhibit poissonian behavior (size->infinity) are assigned a size of 10^10. 

}
\value{
	A named list of: 
		mus = a genes by samples matrix of means for the observation-specific negative binomial
		sizes = a vector of dispersions for each gene.
		vals = named list of summary statistics of the original count matrix: 
			tis = total molecules per cell, 
			tjs = total molecules per gene, 
			dis = total dropouts per cell, 
			djs = total dropouts per genes, 
			total = total molecules in dataset, 
			nc = number of cells, 
			ng = number of genes
}
\examples{
	library(M3DExampleData)
	counts <- NBumiConvertData(Mmus_example_list$data)
	fit <- NBumiFitModel(counts);
}
\keyword{single cell}

\name{Consensus_FS}
\title{Consensus Feature Selection}
\alias{Consensus_FS}
\description{
	Performs seven different feature selection methods then calculates the consensus ranking of features from that.
}
\usage{
	Consensus_FS(counts, norm=NA, is.spike=rep(FALSE, times=nrow(counts)), pcs=c(2,3), include_cors=TRUE)
}
\arguments{
  \item{counts}{raw count matrix, rows=genes, cols=cells}
  \item{norm}{normalized but not log-transformed gene expression matrix, rows=genes, cols=cells}
  \item{is.spike}{logical, vector of whether each gene is/isn't a spike-in}
  \item{pcs}{which principle components to use to score genes}
  \item{include_cors}{logical, whether to perform gene-gene correlation feature selection which is much slower than all other methods.}
}
\details{

	Performs:
	\code{NBumiFeatureSelectionCombinedDrop} (aka: DANB_drop)
	\code{NBumiFeatureSelectionHighVar} (aka: DANB_var)
	\code{BrenneckeGetVariableGenes} (aka: HVG)
	\code{M3DropFeatureSelection} (aka: M3Drop)
	\code{giniFS}
	\code{irlbaPcaFS} (with provided PCs)
	\code{corFS} ("both" direction)

	Genes are ranked by each method and the consensus (Cons) is calculated as the average of those ranks.

	Automatically removes invariant genes. If only raw counts are provided then will apply counts per million normalization (scaled to the median library size) for those methods which require normalized data.
	
}
\value{
	Table of ranking of each gene by each method including the consensus (Cons). Columns are feature selection methods named using the shorter aliases (see: Details). 
}

\examples{
library(M3DExampleData)
norm <- as.matrix(Mmus_example_list$data[1:500,]);
norm <- norm[rowSums(norm) > 0,];
counts <- NBumiConvertToInteger(norm);
spikes <- sample(1:nrow(counts), 50);
spikes <- rownames(norm)[spikes];
spikes <- rownames(norm) %in% spikes;
Features_consensus <- Consensus_FS(counts, norm, is.spike=spikes);
}
\keyword{single cell}
\keyword{feature selection}

\name{bg__MakeSimData}
\alias{bg__MakeSimData}
\alias{bg__MakeSimDE}
\alias{bg__MakeSimDVar}
\alias{bg__MakeSimHVar}
\title{Make Simulated Data}
\description{
	Makes simulated data based on a negative binomial distribution inflated with zeros based on the Michaelis-Menten equation.
}
\usage{
	bg__MakeSimData(dispersion_fun=bg__default_mean2disp, n_cells=300, dispersion_factor=1, base_means=10^rnorm(25000, 1, 1), K=10.3)
	bg__MakeSimDE(dispersion_fun=bg__default_mean2disp, fold_change=10, frac_change=0.1, n_cells=300, sub_pop=0.5, dispersion_factor=1, base_means=10^rnorm(25000,1,1), K=10.3)
	bg__MakeSimDVar(dispersion_fun=bg__default_mean2disp, fold_change=10, frac_change=0.1, n_cells=300, sub_pop=0.5, dispersion_factor=1, base_means=10^rnorm(25000,1,1), K=10.3)
	bg__MakeSimHVar(dispersion_fun=bg__default_mean2disp, fold_change=10, frac_change=0.1, n_cells=300, dispersion_factor=1, base_means=10^rnorm(25000,1,1), K=10.3)
}
\arguments{
  \item{dispersion_fun}{a function which takes mean experssion and returns the dispersion parameter of the negative binomial distribution.}
  \item{n_cells}{total number of cells (columns) in the simulated dataset.}
  \item{sub_pop}{proportion of cells with changed expression.}
  \item{frac_change}{proportion of genes with changed expression.}
  \item{fold_change}{fold change in dispersion or mean expression.}
  \item{dispersion_factor}{a factor that multiplies the calculated mean-specific dispersion for all genes.}
  \item{base_means}{a vector of background mean expression values.}
  \item{K}{K of the Michaelis-Menten function}
}
\details{Generates simulated single-cell gene expression data using a zero-inflated negative binomial distribution. A user-supplied function relates the dispersion parameter (1/size of the R parameterization of the negative binomial distribution). Zeros are added based on a Michaelis-Menten function. 

Default values of base_means, K, and dispersion_fun were fit to the Buettner et al. 2015 data [1].


\code{bg__MakeSimData} generates simulated single-cell data for a single homogeneous population.

\code{bg__MakeSimDE} generates simulated single-cell data for two different populations where a proportion of genes have a \code{fold_change} difference in the mean for population "2".

\code{bg__MakeSimDVar} generates simulated single-cell data for two different populations where a proportion of genes have a \code{fold_change} difference in the dispersion for population "2".

\code{bg__MakeSimHVar} generates simulated single-cell data for a single homogeneous population where a proportion of genes have a \code{fold_change} increase in dispersion over the expectation given the mean expression of the gene.
}
\value{
	\code{bg__MakeSimData} : a gene expression matrix where rows are genes, columns are cells.
	\code{bg__MakeSimDE}, \code{bg__MakeSimDVar}, \code{bg__MakeSimHVar} : a list of three named items:
		data : the gene expression matrix where rows are genes, columns are cells
		cell_labels : a vector of 1 or 2 indicating which cells are the unchanged ("1") or changed ("2") population.
		TP : a vector of row IDs of those genes that change (true positives).
}
\examples{
#  means = c(1,2,5,10,20,50,100,200,500,1000,2000,5000)
#  population1 <- bg__MakeSimData(n_cells=10, base_means=means)
#  population2 <- bg__MakeSimData(n_cells=10, base_means=means*2, dispersion_factor=0.5)
#  sim_DE <- bg__MakeSimDE(n_cells=100, base_means=means)
#  sim_DVar <- bg__MakeSimDVar(n_cells=100, sub_pop=0.25, base_means=means)
#  sim_HVar <- bg__MakeSimHVar(base_means=means, fold_change=3)
}
\keyword{simulations, differential expression, differential variance}
\references{
[1] Buettner et al. (2015) Computational analysis of cell-to-cell heterogeneity in single-cell RNA-sequencing data reveals hidden subpopulations of cells. Nature Biotechnology 33 : 155-160.
}
